/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     InjectorControlImpl
// @module  InjectorControlImpl|implement InjectorControl
//
///////////////////////////////////////////////////////////////////////////////
#ifndef _InjectorControlImpl_H_
#define _InjectorControlImpl_H_

#include "InjectorControl.h"
#include "TestExecuter.h"
#include "TestManager.h"
#include "RemoteRepository.h"
#include <mutex.hpp>
#include <OSTAORB/OSTAORB.h>
#include <string>
#include <map>
#include <vector>
#include <SharedPointer.hpp>
#include "OstaTraceDef.h"

// Test Executer definition:
struct TEInfo
{
    TEInfo() : m_pTE(NULL), m_bTEStopping(false), m_pid(0)
#ifdef _WIN32
        ,m_hProcess(NULL)
#endif
    {}
    TEInfo(const TEInfo& tei) : m_pTE(tei.m_pTE), m_bTEStopping(tei.m_bTEStopping), m_pid(tei.m_pid)
#ifdef _WIN32
        ,m_hProcess(tei.m_hProcess)
#endif
    {}
    void operator=(const TEInfo& tei)
    {
        m_pTE         = tei.m_pTE;
        m_bTEStopping = tei.m_bTEStopping;
        m_pid         = tei.m_pid;
#ifdef _WIN32
        m_hProcess    = tei.m_hProcess;
#endif
    }

    TestExecuter_ptr m_pTE;         // Pointer to Test Executer CORBA object.
    std::string      m_strTarget;   // Test Executer's type.
    TargetType       m_tgtType;     // The Test Executer category.
    std::string      m_strRegName;  // Test Executer's registered name.
    bool             m_bTEStopping; // Set to true if the Test Executer process is
                                    // stopping.
    long             m_pid;         // The Test Executer's process id.
#ifdef _WIN32
    HANDLE           m_hProcess;    // The Test Executer's process kernel object handle.
#endif
};

class InjectorControlImpl:public virtual POA_InjectorControl,public PortableServer::RefCountServantBase
{
public:
	int GetTraceLevel();
    InjectorControlImpl(CORBAServer* corba);
    virtual ~InjectorControlImpl();

    CORBA::Boolean Activate(const char* strManagerRegString, const char* strReposHost,
                            const char* strTestRunString, const char* strTestName);
    CORBA::Boolean IsActivated(const char* strManagerRegString);

    CORBA::Boolean StopTest(CORBA::Boolean bForce, long lTimeout);

    // Starting / Stopping TaskGroups
    CORBA::Boolean StartTaskGroup(TaskGroupExecInfo& tgExec);
    CORBA::Boolean StopTaskGroup(long nTaskGroupID);
    TestExecuter_ptr GetTaskGroupExecuter(long nTaskGroupID);

    // Notifications from TE
    void OnTaskGroupStopped(long nTaskGroupID, TaskGroupStopReason& eReason);
    void OnExecuterStopping(const char* strExecuterRegName);
    long GetTestVUs();

    // Repository Access
    CORBA::Boolean GetTofFile(const char* strTofFile);
    CORBA::Boolean GetProfileFile(const char* strProfileFile);
    CORBA::Boolean GetDataFile(const char* strDataFile);
    CORBA::Boolean SetResultsFile(const char* strFilename);
    CORBA::Boolean StoreResultsFile(const char* strLocalResultsFile);
    void AddTestOutput(const char* strType, const char* strOutput);
	long TransferResults(const char* strTestName, const char* strReposHost,
		const char* strRunFolder);

    // Time Offset
    long GetTimeOffset() { return m_offset; };

    // All running Test Executers:
    CTMutex             m_TEMutex;
    typedef std::vector<SharedPtr<TEInfo> >  TestExecuters_t;
    TestExecuters_t     m_TEs;

private:
	int m_bTraceLevel;
    //// Helpers.
    void Reset();
    void WriteLogMsg(int severity,const string& msg);
    void TEComplete(const char* strExecuterRegName);
    static void* InjectorControlImpl_TECHECKER(void* args);

    //// Data.
    CORBAServer*        m_corba;
    typedef std::map<int,SharedPtr<TEInfo> >  TaskGroupToTE_t;
    TaskGroupToTE_t     m_TaskGroupToTE;   // Maps task groups to Test Executers.
    std::string         m_registered_name;
    CRemoteRepository*  m_pRepository;
    bool                m_bActivated;
    TestManager_ptr     m_pTestManager;
    CyrStTestRun_ptr    m_pTestRun;
    std::string         m_strTestName;
    std::string         m_strTestRunString;
    std::string         m_strManagerRegString;
    std::string         m_strReposHost;
    CTMutex             m_OpenResultsFilesListMutex;
    std::vector<string> m_OpenResultsFilesList;
    std::ofstream*      m_pLogFile;
    omni_thread*        m_pTEChecker;
    long                m_offset;
    long                m_lStopTimeout; // Timeout used for stopping each Test Executer.
};
#endif
