/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     InjectorStatusImpl
// @module  InjectorStatusImpl | This module contains the implementation class
//          for the InjectorStatus CORBA interface.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _InjectorStatusImpl_H_
#define _InjectorStatusImpl_H_

#include "InjectorStatus.h"
#include "TestExecuter.h"
#include "TestManager.h"
#include "RemoteRepository.h"
#include <OSTAORB/OSTAORB.h>
#include <string>
#include "InjectorControl_i.h"
#include "OstaTraceDef.h"

///////////////////////////////////////////////////////////////////////////////
// @class   InjectorStatusImpl | This is the implementation class for the
//          InjectorStatus CORBA interface. The Injector Status object resides
//          in the OpenSTA Daemon process on each Injector (an Injector is a
//          host on which OpenSTA has been installed and on which one or more
//          OpenSTA Executers may run). There is exactly one InjectorStatus
//          object per Injector.
//
//          The InjectorStatus object is responsible for retrieving status data
//          from all the Executers on an Injector, while a Test is running.
//          This data is requested by the Test Manager, on behalf of Commander
//          and is only required when a Test is being monitored using
//          Commander. 
//
class InjectorStatusImpl:public virtual POA_InjectorStatus,public PortableServer::RefCountServantBase
{
// @access Public Members.
public:
	// @cmember Constructor.
    InjectorStatusImpl(CORBAServer* corba, InjectorControlImpl& ic);
	// @cmember Destructor.
    virtual ~InjectorStatusImpl();

	// @cmember Get status snapshots for all Executers on this Injector.
    CORBA::Boolean GetStatsSnapshot(TESnapList& InjectorSnapList);

    // @cmember Get total number of Virtual Users across all Executers on this Injector.
	long GetInjectorVUs(long lPreviousTestVUCount);
	CORBA::Boolean GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots);

// @access Private Members.
private:
	int m_bTraceLevel;
    // @cmember Write message to IS Log and OpenSTA Error Log.
    void LogError(int severity, const std::string& strmsg);
    // @cmember Write message to IS Log.
    void WriteLogMsg(int severity, const std::string& strmsg);

    // @cmember Pointer to CORBA server object.
    CORBAServer* m_corba;
    // @cmember Reference to injector control object.
    InjectorControlImpl& m_ic;
	// @cmember Registered name of this CORBA object.
    std::string  m_registered_name;
	// @cmember Pointer to remote repository object.
	CRemoteRepository* m_pRepository;
    // @cmember Pointer to IS Log file.
    std::ofstream* m_pLogFile;
};
#endif