/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RemoteRepository.cpp: implementation of the CRemoteRepository class.
//
//////////////////////////////////////////////////////////////////////

#include "RemoteRepository.h"
#include <CECInetInfo.h>
#include <fstream>
#include "CECSimpleParse.h"
using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRemoteRepository::CRemoteRepository(CORBAServer* corba,const string& node,
									 string strTestName, string strTestRunFolder)
{
	m_pCorba = corba;
	m_strReposNode = node;
	m_pRootFolder = NULL;
	m_pObjCodeFolder = NULL;
	m_pProfileFolder = NULL;
	m_pDataFolder = NULL;
	m_pProject = NULL;
	m_pReposServer = NULL;
	m_pTestFolder = NULL;
	m_pTestStoreFolder = NULL;
	m_pTestRunFolder = NULL;

    CECInetInfo info;
    string underscore_ip_name=info.GetIPAddressAsString(m_strReposNode);
	
    string factory=underscore_ip_name+".CyranoFactory";
    string dataservername=underscore_ip_name+"_CyrStProvider_001";
    string msg="Node "+underscore_ip_name+" is not running a DataServer";

	// Attempt to connect to factory on the repository host machine
    try
    {
		m_pCorba->FindObjectWithName(Provider::CLASS_NAME,dataservername.c_str());
    }
    catch(...)
    {
    }

	// Create the OrbServer and create connections to the main folders
    try
    {
		m_pReposServer = new CECOrbServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		m_pProject = new CECOrbProject(m_pReposServer, "Repository");
		m_pRootFolder = new CECOrbFolder(m_pProject);
		m_pObjCodeFolder = new CECOrbFolder(m_pRootFolder, "ObjectCode");
		m_pProfileFolder = new CECOrbFolder(m_pRootFolder, "Profiles");
		m_pDataFolder = new CECOrbFolder(m_pRootFolder, "Data");
		m_pTestFolder = new CECOrbFolder(m_pRootFolder, "Tests");
		m_pTestStoreFolder = new CECOrbFolder(m_pTestFolder, strTestName.c_str());
		m_pTestRunFolder = new CECOrbFolder(m_pTestStoreFolder, strTestRunFolder.c_str());
	    ReadTraceSettings();
    }
    catch(...)
    {
    }
}

CRemoteRepository::~CRemoteRepository()
{
	try
	{
		if (m_pTestRunFolder)
			delete m_pTestRunFolder;
		if (m_pTestStoreFolder)
			delete m_pTestStoreFolder;
		if (m_pTestFolder)
			delete m_pTestFolder;
		if (m_pRootFolder)
			delete m_pRootFolder;
		if (m_pObjCodeFolder)
			delete m_pObjCodeFolder;
		if (m_pProfileFolder)
			delete m_pProfileFolder;
		if (m_pDataFolder)
			delete m_pDataFolder;
		if (m_pProject)
			delete m_pProject;
		if (m_pReposServer)
			delete m_pReposServer;
	}
	catch(...)
	{
	}
}

bool CRemoteRepository::CheckoutTofFile(const char* strTofFile)
{
	try
	{
		if (!m_pObjCodeFolder)
			return false;
	
		CECOrbFile tofFile(m_pObjCodeFolder, strTofFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\";
#else
		strDestPath += "Engines/";
#endif
		strDestPath += strTofFile;

		tofFile.CheckOut(strDestPath.c_str(), false);
		return true;
	}
	catch(...)
	{
	}

	return false;
}

bool CRemoteRepository::CheckoutProfileFile(const char* strProfileFile)
{
	try
	{
		if (!m_pProfileFolder)
			return false;
	
		CECOrbFile profileFile(m_pProfileFolder, strProfileFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\";
#else
		strDestPath += "Engines/";
#endif
		strDestPath += strProfileFile;

		profileFile.CheckOut(strDestPath.c_str(), false);
		return true;
	}
	catch(...)
	{
	}
	return false;
}

bool CRemoteRepository::CheckoutDataFile(const char* strDataFile)
{
	try
	{
		if (!m_pDataFolder)
			return false;
	
		CECOrbFile dataFile(m_pDataFolder, strDataFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\";
#else
		strDestPath += "Engines/";
#endif
		strDestPath += strDataFile;

		dataFile.CheckOut(strDestPath.c_str(), false);
		return true;
	}
	catch(...)
	{
	}
	return false;
}

bool CRemoteRepository::CheckoutTraceSettingsFile()
{
    const char FilNam [] = "TraceSettings.txt";

    try
    {
		if (!m_pRootFolder) return false;
	    CECOrbFile trcSetFile(m_pRootFolder, FilNam);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\";
#else
		strDestPath += "Engines/";
#endif
		strDestPath += FilNam;
        m_strTrcFilPath = strDestPath;  // Save Trace file name

		trcSetFile.CheckOut(strDestPath.c_str(), false);
		return true;
    }
    catch (...)
    {
    }
	return false;
}

bool CRemoteRepository::StoreResultsFile(const char* strResultsFile)
{
	bool bRet = false;
	try
	{
		// Construct path of local results file
		string strLocalPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strLocalPath  += "Engines\\";
#else
		strLocalPath  += "Engines/";
#endif
		strLocalPath += strResultsFile;

		// Check in file to the test run folder
		CECOrbFile resFile(m_pTestRunFolder, strResultsFile, "desc");
		resFile.CheckIn(strLocalPath.c_str(), false);
#ifdef _WIN32
		::DeleteFile( strLocalPath.c_str() );
#else
		unlink(strLocalPath.c_str() );
#endif
		bRet = true;
	}
	catch(...)
	{
	}

	return bRet;
}

void CRemoteRepository::ReadTraceSettings()
{
    if (!CheckoutTraceSettingsFile()) m_bReadTrace = false;

	ifstream ifs (m_strTrcFilPath.c_str());

    while (ifs.good() && !ifs.eof())
	{
		string strline;
		getline(ifs, strline);
		CECSimpleParse parse(strline);
		
		string strName = parse.getNext(',');
		string strLevel = parse.getToEnd();
        m_TraceNameToLevel.insert(TraceNameToLevel_t::value_type(strName, strLevel));

    }
    m_bReadTrace = true;
	return;
}

string CRemoteRepository::GetTraceLevel(string name)
{
    string strTrcLev = "10";        // Assume default trace level

    // If we have a TraceSettings file set level from file, otherwise set
    // a hardwired default value.
    if (m_bReadTrace)
    {
        TraceNameToLevel_t::iterator pTrcI = m_TraceNameToLevel.find(name);
        if(pTrcI != m_TraceNameToLevel.end())
        {
            strTrcLev = pTrcI->second;
        }
    }
    return strTrcLev;
}