/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RemoteRepository.cpp: implementation of the CRemoteRepository class.
//
//////////////////////////////////////////////////////////////////////

#include "RemoteRepository.h"
#include <CECInetInfo.h>
#include <iostream>
using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRemoteRepository::CRemoteRepository(CORBAServer* corba,const string& node)
{
	m_pCorba = corba;
	m_strReposNode = node;
	m_pRootFolder = NULL;
	m_pDataFolder = NULL;
	m_pProject = NULL;
	m_pReposServer = NULL;

    CECInetInfo info;
    string underscore_ip_name=info.GetIPAddressAsString(m_strReposNode);
	
    string factory=underscore_ip_name+".CyranoFactory";
    string dataservername=underscore_ip_name+"_CyrStProvider_001";
    string msg="Node "+underscore_ip_name+" is not running a DataServer";

	// Attempt to connect to factory on the repository host machine
    try
    {
		CORBA::Object_ptr obj;
		obj=m_pCorba->FindObjectWithName(Provider::CLASS_NAME,dataservername.c_str());
		if(CORBA::is_nil(obj))
			throw CyrRepositoryException(msg.c_str(),"CORBARepository::CORBARepository",-1);
    }
    catch(...)
    {
        throw CyrRepositoryException(msg.c_str(),"CORBARepository::CORBARepository",-1);
    }

	// Create the OrbServer and create connections to the main folders
    try
    {
		m_pReposServer = new CECOrbServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		m_pProject = new CECOrbProject(m_pReposServer, "Repository");
		m_pRootFolder = new CECOrbFolder(m_pProject);
		m_pDataFolder = new CECOrbFolder(m_pRootFolder, "Data");
    }
    catch(...)
    {
    }
}

CRemoteRepository::~CRemoteRepository()
{
	if (m_pDataFolder)
		delete m_pDataFolder;
	if (m_pProject)
		delete m_pProject;
	if (m_pRootFolder)
		delete m_pRootFolder;
	if (m_pReposServer)
		delete m_pReposServer;
}

string CRemoteRepository::CheckoutVariableFile(string strVarFile)
{
	strVarFile +=".FVR";
	cout<<"In CheckoutVariableFile: Filename = "<<strVarFile.c_str()<<endl;
	if (!m_pDataFolder)
		return false;
	
	cout<<"Create OrbFile"<<endl;
	CECOrbFile* pVarFile = new CECOrbFile(m_pDataFolder, strVarFile.c_str());
	if (pVarFile)
	{
		string strDestPath = getenv("OPENSTA_HOME");
		strDestPath += "Engines\\";
		strDestPath += strVarFile;

		cout<<"Dest Path = "<<strDestPath.c_str()<<endl;
		cout<<"About to check out!"<<endl;
		pVarFile->CheckOut(strDestPath.c_str(), false);
		cout<<"Done!"<<endl;
		delete pVarFile;
		return strDestPath;
	}

	return "";
}

