/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "TPCYRSemaphore.h"
/////////////////////////////////////////////////////
// @mfunc Constructor
TPCYRSemaphore::TPCYRSemaphore(CORBAServer* pcorba,string MutexName)
        :set_flag(false),m_regname(MutexName),m_corba(pcorba)
{
    m_corba->ActivateObject(this);
    m_corba->NameObject(this->_this(),TPSemaphore::CLASS_NAME,m_regname.c_str());
}
/////////////////////////////////////////////////////
// @mfunc Destructor
TPCYRSemaphore::~TPCYRSemaphore()
{
    m_corba->UnNameObject(TPSemaphore::CLASS_NAME,m_regname.c_str());
}
/////////////////////////////////////////////////////
// @mfunc Acquire Semaphore (Clear state)
void TPCYRSemaphore::acquire()
{
    static omni_mutex mutex;
    omni_mutex_lock guard(mutex);
    set_flag=false;
}
/////////////////////////////////////////////////////
// @mfunc Try and Acquire Semaphore (return true if set)
//      does not actually acquire, just returns whether
//      it is set or not
// @rdesc true if acquired
CORBA::Boolean TPCYRSemaphore::tryacquire()
{
    static omni_mutex mutex;
    omni_mutex_lock guard(mutex);
    return set_flag;
}
/////////////////////////////////////////////////////
// @mfunc release Mutex (Set State)
void TPCYRSemaphore::release()
{
    static omni_mutex mutex;
    omni_mutex_lock guard(mutex);
    set_flag=true;
}
