/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrFFApi
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFFolder_h
#define _CECFFFolder_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "CECFFLink.h"
#include "CECFFProject.h"

///////////////////////////////////////////////////////////////////////////////
// @class   CECFFFolder | This class is a wrapper around the COM interface
//          to OpenSTARep's Folder objects.  It provides a basic set of methods
//          for creation and retrieval of Folders.  Specific Folder
//          instances, such as a Project Settings Folder, are derived from
//          this class and are expected to initialize themselves correctly.
// @base    public | CECFFLink
class CECFFFolder : public CECFFLink
{
    // @access Public Members
public:
    // @cmember Constructor for retrieving root Folders
    CECFFFolder( CECFFProject* project );
    // @cmember Constructor for retrieving child Folders
    CECFFFolder( CECFFFolder* parent, const char* name );
    // @cmember Constructor for creating new child Folders
    CECFFFolder( CECFFFolder* parent, const char* name, const char* desc );
    virtual ~CECFFFolder();  // @cmember Destructor

    string GetName() const;         // @cmember Returns name of Folder
    string GetDescription() const;  // @cmember Returns Folder description
    void SetDescription( const char* desc );    // @cmember Sets new description of Folder
    string GetFolders( long& count ) const;  // @cmember Returns list of child Folders
    string GetFiles( long& count ) const;    // @cmember Returns list of files in Folder
    string GetFullPath() const;              // @cmember Returns full path name of folder

    void Update();  // @cmember Persists any changes into OpenSTARep database
    void Remove();  // @cmember Removes Folder from OpenSTARep database
    void Refresh(); // @cmember Retrieves latest versions of all children

    CECFFServer*  GetParentServer();
    CECFFProject* GetParentProject();
    CECFFFolder*  GetParentFolder();
    string GetFolderHierarchy()const;


    // @access Protected Members
protected:
    CECFFProject* m_project;
    CECFFFolder*  m_parent;

    // @cmember Returns underlying servant reference as an item for linking
    bool RetrieveFolder( CECFFFolder* parent, const char* name );
    string m_name;
    string m_desc;
};

#endif
