/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_TRACE_SUBS              */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  PSL Trace Routines.                          */
/*                                       */
/*  This module contains tracing routines that are version           */
/*  dependant and hence are in SVXAPP.                   */
/*                                       */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>          /* Standard I/O */
#include <stdlib.h>         /* Standard Library Routines */
#include <string.h>         /* String functions */

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL Routines */
#include <trc_rtndef.h>         /* PSL Trace Routines */

#ifdef VMS
#pragma nostandard
/*
 *  CLI routines are not prototyped by VMS.
 */
unsigned long   cli$present (struct dsc$descriptor_s *entdsc_ptr);
#pragma standard
#endif /* VMS */

/*
 *  VMS-Specific Structures
 */
#ifdef VMS
#pragma nostandard          /* Keep Alpha Happy! */
#include <descrip.h>            /* VAX Descriptors */
#pragma standard
#endif /* VMS */


/*
 *  Structure Definitions.
 */
#include <shrmsg.h>         /* PSL messages */
#include <psl_trc_prmdef.h>     /* Trace Symbols */
#include <pslclx.clx_str>       /* Generic Command Line Structures */


/************************************************************************/
/*                                  */
/*  PSL_TRACE_SETUP - Set up Trace options              */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     PSL_TRACE_SETUP (trace_flag, trace_keywords_ptr);        */
/*                                  */
/*  Where:                              */
/*                                  */
/*     trace_flag         = Boolean Tracing Enabled flag        */
/*     trace_keywords_ptr = Pointer to Trace Keywords structure */
/*                                  */
/*  This routine sets up the Trace flags from the structures set    */
/*  up after parsing a command line using the generic command   */
/*  line parsing mechanisms.                    */
/*                                  */
/************************************************************************/

void PSL_TRACE_SETUP (Boolean trace_flag,
                      struct trace_keywords *trace_keywords_ptr)
{
    /*
     *  Local Data.
     */
    struct   trace_scr_keywords *trace_scr_keywords_ptr;
    struct   trace_fdr_keywords *trace_fdr_keywords_ptr;
    struct   trace_trm_keywords *trace_trm_keywords_ptr;
    struct   trace_usr_keywords *trace_usr_keywords_ptr;

    /*
     *  If /TRACE was not present, there is nothing more to do, so exit.
     */
    if (!trace_flag) return;

    /*
     *  Initialize Tracing.
     */
    PSL_INIT_TRACE ();

    /*
     *  /TRACE = COMMAND.
     */
    if ((*trace_keywords_ptr).commandn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_COMMAND);
    }

    /*
     *  /TRACE = TIMING.
     */
    if ((*trace_keywords_ptr).timingn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_TIMING);
    }

    /*
     *  /TRACE = ALIGN.
     */
    if ((*trace_keywords_ptr).alignn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_ALIGN);
    }

    /*
     *  /TRACE = ERRORS.
     */
    if ((*trace_keywords_ptr).errorsn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_ERRORS);
    }

    /*
     *  /TRACE = TPARSE.
     */
    if ((*trace_keywords_ptr).tparsen)
    {
        PSL_SET_TRC_ON (PTRC_GEN_TPARSE);
    }

    /*
     *  /TRACE = MISC.
     */
    if ((*trace_keywords_ptr).miscn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_MISC);
    }


    /*
     *  /TRACE = MEM.
     */
    if ((*trace_keywords_ptr).memn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_MEM);
    }



    /*
     *  /TRACE = PIPES
     */
    if ((*trace_keywords_ptr).pipesn)
    {
        PSL_SET_TRC_ON (PTRC_GEN_PIPES);
    }


    /*
     *  /TRACE = SCR.
     */
    if ((*trace_keywords_ptr).scrn)
    {

        PSL_SET_TRC_ON (PTRC_SCR_ENABLED);

        trace_scr_keywords_ptr = &(*trace_keywords_ptr).scr;

        /*
         *  /TRACE = SCR : CVTRTNS.
         */
        if ((*trace_scr_keywords_ptr).cvtrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_CVTRTNS);
        }

        /*
         *  /TRACE = SCR : FILRTNS.
         */
        if ((*trace_scr_keywords_ptr).filrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_FILRTNS);
        }

        /*
         *  /TRACE = SCR : IMGRTNS.
         */
        if ((*trace_scr_keywords_ptr).imgrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_IMGRTNS);
        }

        /*
         *  /TRACE = SCR : INTRTNS.
         */
        if ((*trace_scr_keywords_ptr).intrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_INTRTNS);
        }

        /*
         *  /TRACE = SCR : TRMRTNS.
         */
        if ((*trace_scr_keywords_ptr).trmrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_TRMRTNS);
        }

        /*
         *  /TRACE = SCR : UPDRTNS.
         */
        if ((*trace_scr_keywords_ptr).updrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_UPDRTNS);
        }

        /*
         *  /TRACE = SCR : FILSTRC.
         */
        if ((*trace_scr_keywords_ptr).filstrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_FILSTRC);
        }

        /*
         *  /TRACE = SCR : HDRSTRC.
         */
        if ((*trace_scr_keywords_ptr).hdrstrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_HDRSTRC);
        }

        /*
         *  /TRACE = SCR : IMGSTRC.
         */
        if ((*trace_scr_keywords_ptr).imgstrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_IMGSTRC);
        }

        /*
         *  /TRACE = SCR : IFISTRC.
         */
        if ((*trace_scr_keywords_ptr).ifistrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_IFISTRC);
        }

        /*
         *  /TRACE = SCR : OLDSTRC.
         */
        if ((*trace_scr_keywords_ptr).oldstrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_OLDSTRC);
        }

        /*
         *  /TRACE = SCR : RECSTRC.
         */
        if ((*trace_scr_keywords_ptr).recstrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_RECSTRC);
        }

        /*
         *  /TRACE = SCR : WRKSTRC.
         */
        if ((*trace_scr_keywords_ptr).wrkstrcn)
        {
            PSL_SET_TRC_ON (PTRC_SCR_WRKSTRC);
        }

    }

    /*
     *  /TRACE = FDR.
     */
    if ((*trace_keywords_ptr).fdrn)
    {
        PSL_SET_TRC_ON (PTRC_FDR_ENABLED);

        trace_fdr_keywords_ptr = &(*trace_keywords_ptr).fdr;

        /*
         *  /TRACE = FDR : MONITOR.
         */
        if ((*trace_fdr_keywords_ptr).monitorn)
        {
            PSL_SET_TRC_ON (PTRC_FDR_MONITOR);
        }

        /*
         *  /TRACE = FDR : REPEAT.
         */
        if ((*trace_fdr_keywords_ptr).repeatn)
        {
            PSL_SET_TRC_ON (PTRC_FDR_REPEAT);
        }
    }

    /*
     *  /TRACE = TRM.
     */
    if ((*trace_keywords_ptr).trmn)
    {
        PSL_SET_TRC_ON (PTRC_TRM_ENABLED);

        trace_trm_keywords_ptr = &(*trace_keywords_ptr).trm;

        /*
         *  /TRACE = TRM : CLEAR.
         */
        if ((*trace_trm_keywords_ptr).clearn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_CLEAR);
        }

        /*
         *  /TRACE = TRM : DRWRTNS.
         */
        if ((*trace_trm_keywords_ptr).drwrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_DRWRTNS);
        }

        /*
         *  /TRACE = TRM : GENRTNS.
         */
        if ((*trace_trm_keywords_ptr).genrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_GENRTNS);
        }

        /*
         *  /TRACE = TRM : HLPRTNS.
         */
        if ((*trace_trm_keywords_ptr).hlprtnsn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_HLPRTNS);
        }

        /*
         *  /TRACE = TRM : INPUT.
         */
        if ((*trace_trm_keywords_ptr).inputn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_INPUT);
        }

        /*
         *  /TRACE = TRM : ODDMENTS.
         */
        if ((*trace_trm_keywords_ptr).oddmentsn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_ODDRTNS);
        }

        /*
         *  /TRACE = TRM : OUTPUT.
         */
        if ((*trace_trm_keywords_ptr).outputn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_OUTPUT);
        }

        /*
         *  /TRACE = TRM : SCROLL.
         */
        if ((*trace_trm_keywords_ptr).scrolln)
        {
            PSL_SET_TRC_ON (PTRC_TRM_SCROLL);
        }

        /*
         *  /TRACE = TRM : SPCRTNS.
         */
        if ((*trace_trm_keywords_ptr).spcrtnsn)
        {
            PSL_SET_TRC_ON (PTRC_TRM_SPCRTNS);
        }

        /*
         *  /TRACE = TRM : ALL.
         */
        if ((*trace_trm_keywords_ptr).alln)
        {
            PSL_SET_TRC_ON (PTRC_TRM_CLEAR);
            PSL_SET_TRC_ON (PTRC_TRM_DRWRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_GENRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_HLPRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_INPUT);
            PSL_SET_TRC_ON (PTRC_TRM_ODDRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_OUTPUT);
            PSL_SET_TRC_ON (PTRC_TRM_SCROLL);
            PSL_SET_TRC_ON (PTRC_TRM_SPCRTNS);
        }
    }

    /*
     *  /TRACE = USER.
     */
    if ((*trace_keywords_ptr).usern)
    {
        PSL_SET_TRC_ON (PTRC_USR_ENABLED);

        trace_usr_keywords_ptr = &(*trace_keywords_ptr).user;

        /*
         *  /TRACE = USER : 1.
         */
        if ((*trace_usr_keywords_ptr)._1n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER1);
        }

        /*
         *  /TRACE = USER : 2.
         */
        if ((*trace_usr_keywords_ptr)._2n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER2);
        }

        /*
         *  /TRACE = USER : 3.
         */
        if ((*trace_usr_keywords_ptr)._3n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER3);
        }

        /*
         *  /TRACE = USER : 4.
         */
        if ((*trace_usr_keywords_ptr)._4n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER4);
        }

        /*
         *  /TRACE = USER : 5.
         */
        if ((*trace_usr_keywords_ptr)._5n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER5);
        }

        /*
         *  /TRACE = USER : 6.
         */
        if ((*trace_usr_keywords_ptr)._6n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER6);
        }

        /*
         *  /TRACE = USER : 7.
         */
        if ((*trace_usr_keywords_ptr)._7n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER7);
        }

        /*
         *  /TRACE = USER : 8.
         */
        if ((*trace_usr_keywords_ptr)._8n)
        {
            PSL_SET_TRC_ON (PTRC_USR_USER8);
        }
    }

    /*
     *  /TRACE = IPC.
     */
    if ((*trace_keywords_ptr).ipcn)
    {
        PSL_SET_TRC_ON (PTRC_IPC_IPC);
    }

    /*
     *  /TRACE = DTC.
     */
    if ((*trace_keywords_ptr).dtcn)
    {
        PSL_SET_TRC_ON (PTRC_IPC_DTC);
    }


    /*
     *  /TRACE = IPC2
     */
    if ((*trace_keywords_ptr).ipc2n)
    {
        PSL_SET_TRC_ON (PTRC_IPC_IPC2);
    }

    /*
     *  /TRACE = DTC2.
     */
    if ((*trace_keywords_ptr).dtc2n)
    {
        PSL_SET_TRC_ON (PTRC_IPC_DTC2);
    }


    return;
}



#ifdef VMS

/************************************************************************/
/*                                  */
/*  PSL_TRACE_PARSE - Parse Trace Qualifier             */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     iret = PSL_TRACE_PARSE ();                   */
/*                                  */
/*  Where:                              */
/*                                  */
/*     iret         = Unsigned Long PSL Routine Return Status   */
/*                                  */
/*  This routine parses the various keywords supported for the  */
/*  Trace qualifier.                        */
/*                                  */
/************************************************************************/

unsigned long PSL_TRACE_PARSE (void)
{
    /*
     *  Local Data.
     */
    unsigned long       iret;       /* Return Status */
    struct   dsc$descriptor_s strdsc;   /* String descriptor */

    /*
     *  Set up string descriptor
     */
    strdsc.dsc$b_class = DSC$K_CLASS_S;
    strdsc.dsc$b_dtype = DSC$K_DTYPE_T;

    /*
     *  /TRACE.
     *
     *  If /TRACE was not present, there is nothing more to do, so exit.
     */
    strdsc.dsc$a_pointer = "TRACE";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (!(psl_test_status (iret))) goto end;

    /*
     *  Initialize Tracing.
     */
    PSL_INIT_TRACE ();

    /*
     *  /TRACE = COMMAND.
     */
    strdsc.dsc$a_pointer = "TRACE.COMMAND";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_COMMAND);

    /*
     *  /TRACE = TIMING.
     */
    strdsc.dsc$a_pointer = "TRACE.TIMING";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_TIMING);

    /*
     *  /TRACE = ALIGN.
     */
    strdsc.dsc$a_pointer = "TRACE.ALIGN";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_ALIGN);

    /*
     *  /TRACE = ERRORS.
     */
    strdsc.dsc$a_pointer = "TRACE.ERRORS";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_ERRORS);

    /*
     *  /TRACE = TPARSE.
     */
    strdsc.dsc$a_pointer = "TRACE.TPARSE";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_TPARSE);

    /*
     *  /TRACE = MISC.
     */
    strdsc.dsc$a_pointer = "TRACE.MISC";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_MISC);

    /*
     *  /TRACE = MEM.
     */
    strdsc.dsc$a_pointer = "TRACE.MEM";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_MEM);



    /*
     *  /TRACE = SCR.
     */
    strdsc.dsc$a_pointer = "TRACE.SCR";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret))
    {
        PSL_SET_TRC_ON (PTRC_SCR_ENABLED);

        /*
         *  /TRACE = SCR : CVTRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.CVTRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_CVTRTNS);

        /*
         *  /TRACE = SCR : FILRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.FILRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_FILRTNS);

        /*
         *  /TRACE = SCR : IMGRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.IMGRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_IMGRTNS);

        /*
         *  /TRACE = SCR : INTRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.INTRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_INTRTNS);

        /*
         *  /TRACE = SCR : TRMRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.TRMRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_TRMRTNS);

        /*
         *  /TRACE = SCR : UPDRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.UPDRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_UPDRTNS);

        /*
         *  /TRACE = SCR : FILSTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.FILSTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_FILSTRC);

        /*
         *  /TRACE = SCR : HDRSTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.HDRSTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_HDRSTRC);

        /*
         *  /TRACE = SCR : IMGSTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.IMGSTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_IMGSTRC);

        /*
         *  /TRACE = SCR : IFISTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.IFISTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_IFISTRC);

        /*
         *  /TRACE = SCR : OLDSTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.OLDSTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_OLDSTRC);

        /*
         *  /TRACE = SCR : RECSTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.RECSTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_RECSTRC);

        /*
         *  /TRACE = SCR : WRKSTRC.
         */
        strdsc.dsc$a_pointer = "TRACE.SCR.WRKSTRC";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_SCR_WRKSTRC);

    }

    /*
     *  /TRACE = FDR.
     */
    strdsc.dsc$a_pointer = "TRACE.FDR";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret))
    {
        PSL_SET_TRC_ON (PTRC_FDR_ENABLED);

        /*
         *  /TRACE = FDR : MONITOR.
         */
        strdsc.dsc$a_pointer = "TRACE.FDR.MONITOR";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_FDR_MONITOR);

        /*
         *  /TRACE = FDR : REPEAT.
         */
        strdsc.dsc$a_pointer = "TRACE.FDR.REPEAT";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_FDR_REPEAT);



    }

    /*
     *  /TRACE = USER.
     */
    strdsc.dsc$a_pointer = "TRACE.USER";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret))
    {
        PSL_SET_TRC_ON (PTRC_USR_ENABLED);

        /*
         *  /TRACE = USER : 1.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.1";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER1);

        /*
         *  /TRACE = USER : 2.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.2";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER2);

        /*
         *  /TRACE = USER : 3.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.3";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER3);

        /*
         *  /TRACE = USER : 4.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.4";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER4);

        /*
         *  /TRACE = USER : 5.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.5";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER5);

        /*
         *  /TRACE = USER : 6.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.6";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER6);

        /*
         *  /TRACE = USER : 7.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.7";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER7);

        /*
         *  /TRACE = USER : 8.
         */
        strdsc.dsc$a_pointer = "TRACE.USER.8";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_USR_USER8);

    }

    /*
     *  /TRACE = TRM.
     */
    strdsc.dsc$a_pointer = "TRACE.TRM";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret))
    {
        PSL_SET_TRC_ON (PTRC_TRM_ENABLED);

        /*
         *  /TRACE = TRM : CLEAR.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.CLEAR";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_CLEAR);

        /*
         *  /TRACE = TRM : DRWRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.DRWRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_DRWRTNS);

        /*
         *  /TRACE = TRM : GENRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.GENRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_GENRTNS);

        /*
         *  /TRACE = TRM : HLPRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.HLPRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_HLPRTNS);

        /*
         *  /TRACE = TRM : INPUT.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.INPUT";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_INPUT);

        /*
         *  /TRACE = TRM : ODDMENTS.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.ODDMENTS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_ODDRTNS);

        /*
         *  /TRACE = TRM : OUTPUT.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.OUTPUT";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_OUTPUT);

        /*
         *  /TRACE = TRM : SCROLL.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.SCROLL";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_SCROLL);

        /*
         *  /TRACE = TRM : SPCRTNS.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.SPCRTNS";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_TRM_SPCRTNS);

        /*
         *  /TRACE = TRM : ALL.
         */
        strdsc.dsc$a_pointer = "TRACE.TRM.ALL";
        strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

        iret = cli$present (&strdsc);
        if (psl_test_status (iret))
        {
            PSL_SET_TRC_ON (PTRC_TRM_CLEAR);
            PSL_SET_TRC_ON (PTRC_TRM_DRWRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_GENRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_HLPRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_INPUT);
            PSL_SET_TRC_ON (PTRC_TRM_ODDRTNS);
            PSL_SET_TRC_ON (PTRC_TRM_OUTPUT);
            PSL_SET_TRC_ON (PTRC_TRM_SCROLL);
            PSL_SET_TRC_ON (PTRC_TRM_SPCRTNS);
        }

    }



    /*
     *  /TRACE = IPC.
     */
    strdsc.dsc$a_pointer = "TRACE.IPC";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_IPC_IPC);

    /*
     *  /TRACE = DTC.
     */
    strdsc.dsc$a_pointer = "TRACE.DTC";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_IPC_DTC);



    /*
     *  /TRACE = IPC2.
     */
    strdsc.dsc$a_pointer = "TRACE.IPC2";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_IPC_IPC2);

    /*
     *  /TRACE = DTC2.
     */
    strdsc.dsc$a_pointer = "TRACE.DTC2";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_IPC_DTC2);

    /*
     *  /TRACE = PIPES.
     */
    strdsc.dsc$a_pointer = "TRACE.PIPES";
    strdsc.dsc$w_length = strlen (strdsc.dsc$a_pointer);

    iret = cli$present (&strdsc);
    if (psl_test_status (iret)) PSL_SET_TRC_ON (PTRC_GEN_PIPES);



    /*
     *  END:
     *  ====
     *
     *  Routine exit point.
     */
end:
    ;
    return;
}

#endif /* VMS */
