/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
/*===========================================================================*/
/*									     */
/*			  **** V - T E S T ****   			     */
/*									     */
/*===========================================================================*/
/*									     */
/*			Module Name: VTEST_RTNS_UNIX			     */
/*			============					     */
/*									     */
/* Description:								     */
/* ============								     */
/*									     */
/*    This module contains miscellaneous platform specific routines used     */
/*    by more than one V-TEST component.				     */
/*									     */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

#ifdef	UNIX				/* UNIX SPECIFIC MODULE		*/

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/*
 *  Standard UNIX header files.
 */
#include <unistd.h>
#include <fcntl.h>			/* File control */
#include <sys/wait.h>			/* For wait() - Wait for child	*/
#include <sys/stat.h>			/* File status 			*/
#include <sys/time.h>			/* Timer structures 		*/
#include <sys/types.h>			/* User access routines */    
#include <fcntl.h>			/* File control 		*/
#include <dirent.h>			/* Directory file entry 	*/
#include <limits.h>			/* Max/Min values */
#include <signal.h>			/* For kill */		      
#include <pwd.h>			/* For passwd */	      

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>			/* PSL Routines 		*/
#include <trc_rtndef.h>			/* Trace Routines 		*/

static unsigned long VTEST_IS_DIR	(struct diagstk *diagstk_ptr,
				  	        char    *filnam_ptr,
				  	        Boolean *flag_ptr);
/*
 *  Structure Definitions.
 */
#include <vtest_srvinfdef.h>		/* VTEST Server information	*/

/*
 *  ...and general parameter files.
 */
#include <shrmsg.h>			/* PSL Messages 		*/
#include <vtestmsg.h>			/* V-TEST Messages 		*/
#include <pslerr_prmdef.h>		/* Error Constants 		*/
#include <psldiag_prmdef.h>		/* Diagnostic Routine Symbols 	*/
#include <psl_trc_prmdef.h>		/* Trace Routine Parameters 	*/
#include <psl_vm_prmdef.h>		/* LL Routine Parameters 	*/
#include <pslerr_prmdef.h>		/* Error Routine Symbols 	*/
#include <vtest_prmdef.h>		/* V-TEST Parameters		*/

/*
 *  Define the two pipe functions
 */
#define	READPIPE	0
#define	WRITEPIPE	1



/************************************************************************/
/*									*/
/*    VTEST_CREATE_PROCESS - Create a TEST Server on the local node	*/
/*    ===================						*/
/*									*/
/*    Calling Format:							*/
/*	iret = VTEST_CREATE_PROCESS (diagstk_ptr 			*/
/*				   ,image_name				*/
/*				   ,username				*/
/*				   ,priority				*/
/*				   ,spawned_process			*/
/*				   ,&srvpid);				*/
/*									*/
/*    Where:								*/
/*	iret = PSL Status return					*/
/*	diagstk_ptr - Pointer to the diagnostic stack to use		*/
/*	image_name  - Pointer to name of image to run.			*/
/*	username    - Username under which the process is to be created	*/
/*	priority    - Priority for new process				*/
/*	spawned_process - PTRUE if we are to spawn the process, PFALSE	*/
/*			  if a detached process is required.		*/
/*	srvpid - Address of a long to return the PID of the server	*/
/*									*/
/*    This routine is called in order to create a new process running	*/
/*    running on the local node and executing the specified image.	*/
/*									*/
/************************************************************************/

unsigned long	VTEST_CREATE_PROCESS	(struct   diagstk *diagstk_ptr
					,         char    *image_ptr
					,         char    *username_ptr
					,unsigned long    priority
					,         Boolean spawned_process
					,unsigned long    *srvpid_ptr)
{
/*
 *  ENVS_TO_PASSON is the number of environment variables to pass to the image
 *  envnamlistp is a list of environment variables to explicitly pass on to
 *  the image - this list MUST be terminated with an EXTRA null character to
 *  indicate the end of the list.
 *  It should go without saying that there should be ENVS_TO_PASSON items
 *  in envnamlistp.
 */          
#define ENVS_TO_PASSON 12
    const char *envnamlistp=
"TZ\0VTESTIOM\0VTESTTELNET\0PURIFYOPTIONS\0LANG\0LC_MESSAGES\0SYBASE\0ORACLE_HOME\0mdb_capdir\0VTESTSQLR\0SQLR_THREAD_LIMIT\0LD_LIBRARY_PATH\0\0";
/* -55X005*/
    
unsigned long   iret;                           /* PSL return status */
unsigned long   saved_diagctx;                  /* Saved diagnostics context */
pid_t           prc_id;                         /* PID of child or 0 for parent*/
int             child_status;                   /* Status returned from wait() */
char            *env_ptr;                       /* Environment variable */
/* +51X018 */
const char      *elp;                           /* ptr into envlistp */
char            *envp[ENVS_TO_PASSON+1];        /* Environment to pass to LGR */
char            envvalues[ENVS_TO_PASSON][512]; /* env var values */
/* -51X018 */
char            image_location[512];              /* Path and name of image */
long            envnum;                         /* Number of envs */	     
int             pipefd[2];                      /* Pipe file descriptors */ 
char            *msg_ptr;                       /* Diagnostic segment pointer */
unsigned long   last_error;                     /* Last error context */
unsigned long   context;                        /* Current error context */
size_t          seg_len;                        /* Message segment length */
char            pipemsg[256];                   /* Pipe message */
long            status;                         /* Routine status return */
struct          passwd     *pw_ptr;		/* Password structure */
char	        *args[10];      		/* Argument array 	*/
char 	        **arg_ptr;      		/* Pointer to arg array */
char*		pEndOfArg;
int		isub;		

/*
 *  Assume success 
 */
    iret = PSL_NORMAL;

/*
 *  First things first, add a diagnostic message to the stack.
 */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr
				       ,"During VTEST_CREATE_PROCESS");

/*
 *  Output Trace Information.
 */
    if (PSL_TEST_TRACE (PTRC_GEN_MISC))
    {
	PSL_OUTPUT_TRACE ("Attempting to start a local Test Server\n");
    }

/*
 *  Get the password entry for the specified username
 */
    if ((username_ptr != NULL) && (strlen(username_ptr) != 0))
    {					/* If we specified a username	*/
        setpwent();
	pw_ptr = getpwnam (username_ptr);

/*
 *  Ensure that the password file is closed.
 */
        endpwent();
	if (pw_ptr == NULL) return LGN_INVUSRNAM;
    }

/*
 *  Obtain the value of the environment variables we want to pass on to 
 *  the server:
 *  	TZ defines the time-zone.
 *      LANG defines the local language
 *      LC_MESSAGES defines message text language/format.
 *  	VTESTIOM defines the path to iom.
 *	VTESTTELNET defines an alternative for the telnet image
 *  	PURIFYOPTIONS defines the purify options.
 *      VTESTLGR defines the lgr binary - useful for debugging
 */
/*
 *  start by ensuring the array of pointer to environment variables are all
 *  NULLS - the last element in this array MUST be NULL as it signals the
 *  end of the list to exec().
 */
    for (envnum = 0; envnum <= ENVS_TO_PASSON; envnum++)
    {
        envp[envnum] = NULL;
    }
    envnum = 0;                                 /* No envs done yet */
    elp = envnamlistp;                          /* start with first env */
    while(*elp != '\0')                         /* until there are no more */
    {
        env_ptr = getenv(elp);                  /* get env value */
        if (env_ptr != NULL)                    /* if envvar defined */
        {
            (void)strcpy(envvalues[envnum], elp);
            (void)strcat(envvalues[envnum], "=");
            (void)strcat(envvalues[envnum], env_ptr);
            envp[envnum] = envvalues[envnum];   /* save in environment list */
            envnum++;                           /* next env var */
        }
        elp = strchr(elp, '\0');            	/* find current terminator */
        elp++;                              	/* move to next env var */
    }
/* -51X018 */

/*
 *  We now check for the environment variable VTESTLGR in case the user wants
 *  to force LGN to execute a different version of LGR to that in psl_root.
 */
    env_ptr = getenv("VTESTLGR");
    if (env_ptr == NULL)
    {
	(void) strcpy (image_location ,image_ptr);		      
    }
    else
    {
	(void) strcpy (image_location, env_ptr);		      
    }

    if (PSL_TEST_TRACE (PTRC_GEN_MISC))
    {
	PSL_OUTPUT_TRACE ("Command = \"%s\"\n", image_location);
    }

/*
 *	Create a pipe for communicating with the grandchild process
 */
    status = pipe (pipefd);
    if (status == -1)
    {
	iret = PSL_DIAG_ERRNO (diagstk_ptr, "pipe()");
	goto end;			/* Just return status */
    }

/*
 *	Set the close on exec bit for the pipe so that a read will complete
 *	when the exec succeeds
 */
    status = fcntl (pipefd[WRITEPIPE], F_SETFD, 1);		
    if (status == -1)
    {
	iret = PSL_DIAG_ERRNO (diagstk_ptr, "fcntl()");
	goto end;			/* Just return status */
    }

/*
 *  We now want to create a second process running the image that is a process 
 *  group leader (i.e. has no parent process). This is achieved by forking a 
 *  process to obtain a child and then the child forking again to get a 
 *  grandchild.
 *  The original child then terminates leaving the grandchild without a parent.
 *
 *  Note that we just return with any errors on the diagnostic stack, so
 *  that they will be reported tidily rather than via a panic box.
 */
    prc_id = fork();			/* Fork a child process */
    if (prc_id < 0)			/* Failed to fork */
    {
	iret = PSL_DIAG_ERRNO(diagstk_ptr, "fork()");
	goto end;
    }
    else if (prc_id == 0)			/* Child */
    {
/*
 *	The child process executes this code. It forks again to obtain the
 *	grandchild.
 */
	prc_id = fork();
	if (prc_id < 0)			/* Failed to fork */
	{
	    _exit(errno);		/* Terminate child with failure state */
	}
	else if (prc_id == 0)		/* Grandchild */
/*
 *	The following code executes in the (2nd) forked process.
 *	Any errors which occur here are passed back to the parent
 *	process via the pipe.					   
 */
	{
/*
 *	Close the read end of the pipe
 */
 	    (void) close (pipefd[READPIPE]);			

/*
 *	Write the PID to the pipe as the first item of data
 */
	    prc_id = getpid();
	    (void) write (pipefd[WRITEPIPE], &prc_id, sizeof(prc_id));
					/* Write PID to pipe */

/*
 *	    The grandchild executes this code. It becomes a process group
 *	    leader and then executes the specified image.
 */
	    status = setsid();		/* Become process group leader */
	    if (status == -1)		/* Error */
	    {
		iret = PSL_DIAG_ERRNO (diagstk_ptr, "setsid()");    
		goto write_pipe;	/* Write error to pipe */   
	    }

/*
 *    If username specified, reset uid & gid of current process to that 
 *    for the specified username
 */
	    if ((username_ptr != NULL) && (strlen(username_ptr) != 0))
	    {				/* If we specified a username	*/
		status = setgid (pw_ptr->pw_gid);	
					/* Reset to requested gid */
		if (status == -1)
		{
		    iret = PSL_DIAG_ERRNO (diagstk_ptr, "setgid()");
		    goto write_pipe;	/* Write error to pipe */   
		}
		status = setuid (pw_ptr->pw_uid);	
					/* Reset to requested uid */
		if (status == -1)
		{
		    iret = PSL_DIAG_ERRNO (diagstk_ptr, "setuid()");
		    goto write_pipe;	/* Write error to pipe */   
	 	}
	    }

/*
 *	    Now the grandchild executes, passing the specified parameter 
 *	    as an argument.
 */

/*
 *    We need to identify the name of the image that is to be executed, any
 *    arguments that we may need to pass and the environment. We split the 
 *    command by searching for the spaces.
 */
	    args[0] = image_location;	/* First argument = image */
	    pEndOfArg = strchr (image_location ,' ');
	    isub = 1;
	    while (pEndOfArg != NULL)
	    {				/* If we found a space 		*/
		*pEndOfArg = '\0';	/* ...terminate at it		*/
		args[isub] = pEndOfArg + 1; 
					/* Point to next arg */	
	    	pEndOfArg = strchr (args[isub],' ');
					/* Search for space */
		isub += 1;
	    }
	    args[isub] = NULL;		/* Terminate the list	*/

/*
 *    ..and exec the image, passing to it any arguments that we may have.
 */
	    arg_ptr = args;
	    status = execve (image_location ,arg_ptr ,envp);
/*
 *	If we get here, the exec must have failed
 */
	    iret = PSL_DIAG_ERRNO (diagstk_ptr, "execve()");	    

/*
 *	We come here in the case of errors preceding the exec or from the
 *	exec itself.  We copy the last error from the diagnostic stack to
 *	the pipe, so that the parent process can pick it up.
 */
write_pipe:
	    last_error = PSL_GET_ERRORCTX (diagstk_ptr);
					/* Get context of start of last error */
	    context = 0;		/* Initialize context */
	    msg_ptr = PSL_GET_DIAGNOSTIC (diagstk_ptr, &context);
					/* Get first segment */
	    while (context > last_error)
					/* While more segments to message */
	    {
		seg_len = strlen(msg_ptr) + 1;
					/* Length of segment incl. terminator */
		errno = 0;
		do
		{
		    status = write (pipefd[WRITEPIPE],		
					&seg_len, sizeof(seg_len));
					/* Write segment length to pipe */
		}
		while ((status != 0) && (errno == EINTR));
		errno = 0;
		do
		{
		    status = write (pipefd[WRITEPIPE], msg_ptr, seg_len);
					/* Write segment to pipe */   
		}
		while ((status != 0) && (errno == EINTR));
		msg_ptr = PSL_GET_DIAGNOSTIC (diagstk_ptr, &context);
					/* Get next segment */
	    }
	    seg_len = 0;		/* Terminate with zero length */
	    errno = 0;
	    do
	    {
		(void) write (pipefd[WRITEPIPE], &seg_len, sizeof(seg_len));
					/* Write terminator to pipe */
	    }
	    while ((status != 0) && (errno == EINTR));
	    _exit(0);			/* Exit with success */
	}
	else				/* Child process */
	{
/*
 *	The following code executes in the 1st fork process following the
 *	fork of the 2nd fork process - we exit in order that the 2nd fork
 *	process becomes owned by PID 1
 */
	    _exit(0);			/* Exit with success */
	}
    }
    else				/* Parent process */
    {
/*
 *	Wait for the first forked process to exit
 */
	errno = EINTR;			/* Set interrupt occurred */
	while (errno == EINTR)		/* While waitpid() interrupted */
	{
	    (void) waitpid(prc_id, &child_status, 0);
					/* Wait for process to exit */
	}				

/*
 *	Check the status from the child.
 */
        if (WEXITSTATUS(child_status) != 0)
	{
	    errno = WEXITSTATUS(child_status);
	    iret = PSL_DIAG_ERRNO (diagstk_ptr, "fork() in child");
	    goto close_writepipe;
	}

/*
 *	Close the write end of the pipe
 */
	(void) close (pipefd[WRITEPIPE]);			

/*
 *	Read the pipe to check for successful exec in grandchild
 */
	errno = 0;
	do
	{
	    status = read (pipefd[READPIPE], &prc_id, sizeof(prc_id));
	}
	while ((status != 0) && (errno == EINTR));

/*
 *	The first item should be the PID - if so, return it to the caller
 */
	if (status == sizeof(prc_id))
	{
	    *srvpid_ptr = prc_id;

/*
 *	Read the pipe again to check for any diagnostics
 */
	    errno = 0;
	    do
	    {
		status = read (pipefd[READPIPE], &seg_len, sizeof(seg_len));
								/*50X354*/
	    }
	    while ((status != 0) && (errno == EINTR));

/*
 *	If we got 0 bytes, the write end must have closed on exec, so
 *	assume success.
 */
	    if (status == 0)
	    {
		iret = PSL_NORMAL;	/* Return success */
	    }

/*
 *	 If we got >0 bytes, there must have been an error
 */
	    else if (status > 0)
	    {
		pipemsg[0] = '%';	/* Prefix with % to avoid conversion */
		while (seg_len != 0)	/* While there are message segments */
		{
		    errno = 0;
		    do
		    {
			status = read (pipefd[READPIPE], pipemsg + 1, seg_len);
					/* Read message segment */   
		    }
		    while ((status != 0) && (errno == EINTR));
	 	    if (status < seg_len) 	
					/* If failed to read */
		    {
			seg_len = 0;	/* Terminate reading */
		    }
		    else
		    {
			if (pipemsg[1] == '%')
			{
			    (void) PSL_ADD_DIAGNOSTIC (diagstk_ptr, pipemsg);
					/* Add segment with preceding % */
			}
			else
			{
			    (void) PSL_ADD_DIAGNOSTIC (diagstk_ptr, pipemsg + 1);
					/* Add segment as is */
			}
			errno = 0;
			do
			{
			    status = read (pipefd[READPIPE],
					   &seg_len, sizeof(seg_len));
					/* Read next segment length */
			}
			while ((status != 0) && (errno == EINTR));
			if (status < sizeof(seg_len)) seg_len = 0;
				 	/* Terminate if failed to read */
		    }
		}		    
		PSL_DIAG_INVERT (diagstk_ptr);	
					/* Invert the diagnostics */
		iret = PSL_SYSERVERR;	
	    }
	}
	else	/* (status = sizeof(prc_id)) */
	{
/*
 *	If we didn't read the PID there must have been a serious error,
 *	so just return with status
 */
	    iret = PSL_DIAG_ERRNO (diagstk_ptr, "read() on pipe");
	}
    }

/*
 *  CLOSE_WRITEPIPE:
 *
 *  Close read end of pipe
 */
close_writepipe:
    (void) close (pipefd[READPIPE]);				

/*
 *  Reset diagnostic stack if we were successful
 */
    if (psl_test_status(iret)) PSL_REMOVE_DIAGNOSTICS(diagstk_ptr, saved_diagctx);

/*
 *    END
 *    ---
 *
 *    We branch here if we have either completed the operation or had an error
 *    doing so. 
 */
end:
    return iret;
}
/************************************************************************/
/*									*/
/*    VTEST_LOCAL_SERVERS - Create a linked list of LOCAL TEST SERVERS	*/
/*                                                                      */
/*    Calling Convention:  						*/
/*									*/
/*	iret = VTEST_LOCAL_SERVERS (diagstk_ptr ,ll_ptr); 		*/
/*									*/
/*    Where:								*/
/*	iret = PSL routine return status				*/
/*	diagstk_ptr - Pointer to diagnostics stack to use.		*/
/*	ll_ptr - Pointer to the LL to hold the records.			*/
/*									*/
/*    This routine is called to construct a linked list of all of the 	*/
/*    V-TEST Server processes which are present on the local machine	*/
/*									*/
/*    No notice is taken as to whether or not the current user will be	*/
/*    able to access the server or not.					*/
/*									*/
/*    After the list has been used, FREE_LOCAL_SERVERS should be called	*/
/*    to deallocated the linked list created.				*/
/*									*/
/************************************************************************/

unsigned long	VTEST_LOCAL_SERVERS	 (struct   diagstk *diagstk_ptr
					 ,struct   psl_vm_llb *srv_ll_ptr)

{
unsigned long    iret;			/* PSL return status value	*/
unsigned long    saved_diagctx;		/* Saved diagnostics context	*/
struct	 srvinf  srvinf_rec;		/* Server list record		*/
	 long	 keyfile_fd;		/* FD for keyfiles		*/
	 int	 status;                /* Status from FILE functions */
	 int	 closests;              /* Status from close() */     
Boolean  param_error = PFALSE;	/* Error in parameters flag	*/
	 DIR	 *dirp = NULL;		/* Ptr to directory file 	*/
struct   dirent	 *dirent_ptr;		/* Ptr to returned entry structure */
	 Boolean dir_flg;		/* File is a directory 		*/
	 char	 current_file[_POSIX_PATH_MAX];
					/* The current file 		*/
	 char	 directory[_POSIX_PATH_MAX];
					/* Name of directory 		*/

/*
 *    First things first, add a diagnostic message to the stack.
 */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr 
				     ,"During VTEST_LOCAL_SERVERS");

/*
 *    Validate the input parameters
 *
 *    The linked list pointer must not be zero.
 */
    if (srv_ll_ptr == 0)
    {
	param_error = PTRUE;
	(void) PSL_ADD_STATDIAG (diagstk_ptr ,"Invalid Linked List Pointer");
    }

    if (param_error)
    {
	iret = PSL_BADPARAM;
	PSL_PUTDIAG (diagstk_ptr ,2 ,PSL_SYSERVERR,
				"VTEST_LOCAL_SERVERS",iret);	    
	goto end;
    }

/*
 *  Open the <psl_root>/<node> directory
 */
    PSL_GET_NODEDIR (directory, sizeof (directory));
    dirp = opendir (directory);
    if (dirp == NULL)
    {
	iret = PSL_OPENIN;
	PSL_LOG_CIOERR (diagstk_ptr, PSL_OPENIN, directory);	
	goto end;
    }

/*
 *    Pickup the (next) file in the directory and check to see if its name
 *    has the prefix "VTEST_KEY_" and has an 8 digit suffix after this 
 *    If so then it is a valid V-TEST Keyfile.
 */
    dirent_ptr = (struct dirent *)1;	/* Make sure loop executes once */
    while (dirent_ptr != NULL)		/* While entries exist in directory */
    {
	dirent_ptr = readdir (dirp);	/* Get name of a file 		*/

/*
 *    If we found the name of a file then construct its full file name.
 */
	if (dirent_ptr != NULL)
	{
	    (void) strcpy(current_file, directory);
	    (void) strcat(current_file, "/");	
	    (void) strcat(current_file, (*dirent_ptr).d_name);

/*
 *    Find out if this file is a directory
 */
	    iret = VTEST_IS_DIR (diagstk_ptr, current_file, &dir_flg);
	    if (!(psl_test_status(iret))) goto end;

/*
 *    If this file is not a directory and looks like a VTEST Key file then
 *    process it.
 */
	    if ((!dir_flg) 
            &&  (memcmp ((*dirent_ptr).d_name, "VTEST_KEY", 9) == 0))
	    {

/*
 *    This is a valid keyfile but may not represent a Test Executer or Server
 *    which is active at this time. Check for this by stripping the PID off the
 *    end of the end of the file name and checking for a process with this PID
 *
 *    If we cannot open the file for any reason we simply ignore it, we do not
 *    report an error as the problem is not really that important.
 */
                PSL_CLEARERR ();
                keyfile_fd = open (current_file ,O_RDONLY ,0);	 
                if (keyfile_fd == -1) goto next_file;

/*
 *    Read the keyfile record from the file. This will contain information which
 *    will identify the Test Server/Executer for which this keyfile was created.
 */
		PSL_CLEARERR();					
		do
		{
		    status = read (keyfile_fd ,&srvinf_rec,
						sizeof (struct srvinf));
		}
		while ((status < 0) && (errno == EINTR));    

/*
 *    close the KEY file NOW so this file descriptor is not leaked.   
 */
                closests = close (keyfile_fd);
                if (closests == -1)
                {
                    iret = PSL_CLOSEIN;
                    PSL_LOG_CIOERR (diagstk_ptr, iret, current_file);
                    goto end;
                }
/*
 *    if we failed to read data from the KEY file...
 */
                if (status < 0)
                {			/* If we had an error reading	*/
	            iret = PSL_READERR;	/* ...then report and return it	*/
        	    PSL_LOG_CIOERR (diagstk_ptr, PSL_READERR, current_file);
                    goto end;					
                }
                else if (status > 0)
                {			/* ...else if we read a record	*/

/*
 *    We have read a record from a V-TEST Keyfile. From this we can recover
 *    much information relating to the Test Executer and should check to see
 *    firstly if the file is valid and secondly ensure that we have access to
 *    the Test Executer to which the file pertains.
 */
                    errno = 0;                                     
                    status = kill (srvinf_rec.srvinf_pidl ,0);
					/* ...check for process still exists */

/*
 *    If the process to which this file pertains is no longer active then 
 *    delete the file and skip to the next one.
 *
 *    NOTE - kill() returns an error if:
 *
 *      [a] the process sent the signal is not owned by this user
 *      [b] the process does not exist
 */
                    if ((status != 0) && (errno != EPERM))      
                                        /* ...if process not there	*/
                    {
			PSL_CLEARERR();				
			do
			{
				status = remove (current_file);
					/* ...delete the keyfile	*/
			}
			while (errno == EINTR);		
                        if (status != 0) 
                        {		/* ...if we have an error	*/   
	                    iret = PSL_FILNOTDEL;
					/* ...then report and return it	*/
        	            PSL_LOG_CIOERR (diagstk_ptr, iret ,current_file);
                            goto end;
                        }
                    }
                    else
                    {

/*
 *    All o.k. - Add this keyfile record to the menu linked list.
 */
                        iret = PSL_VM_INSERT (srv_ll_ptr ,(char *) &srvinf_rec
                                             ,NULL ,NULL ,PSL_VM_POS_END);
					/* ...and it to end of LL	*/
                        if (!(psl_test_status (iret)))
                        {		/* ...if we failed to add it	*/
                            PSL_PUTDIAG (diagstk_ptr ,2 ,PSL_SYSERVERR
                                        ,"PSL_VM_INSERT (VTEST_LOCAL_SERVERS)" 
                                        ,iret);
                            goto end;	/* ...signal the error and exit	*/
                        }

                    }			/* ...end of PID check block	*/
                }			/* ...record read from keyfile	*/
next_file:;
            }                        	/* ...recovered V-TEST Key file	*/
        }				/* ...got file from directory	*/
    }					/* ...next file from directory	*/

/*
 *    END
 *    ---
 *
 *    ...and close the directory again before exiting.
 */
end:;
    if (dirp != NULL)
    {
        status = closedir (dirp);
        if (status == -1)
        {
            iret = PSL_CLOSEIN;
            PSL_LOG_CIOERR (diagstk_ptr, PSL_CLOSEIN, directory);
            goto end;
        }
    }

/*
 *    ...and clean up the diagnostics stack if required.
 */
    if (psl_test_status (iret))		/* If routine successful	  */
    {
        PSL_REMOVE_DIAGNOSTICS (diagstk_ptr ,saved_diagctx);
    }					/* Remove diagnostics		  */
    return iret;			/* ...and return with status	  */
}

/************************************************************************/
/*									*/
/*    VTEST_IS_DIR - Is the specified file a directory file		*/
/*    ============							*/
/*									*/
/*    Calling Format:							*/
/*	iret = VTEST_IS_DIR (diagstk_ptr, filnam_ptr, flag_ptr);	*/
/*									*/
/*    Inputs:								*/
/*	diagstk_ptr    - Pointer to the diagnostic stack		*/
/*	filnam_ptr  - Pointer to null terminated filename		*/
/*									*/
/*    Outputs:								*/
/*	flag_ptr    - Pointer to Boolean for file is{not} directory	*/
/*									*/
/*    Return Status 							*/
/*	PSL_NORMAL if no errors						*/
/*	error stat otherwise.						*/
/*									*/
/*    This routine is called to determine whether the specified		*/
/*    filespec is a directory or not.					*/
/*									*/
/************************************************************************/
static unsigned long VTEST_IS_DIR	(struct diagstk *diagstk_ptr,
				  	        char    *filnam_ptr,
				  	        Boolean *flag_ptr)
{
/*
 *    Local data
 */
struct	 stat	file_stat;		/* File status structure */
unsigned long	iret;			/* Return status */
	 int	statsts;		/* Status from stat() */

/*
 *  Assume success and file is not a directory.
 */
    iret = PSL_NORMAL;
    *flag_ptr = PFALSE;

/*
 *  We use the stat() function to get information on this file.
 *  stat() will fail if the file does not exist or if the directory this
 *  file is in is not searchable - other errors are unexpected.
 */
    PSL_CLEARERR();			/* Clear file error status */
    do							
    {
	statsts = stat(filnam_ptr, &file_stat);
    }
    while (errno == EINTR);		

/*
 *  If stat fails we need to see why.
 *  If a file does not exist then it is obviously not a directory.
 */
    if (statsts != 0)			/* Not successful */
    {
	if (errno == ENOENT)		/* File does not exist */
	{
	    goto end;
	}
	else				/* Some other error */
	{
	    if (errno != 0)
	    {
		iret = PSL_DIAG_ERRNO(diagstk_ptr, "stat()");
	    }
	    else
	    {
		iret = PSL_OPENIN;
		PSL_LOG_CIOERR(diagstk_ptr, iret, filnam_ptr);
	    }
	    goto end;
	}
    }

/*
 *  Now use the S_ISDIR macro to determine if this file is a directory.
 */
    if (S_ISDIR(file_stat.st_mode) == 0)
    {
	*flag_ptr = PFALSE;
    }
    else
    {
	*flag_ptr = PTRUE;
    }

/*
 *  This label is branched to with an error or is dropped into if the function
 *  is successful.
 */
end:;
    return iret;
}

/************************************************************************/
/*									*/
/*    VTEST_CHK_SRVACC - Check current user has access to server   	*/
/*    ================							*/
/*									*/
/*    Calling Format:							*/
/*	iret = VTEST_CHK_SRVACC (scope, groupid, userid);		*/
/*									*/
/*    Inputs:								*/
/*	scope	    -	The scope to check against			*/
/*	groupid	    -	Group ID of current user			*/
/*	userid	    -	User ID of current user				*/
/*									*/
/*    Return Status 							*/
/*	PSL_NORMAL if no errors						*/
/*	PSL_NOPRIV if user has no access to the runtime component	*/
/*									*/
/*    This routine is called to determine whether we may access the	*/
/*    specified V-TEST environment.					*/
/*									*/
/*    This is determined by the scope of that environment and our UID.	*/
/*									*/
/*    The rules are:							*/
/*									*/
/*    a) If the scope is WORLD then access is granted			*/
/*									*/
/*    b) If the scope is GROUP AND the user is in the same group as the */
/*	 supervisor, OR is the superuser then access is granted.	*/
/*									*/
/*    c) If the scope is USER AND this user is the same OR he is the	*/
/*	 superuser access is granted.					*/
/*									*/
/************************************************************************/

unsigned long VTEST_CHK_SRVACC (unsigned long scope,
			        unsigned long groupid,
			        unsigned long userid)
{
/*
 *  Local Data,
 */
    uid_t effuid;			/* Effective user ID */
    gid_t effgid;			/* Effective group ID */

/*
 *  If we have scope WORLD then allow access.
 */
    if (scope == VTEST_SCOPE_WORLD) return PSL_NORMAL;

/*
 *  Get the current user's effective user and group IDs.
 */
    effgid = getegid();			/* Get effective group ID */
    effuid = geteuid();			/* Get effective user ID */

/*
 *  If we have scope GROUP the current user must be in the same group or
 *  be the superuser.
 */
    if (scope == VTEST_SCOPE_GROUP)
    {
	if (((gid_t)groupid == effgid) ||
					/* In same group */
	    (effuid == 0))		/* ... or Superuser */
	{
	    return PSL_NORMAL;
	}
    }

/*
 *  If we have scope USER then the users must be the same or the current user
 *  must be the superuser.
 */
    if (scope == VTEST_SCOPE_USER)
    {
	if (((uid_t)userid == effuid) ||
					/* The same effective user */
	    (effuid == 0))		/* ... or superuser */
	{
	    return PSL_NORMAL;
	}
    }

/*
 *  We have failed to find an access match so return PSL_NOPRIV.
 */
    return PSL_NOPRIV;
}
/************************************************************************/
/*									*/
/*    VTEST_GET_USERDATA - Get the GID/UID for the specified user  	*/
/*    ==================						*/
/*									*/
/*    Calling Format:							*/
/*	iret = VTEST_GET_USERDATA (diag_ptr 				*/
/*                                ,username_ptr 			*/
/*                                ,gid_ptr 				*/
/*                                ,uid_ptr);				*/
/*									*/
/*    Where:								*/
/*									*/
/*	iret         = PSL routine return status			*/
/*		       PSL_NORMAL if username found/details returned	*/
/*		       PSL_USRNOTFOU if username not found		*/
/*                     other error from underlying routines		*/
/*	diag_ptr     = Pointer to the diagnostics stack to use		*/
/*	username_ptr = Pointer to null terminated user name. 		*/
/*	gid_ptr      = Pointer to long group ID returned		*/
/*	uid_ptr      = Pointer to long process ID returned		*/
/*									*/
/*    This routine is called to translate the specified username and 	*/
/*    return the group and member ID for it. If the username does not	*/
/*    exist on this node then a status of PSL_USRNOTFOU	is returned	*/
/*									*/
/*    Error Handling:							*/
/*	All errors are returned to the caller with appropriate diags	*/
/*      added to the stack.						*/
/*									*/
/************************************************************************/

unsigned long 	VTEST_GET_USERDATA	(struct   diagstk *diag_ptr
                                        ,         char    *username_ptr
                                        ,         long    *gid_ptr    
                                        ,         long    *uid_ptr)
{
/*
 *    Local data
 */
struct	 passwd	 *passwd_ptr;		/* Pointer to Password Structure */
unsigned long    iret = PSL_NORMAL;	/* PSL return status value	*/
unsigned long    saved_diagctx;		/* Saved diagnostics context	*/

/*
 *    First things first, add a diagnostic message to the stack.
 */
    saved_diagctx = PSL_ADD_STATDIAG (diag_ptr ,"During VTEST_GET_USERDATA");

/*
 *    Initially set the returned values to 0.
 */
    *gid_ptr = 0;
    *uid_ptr = 0;

/*
 *  Get a password information structure for this user.
 */
    setpwent ();
    passwd_ptr = getpwnam (username_ptr);

    if (passwd_ptr == NULL)
    {
	PSL_LOG_CIOERR (diag_ptr ,PSL_UNIXERR 
                       ,"getpwuid (VTEST_GET_USERDATA)");
	iret = PSL_USERNOTFOU;
	goto end;
    }

/*
 *  Ensure that the password file is closed.
 */
    endpwent ();

/*
 *    From this structure we can recover and return the GID and UID
 */
    *gid_ptr = (long) (*passwd_ptr).pw_gid;
    *uid_ptr = (long) (*passwd_ptr).pw_uid;

/*
 *    END
 *    ---
 *
 *    Data set up, so end
 */
end:;
    if (psl_test_status (iret))		/* If routine successful	  */
    {
        PSL_REMOVE_DIAGNOSTICS (diag_ptr ,saved_diagctx);
    }					/* Remove diagnostics		*/
    return iret;			/* ...and return with status	  */
}
#endif	/*** UNIX ONLY MODULE ***/
