/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TestManager_i.h: interface for the TestManager_i class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TESTMANAGER_I_H__2AF23798_4853_4B21_8A99_E6342BC6FFE6__INCLUDED_)
#define AFX_TESTMANAGER_I_H__2AF23798_4853_4B21_8A99_E6342BC6FFE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <TestManager.h>
#include <TestExecuter.h>
#include <InjectorStatus.h>
#include <OSTAORB\OSTAORB.h>
#include "LocalRepository.h"
#include "mutex.hpp"

class TestManagerImpl : public POA_TestManager, public PortableServer::RefCountServantBase  
{
public:
	TestManagerImpl(CORBAServer* pServer);
	virtual ~TestManagerImpl();

	char * TestName();
	char * GetTestRunFolderName();

    // Methods for starting and stopping a Task Group.
	CORBA::Boolean StartTaskGroup(TaskGroupStartupInfo& tgInfo);
    TestExecuter_ptr GetTaskGroupExecuter(long lTaskGroupId);
    CORBA::Boolean StopTaskGroup(long lTaskGroupId);

    // Methods for terminating a Test.
    CORBA::Boolean StopTest(CORBA::Boolean bForce, long lTimeout);

    CORBA::Boolean GetStatsSnapshot(TESnapList& TestSnapList);
    long    GetTestVUs();
    void    OnTaskGroupStopped(long lTaskGroupId, TaskGroupStopReason& eReason);
    void    OnInjectorStopped(const char* strInjectorHost);
	TGExecStatus GetTaskGroupStatus(long lTaskGroupId);
	CORBA::Boolean GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots);
	BOOL Initialize(string strTest);
	void CloseDown();

	struct TGExecInfo
	{
		TaskGroupStartupInfo tgStartup;
		TGExecStatus status;
		InjectorControl_ptr pIC;
	};
//	vector <TGExecInfo> m_execInfo;
	CMap <long,long,TGExecInfo,TGExecInfo> m_execInfo;
	CString m_strReg;
	string m_strTestRunString;
	string m_strTestName;

	struct InjectorHost
	{
		BOOL bComplete;
		string strHostName;
		InjectorStatus_ptr pIS;
	};
	vector<InjectorHost> m_injectorsInUse;

	InjectorControl_ptr ConnectToInjector(const char* strHost);
	int m_nDelayedCount;

	CLocalRepository* GetRepository() { return m_pRepos; };
    long RefreshTestVUs();

	// Stop On Complete
	BOOL m_bUseStopOnComplete;
	int m_nStopOnCompleteRemaining;

private:
	CORBAServer* m_pServer;
	int m_nPID;
	CLocalRepository* m_pRepos;
	omni_thread* m_pScheduler;
    long m_lCachedTotalVUs;


	CTMutex m_InjListMutex;
};

#endif // !defined(AFX_TESTMANAGER_I_H__2AF23798_4853_4B21_8A99_E6342BC6FFE6__INCLUDED_)
