/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitorSelect.cpp : implementation file
//

#include "stdafx.h"
#include "commander.h"
#include "MonitorSelect.h"
#include "CmdExec.h"
#include "RegFunctions.h"
#include "CorbaInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMonitorSelect dialog


CMonitorSelect::CMonitorSelect(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitorSelect::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitorSelect)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CMonitorSelect::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitorSelect)
	DDX_Control(pDX, IDC_MONITOR, m_ctlMonitorBtn);
	DDX_Control(pDX, IDC_TESTLIST, m_ctlTestList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitorSelect, CDialog)
	//{{AFX_MSG_MAP(CMonitorSelect)
	ON_BN_CLICKED(IDC_UPDATE, OnUpdate)
	ON_BN_CLICKED(IDC_MONITOR, OnMonitor)
	ON_LBN_SELCHANGE(IDC_TESTLIST, OnSelchangeTestlist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitorSelect message handlers

BOOL CMonitorSelect::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	OnUpdate();
	m_ctlTestList.SetCurSel(0);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMonitorSelect::OnUpdate() 
{
	FindActiveTests();	
}

void CMonitorSelect::FindActiveTests()
{
	m_ctlTestList.ResetContent();

	try
	{
		CStringArray arstrTests;

		CCORBAInfo Info;
		Info.GetActiveTests(arstrTests);
		
		for(int nTest=0;nTest<arstrTests.GetSize();nTest++)
		{	
			m_ctlTestList.AddString(arstrTests[nTest]);
		}
		
		if(arstrTests.GetSize())
		{
			m_ctlTestList.SetCurSel(0);
			m_ctlMonitorBtn.EnableWindow(TRUE);
		}
		else
			m_ctlMonitorBtn.EnableWindow(FALSE);
	}
	catch(...)
	{
		AfxMessageBox("Unable to connect with the OpenSTA Name Server.\rYou will not be able to execute any tests until\rthe name server has been re-started.", MB_OK|MB_ICONWARNING);
	}
}

int CMonitorSelect::GetNextField(CString sep, CString &inStr, CString &field)
{
    if (inStr=="")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep,0);

    if (nPos == -1)
    {
        field = inStr;
        inStr="";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr=inStr.Right(len-nPos-1);
        return 0;
    }
}

void CMonitorSelect::OnMonitor() 
{
	CString strTest;
	int nSel = m_ctlTestList.GetCurSel();
	m_ctlTestList.GetText(nSel, strTest);

	m_strTestFile = GetRepositoryPath() + "Tests\\" + strTest + "\\" + strTest + ".TST";

	CDialog::OnOK();
}

CString CMonitorSelect::GetTestFile()
{
	return m_strTestFile;
}

void CMonitorSelect::OnSelchangeTestlist() 
{
	int nCurSel = m_ctlTestList.GetCurSel();
	m_ctlMonitorBtn.EnableWindow(nCurSel != LB_ERR);
}
