/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// CORBAInfo.cpp: implementation of the CCORBAInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CORBAInfo.h"
#include <string>
#include <vector>
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCORBAInfo::CCORBAInfo()
{
	try
	{
		m_pCorba = new CORBAServer ( 0, NULL );
	}
	catch(...)
	{
		m_pCorba=NULL;
	}
}
CCORBAInfo::~CCORBAInfo()
{
	try
	{
		delete m_pCorba;
	}
	catch(...)
	{
	}
}


BOOL CCORBAInfo::IsTestActive(CString TestName, CString& strReg)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());

			string strTest = TM->TestName();
			if(TestName.CompareNoCase(strTest.c_str()) == 0)
			{
				strReg = TMList[iter].Id.c_str();
				return true;
			}

		}
	}
	catch(...)
	{
	}
	return false;
}

int CCORBAInfo::IntIsTestActive(CString TestName, CString& strReg)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());

			string strTest = TM->TestName();
			if(TestName.CompareNoCase(strTest.c_str()) == 0)
			{
				strReg = TMList[iter].Id.c_str();
				return 2;
			}

		}
	}
	catch(...)
	{
		return 0;//no daemon running
	}
	return 1;
}

void CCORBAInfo::StopTest(CString strTest)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());
			string TestName = TM->TestName();

			if(strTest.CompareNoCase(TestName.c_str()) == 0)
			{
				TM->StopTest(false, 30);
				return;
			}

		}
	}
	catch(...)
	{
	}
}

void CCORBAInfo::KillTest(CString strTest)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());
			string TestName = TM->TestName();

			if(strTest.CompareNoCase(TestName.c_str()) == 0)
			{
				TM->StopTest(true,0);
				return;
			}

		}
	}
	catch(...)
	{
	}
}

void CCORBAInfo::GetActiveTests(CStringArray& arrTests)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());

			string strTest = TM->TestName();
			arrTests.Add(strTest.c_str());
		}
	}
	catch(...)
	{
	}
}

BOOL CCORBAInfo::StartTaskGroup(TaskGroupStartupInfo& tgStartup, CString strManagerReg)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		return TM->StartTaskGroup(tgStartup);
	}
	catch(...)
	{
	}
	return false;
}

BOOL CCORBAInfo::StopTaskGroup(long lTaskGroupID, CString strManagerReg)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		return TM->StopTaskGroup(lTaskGroupID);
	}
	catch(...)
	{
	}
	return false;

}

TGExecStatus CCORBAInfo::GetTaskGroupStatus(long lTaskGroupID, CString strManagerReg)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		return TM->GetTaskGroupStatus(lTaskGroupID);
	}
	catch(...)
	{
	}
	return INACTIVE;
}

BOOL CCORBAInfo::GetTaskGroupSummaryWithTasks(long lTaskGroupID, CString strManagerReg, TaskGroupSummaryWithTasks &summarywt)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		TESnapList snapList;
		TaskGroupSnapList TGSnap;
		if(TM->GetStatsSnapshot(snapList))
		{
			for (int i = 0; i<snapList.length(); i++)
			{
				TGSnap=snapList[i].seqTaskGroupSnaps;
				for(int j = 0; j<TGSnap.length(); j++)
				{
					if(TGSnap[j].lTaskGroupId==lTaskGroupID)
					{
						//copy into summary
						summarywt.lTaskGroupId=lTaskGroupID;
						summarywt.lActiveUsers=TGSnap[j].lActiveUsers;
						summarywt.lCompletedIterations=TGSnap[j].lCompletedIterations;
						summarywt.lRunTime=TGSnap[j].lRunTime;
						summarywt.lBytes=0;		//if idl changes?
						summarywt.lRequests=0;	//if idl changes?

						TaskSnapList seqTasks = TGSnap[j].seqTaskSnaps;
						summarywt.TaskInfo.clear();
						for(int k=0;k<seqTasks.length();k++)
						{
							ScriptSummary ss;
							ss.lTaskID=seqTasks[k].lTaskId;
							ss.lActiveUsers=seqTasks[k].lActiveUsers;
							summarywt.TaskInfo.push_back(ss);
						}
						return true;
					}
				}
			}
		}
	}
	catch(...)
	{
	}

	return false;
}

BOOL CCORBAInfo::GetTaskGroupSummary(long lTaskGroupID, CString strManagerReg, TaskGroupSummary &summary)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		TESnapList snapList;
		TaskGroupSnapList TGSnap;
		if(TM->GetStatsSnapshot(snapList))
		{
			for (int i = 0; i<snapList.length(); i++)
			{
				TGSnap=snapList[i].seqTaskGroupSnaps;
				for(int j = 0; j<TGSnap.length(); j++)
				{
					if(TGSnap[j].lTaskGroupId==lTaskGroupID)
					{
						//copy into summary
						summary.lTaskGroupId=lTaskGroupID;
						summary.lActiveUsers=TGSnap[j].lActiveUsers;
						summary.lCompletedIterations=TGSnap[j].lCompletedIterations;
						summary.lRunTime=TGSnap[j].lRunTime;
						return true;
					}
				}
			}
		}
	}
	catch(...)
	{
	}

	return false;
}

BOOL CCORBAInfo::GetRunFolderName(CString &StrFolder, CString strManagerReg)
{
	CString temp;

	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		StrFolder=TM->GetTestRunFolderName();
		if(!StrFolder.IsEmpty())
		{
			return true;
		}
	}
	catch(...)
	{
	}

  return false;
}

BOOL CCORBAInfo::GetScriptSummary(long lTaskGroupID, CString strManagerReg, ScriptSummaryInfo &summaryInfo)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
		TESnapList snapList;
		TaskGroupSnapList TGSnap;
		if(TM->GetStatsSnapshot(snapList))
		{
			for (int i = 0; i<snapList.length(); i++)
			{
				TGSnap=snapList[i].seqTaskGroupSnaps;
				for(int j = 0; j<TGSnap.length(); j++)
				{
					if(TGSnap[j].lTaskGroupId==lTaskGroupID)
					{
						TaskSnapList seqTasks = TGSnap[j].seqTaskSnaps;
						summaryInfo.clear();
						for (int k = 0; k<seqTasks.length(); k++)
						{
							ScriptSummary summary;
							summary.lTaskID = seqTasks[k].lTaskId;
							summary.lActiveUsers = seqTasks[k].lActiveUsers;
							summaryInfo.push_back(summary);
						}
						return true;
					}
				}
			}
		}
	}
	catch(...)
	{
	}

	return false;
}

BOOL CCORBAInfo::GetTestSummary(CString strManagerReg,TESnapList &teSnapList)
{
	try
	{
		CORBA::Object_var obj;
		obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());
	
		if(TM->GetStatsSnapshot(teSnapList))
		{
			return true;
		}

	}
	catch(...)
	{
	}

	return false;

}

BOOL CCORBAInfo::HasServerBeenCreated()
{
	try
	{
		if(m_pCorba)
		{
			return true;
		}
	}
	catch(...)
	{
	}
	return false;
}

BOOL CCORBAInfo::CreateServer()
{
	if(m_pCorba!=NULL)
	{
		return false;
	}

	try
	{
		m_pCorba = new CORBAServer ( 0, NULL );
		return true;
	}
	catch(...)
	{
	}
	m_pCorba=NULL;
	return false;
}
