/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TimeCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "TimeCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTimeCtrl

CTimeCtrl::CTimeCtrl():
	m_nHours(0),m_nMins(0),m_nSecs(0)
{
}

CTimeCtrl::~CTimeCtrl()
{
}


BEGIN_MESSAGE_MAP(CTimeCtrl, CEdit)
	//{{AFX_MSG_MAP(CTimeCtrl)
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTimeCtrl message handlers

void CTimeCtrl::OnSize(UINT nType, int cx, int cy) 
{
	CEdit::OnSize(nType, cx, cy);
}

int CTimeCtrl::GetHours()
{
	CString str;
	m_ctlHours.GetWindowText(str);
	return atoi(str);
}

int CTimeCtrl::GetMinutes()
{
	CString str;
	m_ctlMins.GetWindowText(str);
	return atoi(str);
}

int CTimeCtrl::GetSeconds()
{
	CString str;
	m_ctlSecs.GetWindowText(str);
	return atoi(str);
}

void CTimeCtrl::SetHours(int nHours)
{
	m_nHours = nHours;
	CString str;
	str.Format("%02d",nHours);
	m_ctlHours.SetWindowText(str);
}

void CTimeCtrl::SetMinutes(int nMins)
{
	m_nMins = nMins;
	CString str;
	str.Format("%02d",nMins);
	m_ctlMins.SetWindowText(str);
}

void CTimeCtrl::SetSeconds(int nSecs)
{
	m_nSecs = nSecs;
	CString str;
	str.Format("%02d",nSecs);
	m_ctlSecs.SetWindowText(str);
}

void CTimeCtrl::PreSubclassWindow() 
{
	m_Font.CreateStockObject(ANSI_FIXED_FONT);
	CClientDC dc(this);
	CFont *pOldFont = dc.SelectObject(&m_Font);
	CSize szTxt = dc.GetTextExtent("  ");
	CSize szTxt2 = dc.GetTextExtent("  :");
	dc.SelectObject(pOldFont);
	
	int nID = GetDlgCtrlID();
	m_ctlHours.Create(WS_CHILD|WS_VISIBLE|ES_NUMBER,CRect(0,0,szTxt.cx,szTxt.cy),this,nID+1000);
	m_ctlMins.Create(WS_CHILD|WS_VISIBLE|ES_NUMBER,CRect(szTxt2.cx,0,szTxt2.cx+szTxt.cx,szTxt.cy),this,nID+1001);
	m_ctlSecs.Create(WS_CHILD|WS_VISIBLE|ES_NUMBER,CRect(szTxt2.cx*2,0,(szTxt2.cx*2)+szTxt.cx,szTxt.cy),this,nID+1002);
		
	m_ctlHours.SetFont(&m_Font);
	m_ctlMins.SetFont(&m_Font);
	m_ctlSecs.SetFont(&m_Font);
	
	m_ctlHours.SetLimitText(3);
	m_ctlMins.SetLimitText(2);
	m_ctlSecs.SetLimitText(2);

	SetFont(&m_Font);
	SetWindowText("  :  :");

	SetSeconds(m_nSecs);
	SetMinutes(m_nMins);
	SetHours(m_nHours);
	
	CEdit::PreSubclassWindow();
}

BOOL CTimeCtrl::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		if(pMsg->wParam == VK_TAB)
		{
			BOOL bShift = (GetKeyState(VK_SHIFT) & 0x8000);
			
			if((!bShift && (pMsg->hwnd == m_ctlHours.m_hWnd)) || 
				(bShift && (pMsg->hwnd == m_ctlSecs.m_hWnd)))
			{
				m_ctlMins.SetFocus();
				m_ctlMins.SetSel(0,-1);
				return true;
			}
			else
			if(pMsg->hwnd == m_ctlMins.m_hWnd)
			{
				if(bShift)
				{
					m_ctlHours.SetFocus();
					m_ctlHours.SetSel(0,-1);
				}
				else
				{
					m_ctlSecs.SetFocus();
					m_ctlSecs.SetSel(0,-1);
				}
				return true;
			}
		}
	break;
	}
	return CEdit::PreTranslateMessage(pMsg);
}

void CTimeCtrl::OnSetFocus(CWnd* pOldWnd) 
{
	CEdit::OnSetFocus(pOldWnd);

	CWnd *p = GetParent()->GetNextDlgTabItem(this,TRUE);
	if(p == pOldWnd)
	{
		m_ctlHours.SetFocus();
		m_ctlHours.SetSel(0,-1);
	}
	else
	{
		m_ctlSecs.SetFocus();
		m_ctlSecs.SetSel(0,-1);
	}
}

void CTimeCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	
//	CEdit::OnLButtonDown(nFlags, point);
}

BOOL CTimeCtrl::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	WORD wID = LOWORD(wParam);
	WORD wCode = HIWORD(wParam);
	HWND h = (HWND)lParam;
	if(wCode == EN_CHANGE)
	{
		if(GetParent() && (wID != GetDlgCtrlID()))
			GetParent()->PostMessage(WM_COMMAND,MAKEWPARAM(GetDlgCtrlID(),EN_CHANGE),(LPARAM)m_hWnd);
	}
	
	return CEdit::OnCommand(wParam, lParam);
}

BOOL CTimeCtrl::EnableWindow(BOOL bEnable)
{
	m_ctlHours.EnableWindow(bEnable);
	m_ctlMins.EnableWindow(bEnable);
	m_ctlSecs.EnableWindow(bEnable);

	return CEdit::EnableWindow(bEnable);
}