/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CYRWDT001.cpp
// -------------
//
//    This is the implementation file for the CYRWDT001 class
//
//=========================================================================

#include "stdafx.h"
#include "CYRWDT001.h"

//
// CYRWDT001
// =========
CYRWDT001::CYRWDT001()
{
    m_iStartParam   = 0;
    m_iIncParam     = 1;
    m_iState        = m_iStartParam;

    m_hOmniNames    = NULL;
    m_OmniNamesPid  = 0;
    m_bIsRunning = FALSE;
    m_hRunThread = NULL;
}

//
// Run
// ===
//
// The main routine called when the service is to be executed
void CYRWDT001::Run()
{
	if(m_bIsRunning)
		return;

	m_hRunThread = AfxBeginThread(&RunThread,this);
}

UINT CYRWDT001::RunThread(LPVOID pParam)
{
	CYRWDT001 *pParent=(CYRWDT001*)pParam;
    if (pParent->GetCyranoHome())
    {
        // Try starting Omninames
        if (pParent->StartOmniNames())
        {
            pParent->m_iState += pParent->m_iIncParam;
            pParent->m_bIsRunning = TRUE;
        }
    }
    BOOL bFallOver = FALSE;

    while (pParent->m_bIsRunning)
    {
        if (pParent->m_hOmniNames != 0)
        {
            // And wait for it to fallover
            if (WaitForSingleObject(pParent->m_hOmniNames, 0) == WAIT_OBJECT_0)
                bFallOver = TRUE;
        }
        else
        {
            bFallOver = TRUE;
        }

        // Keep checking every five seconds
        ::Sleep(5000);
    }
	return 0;
}

//
// StartOmniNames
// ==============
//
// Start the OmniNames names service if present
//
BOOL CYRWDT001::StartOmniNames ()
{
    //
    // Get the execuable name for omninames. If this file does not exist we will not start
    // up the server.
    (void) strcpy(m_chImageName, m_chCyranoHome);
    (void) strcat(m_chImageName, "\\Server\\OmniOrb\\omninames.exe");

    //
    // Tidy up the temporary work files which should be in the OmniNames Log directory
    if (!DeleteLogs())
        return FALSE;

    //
    // Read in the OmniNames Port Number from the registry
    GetOmniPortNo();

    //
    //    ...and run it.
    return StartProcess ();
}

//
// DeleteLogs
// ==========
//
// Delete the existing Omninames logs
BOOL CYRWDT001::DeleteLogs()
{
	CString strOmniLogDir = getenv("OMNINAMES_LOGDIR");
	if (strOmniLogDir.IsEmpty())
		return false;

	CFileFind FF;
	if(!FF.FindFile(strOmniLogDir + "\\omninames-*.*"))
		return TRUE;

	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		while (!::DeleteFile(FF.GetFilePath()))
		{
			::Sleep(500);
		}
	}

    return TRUE;
}

//
// StartProcess
// ============
//
// Start up the specified process and returning status of call
BOOL CYRWDT001::StartProcess ()
{
    char    chCommandLine[512];


    //
    // Format the command line
    GetShortPathName(m_chImageName,chCommandLine,512);

    (void) strcat (chCommandLine, " -start ");
    (void) strcat (chCommandLine, m_chOmniPortNo);

    char ErrorBuffer[255];
    STARTUPINFO StartInfo = {sizeof(StartInfo)};

    ZeroMemory(&StartInfo,sizeof(STARTUPINFO));
    StartInfo.cb = sizeof(STARTUPINFO);
    StartInfo.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    StartInfo.hStdOutput = NULL;
    StartInfo.hStdInput  = NULL;
    StartInfo.hStdError  = NULL;
    StartInfo.wShowWindow = SW_HIDE;

    char chCommonDir[255];
    (void) strcpy (chCommonDir, m_chCyranoHome);
    (void) strcat (chCommonDir, "\\Common");

    ::SetCurrentDirectory(chCommonDir);

    // Create the process to execute the command
    PROCESS_INFORMATION pi;

    if (!::CreateProcess(NULL,
                         chCommandLine,       // command line
                         NULL,                // process security attributes
                         NULL,                // primary thread security attributes
                         TRUE,                // handles are inherited
                         CREATE_NEW_CONSOLE,  // creation flags
                         NULL,                // use parent's environment
                         chCommonDir,         // Current directory
                         &StartInfo,          // STARTUPINFO pointer
                         &pi))                // receives PROCESS_INFORMATION
    {
    //    DebugBreak();
        (void) strcpy (ErrorBuffer, "Error invoking the image ");
        (void) strcpy (ErrorBuffer, chCommandLine);

        LPTSTR lpMsgBuf;

        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);
        //
        // Free the buffer.
        LocalFree( lpMsgBuf );
        return FALSE;
    }

    //
    // ...and save the process details.

    m_hOmniNames = pi.hProcess;
    m_OmniNamesPid = pi.dwProcessId;

    return TRUE;
}

//
// OnStop
// ======
//
// Called when the service control manager wants to stop the service
//
void CYRWDT001::OnStop()
{
    if (m_hOmniNames != 0)
    {
		m_bIsRunning = false;
        ::TerminateProcess (m_hOmniNames ,0);
        m_hOmniNames = NULL;
        m_OmniNamesPid = 0;
		WaitForSingleObject(m_hRunThread,INFINITE);
		DeleteLogs();
    }
}


//
// GetOmniPortNo
// =============
//
// Obtain the OmniNames Port Number
//
void CYRWDT001::GetOmniPortNo()
{
    // From HKEY_LOCAL_MACHINE\SOFTWARE\ORL\OMNIORB\2.0
    HKEY    hRegKey = NULL;
    char    szKeyValueData[256];
    DWORD   dwKeyLen = sizeof(szKeyValueData);

    //
    // First lets open the key
    if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "SOFTWARE\\ORL\\OMNIORB\\2.0", 0, KEY_READ, &hRegKey) == ERROR_SUCCESS)
    {
        //
        // And query the Requested Field
        if (RegQueryValueEx(hRegKey, "ORBInitialPort", 0, NULL, (unsigned char *) &szKeyValueData, &dwKeyLen) == ERROR_SUCCESS)
        {
            (void) strcpy (m_chOmniPortNo, szKeyValueData);
        }

        // then close the key again
        RegCloseKey(hRegKey);
    }
}

//
// GetCyranoHome
// =============
//
// Obtain the Cyrano Home Environmental Variable
//
BOOL CYRWDT001::GetCyranoHome()
{
    if (GetEnvironmentVariable ("OPENSTA_HOME" ,m_chCyranoHome ,sizeof (m_chCyranoHome)) <= 0)
    {
        return FALSE;
    }
    return TRUE;
}

BOOL CYRWDT001::IsRunning()
{
	DWORD dwExitCode;
	if(GetExitCodeProcess(m_hOmniNames,&dwExitCode) && dwExitCode == STILL_ACTIVE)
		return true;
	
	return false;
}