/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "stdafx.h"
#include "PropertiesBar.h"
#include "TestFile.h"
#include "TestCtrl.h"
#include "ConfigurationDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CCORBAInfo* m_pCorbaInfo;
extern HINSTANCE m_hInstance;
/////////////////////////////////////////////////////////////////////////////
// CPropertiesBar

CPropertiesBar::CPropertiesBar(CWnd* pParent /*=NULL*/):
	m_pActivePage(NULL),
	m_pTestFile(NULL),
	m_hwndNotify(NULL),
	m_nLastRow(-1),
	m_nLastColumn(-1),
	m_bNotEditable(FALSE)
{
		m_bTestActive=FALSE;
}

BEGIN_MESSAGE_MAP(CPropertiesBar, CSizingControlBarG)
	//{{AFX_MSG_MAP(CPropertiesBar)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
//	ON_MESSAGE(WMU_REFRESH_LIST,OnRefreshList)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPropertiesBar message handlers

int CPropertiesBar::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSizingControlBarG::OnCreate(lpCreateStruct) == -1)
		return -1;

	AfxSetResourceHandle(m_hInstance);

	//Create all property dialogs
	m_dlgTaskGroup.Create(IDD_TASKGROUP_DLG,this);
	m_dlgHost.Create(IDD_HOST_DLG,this);
	m_dlgVUs.Create(IDD_VUS_DLG,this);
	m_dlgScriptTask.Create(IDD_SCRIPTTASK_DLG,this);

	AfxSetResourceHandle(AfxGetInstanceHandle());
	return 0;
}

void CPropertiesBar::OnSize(UINT nType, int cx, int cy) 
{
	CSizingControlBarG::OnSize(nType, cx, cy);

	if(m_pActivePage && m_pActivePage->GetSafeHwnd())
	{
		m_pActivePage->MoveWindow(0,0,cx,cy);
	}
	Invalidate();
}

void CPropertiesBar::OnDestroy() 
{
	CSizingControlBarG::OnDestroy();
}

void CPropertiesBar::Activate(CTestFile* pTestFile,HWND hwndNotify,const CString& strRegName)
{
	m_pTestFile = pTestFile;
	m_hwndNotify = hwndNotify;

	m_dlgTaskGroup.SetTestRegName(strRegName);
}

void CPropertiesBar::Edit(int nRow, int nColumn)
{
	//this is called when a new task is dropped and
	//when a single left click on the testlist is performed
	m_nLastRow=nRow;
	m_nLastColumn=nColumn;
	//these are needed by the refresh()
	if(!m_pTestFile  || !nRow ||nRow<0|| (nRow-1 >= m_pTestFile->GetNumTaskGroups()))
	{
		EditNothing();
		return;
	}

	switch(nColumn)
	{
	case CTestCtrl::enScheduleCol:
		EditScheduling(nRow-1);
		return;
	case CTestCtrl::enHostCol:
		EditHost(nRow-1);
		return;
	case CTestCtrl::enStartStopStatusCol:
		EditStartStop(nRow-1);
		return;
	case CTestCtrl::enVUCol:
		CTestFile::TaskType nType = m_pTestFile->GetTaskType(nRow-1,0);
		if(nType == CTestFile::enScript)
		{
			EditVUs(nRow-1);
			return;
		}
		else
		{
			// it's not a script
			EditNothing();
			return;
		}
	}
	
	if(nColumn < CTestCtrl::enFirstTaskCol)
	{
		EditNothing();
		return;
	}
	if(nRow==-1 || m_pTestFile->GetNumTasks(nRow-1)<=nColumn-CTestCtrl::enFirstTaskCol)
	{
		EditNothing();
		return;
	}


	// from here they are all tasks

	CTestFile::TaskType nType = m_pTestFile->GetTaskType(nRow-1,nColumn-CTestCtrl::enFirstTaskCol);
	switch(nType)
	{
	case CTestFile::enScript:
		EditScriptTask(nRow-1,nColumn-CTestCtrl::enFirstTaskCol);
	break;
	case CTestFile::enProfile:
		EditProfileTask(nRow-1);
	break;
	default: 
		EditNothing(); 
	break;
	}
}

BOOL CPropertiesBar::EditNothing()  
{
	//Empty property window table
	ShowDialog(NULL,-1);

	return true;
}

BOOL CPropertiesBar::EditStartStop(int nGroup) 
{
	ShowDialog(NULL,-1);
	return true;
}

BOOL CPropertiesBar::EditVUs(int nGroup)
{
	m_dlgVUs.SetTestFile(m_pTestFile,nGroup,m_hwndNotify);
	ShowDialog(&m_dlgVUs,nGroup);
	m_dlgVUs.SetWindowEnabled();
	return true;
}

BOOL CPropertiesBar::OnEraseBkgnd(CDC *pDC)
{
	CRect rc;
	if(m_pActivePage)
	{
		m_pActivePage->GetWindowRect(&rc);
		ScreenToClient(&rc);
		
		pDC->ExcludeClipRect(rc);

		GetClientRect(&rc);
		pDC->DrawFrameControl(rc,DFC_BUTTON,DFCS_BUTTONPUSH);
		return true;
	}

	GetClientRect(&rc);
	pDC->FillSolidRect(rc,GetSysColor(COLOR_3DFACE));

	CFont *pOldFont = (CFont*)pDC->SelectStockObject(DEFAULT_GUI_FONT);
	int nOldBkMode = pDC->SetBkMode(TRANSPARENT);

	pDC->TextOut(10,10,"No Properties Available");

	pDC->SetBkMode(nOldBkMode);
	pDC->SelectObject(pOldFont);
	
	return true;//CSizingControlBarG::OnEraseBkgnd(pDC);
}

BOOL CPropertiesBar::EditHost(int nGroup)
{
	m_dlgHost.SetTestFile(m_pTestFile,nGroup,m_hwndNotify);
	ShowDialog(&m_dlgHost,nGroup);
	m_dlgHost.SetWindowEnabled();
	return true;
}

void CPropertiesBar::ShowDialog(CDialog *pDlg, int nGroup)
{
	if(m_pActivePage)
		m_pActivePage->ShowWindow(SW_HIDE);

	m_pActivePage = pDlg;
	if(m_pActivePage)
	{
		m_pActivePage->ShowWindow(SW_SHOW);	
			CRect rc;
		GetClientRect(&rc);
		m_pActivePage->MoveWindow(0,0,rc.Width(),rc.Height());
	}
	
	if(GetTaskGroupCannotBeEdited(nGroup)&&m_pActivePage)
	{
		//stop editing of property window when not disabled
		if(m_pActivePage->GetSafeHwnd())
		{
			m_pActivePage->EnableWindow(FALSE);
			m_bNotEditable=TRUE;
		}
	}
	else if(m_pActivePage)
	{
		//allow editing of property window when disabled
		if(m_pActivePage->GetSafeHwnd())
		{
			m_pActivePage->EnableWindow(TRUE);
			m_bNotEditable=FALSE;
		}
	}
	Invalidate();
}

BOOL CPropertiesBar::EditScheduling(int nGroup) 
{  
	m_dlgTaskGroup.SetTestFile(m_pTestFile,nGroup,m_hwndNotify);
	ShowDialog(&m_dlgTaskGroup,nGroup);
	m_dlgTaskGroup.SetWindowEnabled();
	return true;
}

BOOL CPropertiesBar::EditProfileTask(int nGroup)
{
	ShowDialog(NULL,-1);
	return true;
}

BOOL CPropertiesBar::EditScriptTask(int nGroup,int nTask)
{
	CString strPath = m_pTestFile->GetTaskPath(nGroup,nTask);
	CFileStatus fs;
	if(!CFile::GetStatus(strPath,fs))
	{
		//if the script has been deleted!
		ShowDialog(NULL,-1);
		return true;
	}

	m_dlgScriptTask.SetTestFile(m_pTestFile,nGroup,nTask,m_hwndNotify);
	ShowDialog(&m_dlgScriptTask,nGroup);
	m_dlgScriptTask.SetWindowEnabled();
	return true;
}

BOOL CPropertiesBar::GetTaskGroupCannotBeEdited(int nGroup)
{
	if(nGroup>=0 && (m_pTestFile->GetNumTaskGroups() > nGroup))
	{
		//if test is not active Enabled and Disabled
		if(GetTestActive()==TRUE)
		{
			//if its disabled but test is active
			DWORD  dwBufSize=0;
			LPBYTE pLoadData = NULL;
			m_pTestFile->GetTaskGroupData(nGroup,&pLoadData,dwBufSize);
			CTaskGroupDefData data;
			data.LoadFromBuffer(pLoadData);
			if(data.GetTaskGroupEnabled()==TRUE)
			{
				return true;
			}
		}
	}
	return false;
}

/*			//TO DO change this so that enabled tgs can be edited if the status changes to
			if(data.GetTaskGroupEnabled()==TRUE)
			{
				TGExecStatus current=GetTaskGroupStatus(m_nLastRow-1);
				if(current==DELAYED ||current==ACTIVE||current==STOPPING)
				{
					return true;
				}
			}
		}
	}
	return false;
}


*/

TGExecStatus CPropertiesBar::GetTaskGroupStatus(int nGroup)
{
	return INACTIVE;

	AfxMessageBox("This function should not be called\nCPropertiesBar::GetTaskGroupStatus(int nGroup)");
	//
	//	remove threads and call from 10 lines above
	//
	if(GetTestActive()==FALSE)
	{
		//this shouldnt ever happen as its called by refresh
		return INACTIVE;	
	}
	//from here the test is running so use corba - DONT! USE A NEW THREAD?
	CString strReg="";
	m_pCorbaInfo->IsTestActive(nGroup,strReg);
	//AfxMessageBox(strReg);
	return m_pCorbaInfo->GetTaskGroupStatus(nGroup,strReg);
}

void CPropertiesBar::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent==IDT_UPDATE_PROPBAR)
	{
		Refresh();
	}
	CSizingControlBarG::OnTimer(nIDEvent);
}

void CPropertiesBar::Refresh()
{
	BOOL notEditable=GetTaskGroupCannotBeEdited(m_nLastRow-1);
	if(notEditable!=m_bNotEditable)
	{
		//if it's changed then call edit() to reload the dialog
		Edit(m_nLastRow,m_nLastColumn);
		m_bNotEditable=notEditable;
	}
	return;
}

BOOL CPropertiesBar::GetTestActive()
{
	return m_bTestActive;
}


void CPropertiesBar::StopActiveTest()
{
	m_bTestActive=FALSE;
	Refresh();
}

void CPropertiesBar::SetTestActive()
{
	m_bTestActive=TRUE;
	Refresh();//call to reload the dialog as it may not be editable
}
