/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitorSummary.cpp : implementation file
//

#include "stdafx.h"
#include "MonitorSummary.h"
#include "TaskGroupDefData.h"
#include "CORBAInfo.h"	//access to the struct

extern CCORBAInfo *m_pCorbaInfo;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_DATA 9
static int ID_REFRESH_RATE = 5000;
const int COL_WIDTH=60;

/////////////////////////////////////////////////////////////////////////////
// CMonitorSummary dialog

UINT CMonitorSummary::GetUpdate(LPVOID pParam)
{
	CMonitorSummary* pWnd = (CMonitorSummary*)pParam;
	try
	{
		DWORD  dwBufSize=0;
		LPBYTE pData = NULL;

		if (!m_pCorbaInfo->HasServerBeenCreated())
			return 0;
		
		do
		{
			CSingleLock lock(&(pWnd->m_ArrayLocked));
			lock.Lock();

		
			pWnd->m_TaskStatus.RemoveAll();
			for(int a=0;a<pWnd->m_pTestFile->GetNumTaskGroups();a++)
			{
				CMonitorSummary::taskStatus newitem;
				newitem.lTaskGroupId=a;

				pWnd->m_pTestFile->GetTaskGroupData(a,&pData,dwBufSize);
				CTaskGroupDefData data;
				data.LoadFromBuffer(pData);
				newitem.bEnabled=data.GetTaskGroupEnabled();
				newitem.status=m_pCorbaInfo->GetTaskGroupStatus(a,pWnd->m_strManagerReg);
				pWnd->m_TaskStatus.Add(newitem);
			}
			///////////////////////
			m_pCorbaInfo->GetTestSummary(pWnd->m_strManagerReg,pWnd->m_snapList);
			lock.Unlock();
		}
		while(WaitForSingleObject(pWnd->m_hCloseThread,ID_REFRESH_RATE)==WAIT_TIMEOUT);
	}
	catch(...)
	{
	}
	return 0;
}

CMonitorSummary::CMonitorSummary(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitorSummary::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitorSummary)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_pTestFile = NULL;
	m_nTotalIndex = 0;
	m_pUpdateThread = NULL;
	m_hCloseThread = CreateEvent(NULL,FALSE,FALSE,NULL);
	m_nMaxUsedTaskCol=-1;//no tasks defined
	m_nTaskGroups=0;	//number of task groups
	m_TaskStatus.RemoveAll();
}

CMonitorSummary::~CMonitorSummary()
{
	if(m_pUpdateThread)
	{
		SetEvent(m_hCloseThread);
		WaitForSingleObject(m_pUpdateThread->m_hThread,5000);
	}
	CloseHandle(m_hCloseThread);
}

void CMonitorSummary::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitorSummary)
	DDX_Control(pDX, IDC_SUMMARY_LIST, m_ctlSummary);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitorSummary, CDialog)
	//{{AFX_MSG_MAP(CMonitorSummary)
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitorSummary message handlers

BOOL CMonitorSummary::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	LoadProfileTypes();

	CRect r;
	GetClientRect(r);
	m_ctlSummary.InsertColumn(0,"Task Group",LVCFMT_LEFT,100);
	m_ctlSummary.InsertColumn(1,"Status",LVCFMT_LEFT,100);
	m_ctlSummary.InsertColumn(2,"Duration",LVCFMT_LEFT,75);
	m_ctlSummary.InsertColumn(3,"Http Requests",LVCFMT_LEFT,75);
	m_ctlSummary.InsertColumn(4,"Http Bytes Exchanged",LVCFMT_LEFT,75);
	m_ctlSummary.InsertColumn(5,"Total VUs",LVCFMT_LEFT,75);

	ASSERT(m_pTestFile != NULL);

	for (int group = 0; group<m_pTestFile->GetNumTaskGroups(); group++)
	{
		m_ctlSummary.InsertItem(group,m_pTestFile->GetTaskGroupName(group));
		m_ctlSummary.SetItemText(group,1,"-");
		m_ctlSummary.SetItemText(group,2,"-");
		m_ctlSummary.SetItemText(group,3,"-");		
		m_ctlSummary.SetItemText(group,4,"-");
		m_ctlSummary.SetItemText(group,5,"-");

		for(int tasks=0;tasks<m_pTestFile->GetNumTasks(group);tasks++)
		{
			if(tasks>m_nMaxUsedTaskCol)
			{
				//create a new column
				CString heading;
				heading.Format("Task %d",tasks+1);
				m_ctlSummary.InsertColumn(tasks+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
				m_nMaxUsedTaskCol=tasks;
			}
			m_ctlSummary.SetItemText(group,tasks+FirstTaskCol,"-");
		
		}
	}
	m_nTaskGroups=group+1;
	m_pUpdateThread = AfxBeginThread(GetUpdate,this);
	SetTimer(IDT_DATA, ID_REFRESH_RATE, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CMonitorSummary::Init(CString strManagerReg, CTestFile* pTestFile)
{
	m_pTestFile = pTestFile;
	m_strManagerReg = strManagerReg;

	return true;
}

void CMonitorSummary::OnTimer(UINT nIDEvent) 
{
	UpdateSummary();
	CDialog::OnTimer(nIDEvent);
}

void CMonitorSummary::UpdateSummary()
{

	CSingleLock lock(&m_ArrayLocked);
	lock.Lock();
	//also locked in Update Member function + OnShowWindow

	for(int a=0;a<m_TaskStatus.GetSize();a++)
	{
		long ltgId=m_TaskStatus[a].lTaskGroupId;
		if(m_TaskStatus[a].bEnabled)
		{
			switch(m_TaskStatus[a].status)
			{
				case ACTIVE: m_ctlSummary.SetItemText(ltgId,1,"Running");
				break;
				case INACTIVE: m_ctlSummary.SetItemText(ltgId,1,"Waiting enabled");
				break;
				case DELAYED: m_ctlSummary.SetItemText(ltgId,1,"Scheduled");
				break;
				case FAILED: m_ctlSummary.SetItemText(ltgId,1,"Failed");
				break;
				case STOPPED: m_ctlSummary.SetItemText(ltgId,1,"Stopped");
				break;
				case STOPPING: m_ctlSummary.SetItemText(ltgId,1,"Stopping");
				break;
				case COMPLETED: m_ctlSummary.SetItemText(ltgId,1,"Completed");
				break;
				default:m_ctlSummary.SetItemText(ltgId,1,"No status");
			}
		}
		else
		{
			m_ctlSummary.SetItemText(ltgId,1,"Disabled");
		}
	}

	
	/////////////////////////////////////////
	TaskGroupSnapList TGSnap;
	CString temp;

	for (int i = 0; i<m_snapList.length(); i++)
	{
		TGSnap=m_snapList[i].seqTaskGroupSnaps;
		for(int j = 0; j<TGSnap.length(); j++)
		{
			long lTaskGroupId=TGSnap[j].lTaskGroupId;
			m_ctlSummary.SetItemText(lTaskGroupId,0,m_pTestFile->GetTaskGroupName(lTaskGroupId));

			
			if((TGSnap[j].lRunTime>0)&&(m_TaskStatus[lTaskGroupId].status==ACTIVE ||m_TaskStatus[lTaskGroupId].status==STOPPING))
			{
			
				m_ctlSummary.SetItemText(lTaskGroupId,2,ConvertTimeToString(TGSnap[j].lRunTime));

				CTestFile::TaskType taskType=m_pTestFile->GetTaskType(lTaskGroupId,0);

				if(taskType==CTestFile::enScript)
				{	
					temp.Format("%d", TGSnap[j].lRequests);
					m_ctlSummary.SetItemText(lTaskGroupId,3,temp);
					temp.Format("%d",TGSnap[j].lBytes);
					m_ctlSummary.SetItemText(lTaskGroupId,4,temp);

					DWORD  dwBufSize=0;
					LPBYTE pData = NULL;
					m_pTestFile->GetTaskGroupData(lTaskGroupId,&pData,dwBufSize);
					CTaskGroupDefData data;
					data.LoadFromBuffer(pData);
					temp.Format("%d/%d",TGSnap[j].lActiveUsers,data.GetNumVUs());
					m_ctlSummary.SetItemText(lTaskGroupId,5,temp);
				}
				else if(taskType==CTestFile::enProfile)
				{
					m_ctlSummary.SetItemText(lTaskGroupId,3,"-");
					m_ctlSummary.SetItemText(lTaskGroupId,4,"-");
					m_ctlSummary.SetItemText(lTaskGroupId,5,"-");
				}

				TaskSnapList seqTasks = TGSnap[j].seqTaskSnaps;
				for (int k = 0; k<seqTasks.length(); k++)
				{
					if(seqTasks[k].lTaskId>m_nMaxUsedTaskCol)
					{
						temp.Format("Task %d",seqTasks[k].lTaskId+1);
						m_ctlSummary.InsertColumn(seqTasks[k].lTaskId+FirstTaskCol,temp,LVCFMT_LEFT,COL_WIDTH);
						m_nMaxUsedTaskCol=seqTasks[k].lTaskId;
					}
					if(taskType==CTestFile::enScript)
					{
						temp.Format("%d",seqTasks[k].lActiveUsers);
						m_ctlSummary.SetItemText(lTaskGroupId,seqTasks[k].lTaskId+FirstTaskCol,temp);
					}
					else if(taskType==CTestFile::enProfile)
					{
						m_ctlSummary.SetItemText(lTaskGroupId,seqTasks[k].lTaskId+FirstTaskCol,"-");
					}
				}
						
				for(int j=seqTasks.length();j<FirstTaskCol+m_nMaxUsedTaskCol;j++)
				{
					//incase some tasks have been removed
					m_ctlSummary.SetItemText(lTaskGroupId,j+FirstTaskCol,"");
				}
			}
			else
			{
				//duration is 0
				m_ctlSummary.SetItemText(lTaskGroupId,2,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,3,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,4,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,5,"-");
				for(int task2=0;task2<m_pTestFile->GetNumTasks(lTaskGroupId); task2++)
				{
					if(task2>m_nMaxUsedTaskCol)
					{
						temp.Format("Task %d",task2+1);
						m_ctlSummary.InsertColumn(task2+FirstTaskCol,temp,LVCFMT_LEFT,COL_WIDTH);
						m_nMaxUsedTaskCol=task2;
					}
					m_ctlSummary.SetItemText(lTaskGroupId,task2+FirstTaskCol,"-");
				}
			}
		
		}
	}

	lock.Unlock();
	return;
}

void CMonitorSummary::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
		
	if(m_ctlSummary.m_hWnd != NULL)
	{
		m_ctlSummary.MoveWindow(0,0,cx,cy);
	}
}

void CMonitorSummary::OnDestroy() 
{
	KillTimer(IDT_DATA);

	if(m_pUpdateThread)
	{
		SetEvent(m_hCloseThread);
		WaitForSingleObject(m_pUpdateThread->m_hThread,5000);
		m_pUpdateThread = NULL;
	}

	CDialog::OnDestroy();
}

void CMonitorSummary::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);

	int highestusedcol=0;
	
	//check that the number of tasks is correct, added/removed?
	for(int group=0;group<m_pTestFile->GetNumTaskGroups();group++)
	{
		if(m_pTestFile->GetNumTasks(group)>highestusedcol)
		{
			highestusedcol=m_pTestFile->GetNumTasks(group)-1;
		}

	}

	//delete the extra task columns
	if(highestusedcol<m_nMaxUsedTaskCol)
	{
		for(int b=highestusedcol;b<m_nMaxUsedTaskCol;b++)
		{
			m_ctlSummary.DeleteColumn(highestusedcol+FirstTaskCol+1);
		}
	}

	//delete all info from deleted tasks
	for(int b=0;b<m_pTestFile->GetNumTaskGroups();b++)
	{
		for(int a=0;a<m_pTestFile->GetNumTasks(b);a++)
		{
			if(a>m_nMaxUsedTaskCol)
			{
				CString temp;
				temp.Format("Task %d",a+1);
				m_ctlSummary.InsertColumn(a+FirstTaskCol,temp,LVCFMT_LEFT,COL_WIDTH);
				m_nMaxUsedTaskCol=a;
			}
			m_ctlSummary.SetItemText(b,FirstTaskCol+a,"-");
		}
		for(int c=m_pTestFile->GetNumTasks(b);c<highestusedcol+FirstTaskCol;c++)
		{
			m_ctlSummary.SetItemText(b,FirstTaskCol+c,"");
		}
	}

	//add any extra task groups
	for(int extraTG=m_nTaskGroups-1; extraTG<m_pTestFile->GetNumTaskGroups(); extraTG++)
	{
		m_ctlSummary.InsertItem(extraTG,m_pTestFile->GetTaskGroupName(extraTG));
		m_ctlSummary.SetItemText(extraTG,1,"-");
		m_ctlSummary.SetItemText(extraTG,2,"-");
		m_ctlSummary.SetItemText(extraTG,3,"-");		
		m_ctlSummary.SetItemText(extraTG,4,"-");
		m_ctlSummary.SetItemText(extraTG,5,"-");
		for(int tasks=0;tasks<m_pTestFile->GetNumTasks(extraTG);tasks++)
		{
			if(tasks>m_nMaxUsedTaskCol)
			{
				CString heading;
				heading.Format("Task %d",tasks+1);
				m_ctlSummary.InsertColumn(tasks+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
				m_nMaxUsedTaskCol=tasks;
			}
			m_ctlSummary.SetItemText(extraTG,tasks+FirstTaskCol,"-");
		}
	}
	m_nTaskGroups=extraTG+1;

	UpdateSummary();
}

CString CMonitorSummary::LookupTargetType(CString strTarget)
{
	CString strDesc = "";
	if (strTarget.CompareNoCase("HTP")==0)
	{
		strDesc = "HTTP Load";
		return strDesc;
	}

	POSITION pos = m_listPlugins.GetHeadPosition();
	while(pos)
	{
		PluginInfo &PI = m_listPlugins.GetNext(pos);
		CString strPIExts(PI.strExtension);
		if (strPIExts.CompareNoCase(strTarget) == 0)
		{
			strDesc = PI.strName;
			strDesc += " Collector";
			return strDesc;
		}
	}
	return strTarget;
}

int CMonitorSummary::LoadProfileTypes()
{
	int nCount = 0;
	CString strRegSubKey("SOFTWARE\\OpenSTA\\PlugIns\\Profiles");

	HKEY hKey;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,strRegSubKey,0,KEY_READ,&hKey)!=ERROR_SUCCESS)
		return nCount;

	DWORD dwIndex=0;
	DWORD dwNameSize=64;
	char szName[64];
	while(RegEnumKeyEx(hKey,dwIndex,szName,&dwNameSize,0,NULL,NULL,NULL)==ERROR_SUCCESS)
	{
		dwNameSize = 64;
		HKEY hKeyPlugin;
		if(RegOpenKeyEx(hKey,szName,0,KEY_READ,&hKeyPlugin) == ERROR_SUCCESS)
		{
			PluginInfo PI;
			PI.strName = szName;

			DWORD dwType;
			DWORD dwDataSize = 256;
			BYTE byData[256];
			//Get Extension for this type
			if(RegQueryValueEx(hKeyPlugin,"Extension",0,&dwType,byData,&dwDataSize)==ERROR_SUCCESS)
				PI.strExtension = (char*)byData;
			dwDataSize = 256;

			RegCloseKey(hKeyPlugin);
			m_listPlugins.AddTail(PI);
			nCount++;
		}
		dwIndex++;
	}
	return nCount;
}

CString CMonitorSummary::ConvertTimeToString(long duration)
{

	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int days=0;

	temp=div(duration,3600*24);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60);
	m=temp.quot;
	s=temp.rem;

	if(days>0)
	{
		answer.Format("%d %02d:%02d:%02d",days,h,m,s);
	}
	else
	{
		answer.Format("%02d:%02d:%02d",h,m,s);
	}
	return answer;
}


/*

		//////////////////////////////////////////////////////////////

				}
				else if(rec.taskGroupType==CTestFile::enProfile)
				{
					m_ctlSummary.SetItemText(rec.nTaskGroup,2,ConvertTimeToString(lSecs));
					m_ctlSummary.SetItemText(rec.nTaskGroup,3,"-");
					m_ctlSummary.SetItemText(rec.nTaskGroup,4,"-");
					m_ctlSummary.SetItemText(rec.nTaskGroup,5,"-");
					for(int task=0;task<rec.tasks; task++)
					{
						if(task>m_nMaxUsedTaskCol)
						{
							CString heading;
							heading.Format("Task %d",task+1);
							m_ctlSummary.InsertColumn(task+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
							m_nMaxUsedTaskCol=task;
						}
						m_ctlSummary.SetItemText(rec.nTaskGroup,task+FirstTaskCol,"-");
					}
			}








			}
			else
			{
				m_ctlSummary.SetItemText(lTaskGroupId,2,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,3,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,4,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,5,"-");
				for(int task=0;task<m_nTaskGroups; task++)
				{
					if(task>m_nMaxUsedTaskCol)
					{
						CString heading;
						heading.Format("Task %d",task+1);
						m_ctlSummary.InsertColumn(task+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
						m_nMaxUsedTaskCol=task;
					}
					m_ctlSummary.SetItemText(lTaskGroupId,task+FirstTaskCol,"-");
				}
			}

			///////////////////////////////////////////









			if(TGSnap[j].lTaskGroupId==lTaskGroupID)
			{
				//copy into summary
				summarywt.lTaskGroupId=lTaskGroupID;
				summarywt.lActiveUsers=TGSnap[j].lActiveUsers;
				summarywt.lCompletedIterations=TGSnap[j].lCompletedIterations;
				summarywt.lRunTime=TGSnap[j].lRunTime;
				summarywt.lBytes=0;		//if idl changes?
				summarywt.lRequests=0;	//if idl changes?

						TaskSnapList seqTasks = TGSnap[j].seqTaskSnaps;
						summarywt.TaskInfo.clear();
						for(int k=0;k<seqTasks.length();k++)
						{
							ScriptSummary ss;
							ss.lTaskID=seqTasks[k].lTaskId;
							ss.lActiveUsers=seqTasks[k].lActiveUsers;
							summarywt.TaskInfo.push_back(ss);
						}
						return true;
					}
				}
			}
		}



/*	for (int i = 0; i<m_SummaryInfo.GetSize(); i++)
	{
		SummaryRecord rec = m_SummaryInfo.GetAt(i);

		if(rec.bEnabled)
		{
			switch(rec.TaskGroupStatus)
			{
			case ACTIVE: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Running");
			break;
			case INACTIVE: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Waiting enabled");
			break;
			case DELAYED: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Scheduled");
			break;
			case FAILED: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Failed");
			break;
			case STOPPED: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Stopped");
			break;
			case STOPPING: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Stopping");
			break;
			case COMPLETED: m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Completed");
			break;
			default:m_ctlSummary.SetItemText(rec.nTaskGroup,1,"No status");
			}
		}
		else
		{
			m_ctlSummary.SetItemText(rec.nTaskGroup,1,"Disabled");
		}
		long lSecs = rec.lRunTime;
		if (lSecs > 0)
		{
			if(rec.taskGroupType==CTestFile::enScript)
			{
				
				m_ctlSummary.SetItemText(rec.nTaskGroup,2,ConvertTimeToString(lSecs));

				CString temp;
				temp.Format("%d", rec.lHTTPRequests);
				m_ctlSummary.SetItemText(rec.nTaskGroup,3,temp);
				temp.Format("%d",rec.lBytesExchanged);
				m_ctlSummary.SetItemText(rec.nTaskGroup,4,temp);

				temp.Format("%d/%d",rec.lActiveUsers,rec.lTotalUsers);
				m_ctlSummary.SetItemText(rec.nTaskGroup,5,temp);

				for(int task=0;task<rec.tasks; task++)
				{
					if(task>m_nMaxUsedTaskCol)
					{
						CString heading;
						heading.Format("Task %d",task+1);
						m_ctlSummary.InsertColumn(task+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
						m_nMaxUsedTaskCol=task;
					}
					temp.Format("%d",rec.TasksVUInfo[task].lActiveVUsInThisTask);
					m_ctlSummary.SetItemText(rec.nTaskGroup,task+FirstTaskCol,temp);
				}
			}
			else if(rec.taskGroupType==CTestFile::enProfile)
			{
				m_ctlSummary.SetItemText(rec.nTaskGroup,2,ConvertTimeToString(lSecs));
				m_ctlSummary.SetItemText(rec.nTaskGroup,3,"-");
				m_ctlSummary.SetItemText(rec.nTaskGroup,4,"-");
				m_ctlSummary.SetItemText(rec.nTaskGroup,5,"-");
				for(int task=0;task<rec.tasks; task++)
				{
					if(task>m_nMaxUsedTaskCol)
					{
						CString heading;
						heading.Format("Task %d",task+1);
						m_ctlSummary.InsertColumn(task+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
						m_nMaxUsedTaskCol=task;
					}
					m_ctlSummary.SetItemText(rec.nTaskGroup,task+FirstTaskCol,"-");
				}
			}
		}
		else
		{
			m_ctlSummary.SetItemText(rec.nTaskGroup,2,"-");
			m_ctlSummary.SetItemText(rec.nTaskGroup,3,"-");
			m_ctlSummary.SetItemText(rec.nTaskGroup,4,"-");
			m_ctlSummary.SetItemText(rec.nTaskGroup,5,"-");
			for(int task=0;task<rec.tasks; task++)
			{
				if(task>m_nMaxUsedTaskCol)
				{
					CString heading;
					heading.Format("Task %d",task+1);
					m_ctlSummary.InsertColumn(task+FirstTaskCol,heading,LVCFMT_LEFT,COL_WIDTH);
					m_nMaxUsedTaskCol=task;
				}
				m_ctlSummary.SetItemText(rec.nTaskGroup,task+FirstTaskCol,"-");
			}
		}
	}
	return;



  	/*
		if(m_SummaryInfo[group].tasks>m_pTestFile->GetNumTasks(group))
		{		
			m_SummaryInfo[group].tasks=m_pTestFile->GetNumTasks(group);
			for(int a=m_SummaryInfo[group].tasks;a<m_nMaxUsedTaskCol+FirstTaskCol;a++)
			{
				m_ctlSummary.SetItemText(group,FirstTaskCol+a,"");
			}
		}
		else if(m_SummaryInfo[group].tasks<m_pTestFile->GetNumTasks(group))
		{
			m_SummaryInfo[group].tasks=m_pTestFile->GetNumTasks(group);
		}
		*/
