

#include "stdafx.h"
#include <afxdllx.h>
#include "PluginTypes.h"
#include "resource.h"       // main symbols
#include "MonitoringDlg.h"
#include "TestFile.h"
#include "CorbaInfo.h"
#include "PluginManager.h"
#include "ostadefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CMonitoringDlg *m_pMonitoringDlg = NULL;
CMonitoringBar *m_pMonitoringBar = NULL;
CToolBar *m_pToolbar = NULL;

CTestFile *m_pTestFile = NULL;
CDocument *m_pDocument = NULL;
CPluginManager *m_pPluginManager=NULL;
CCORBAInfo *m_pCorbaInfo = NULL;

static AFX_EXTENSION_MODULE CMonitoringTabDll = { NULL, NULL };

HINSTANCE m_hInstance = NULL;
extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	m_hInstance = hInstance;
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("MonitoringTabDll.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CMonitoringTabDll, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.

		CyrtoolsInitialize();

		new CDynLinkLibrary(CMonitoringTabDll);
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CMonitoringTabDll.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CMonitoringTabDll);
	
		delete m_pMonitoringDlg;
		m_pMonitoringDlg = NULL;

		delete m_pMonitoringBar;
		m_pMonitoringBar = NULL;
		delete m_pToolbar;
		m_pToolbar = NULL;

		CyrtoolsUninitialize();
    }

    return 1;   // ok
}

extern "C" AFX_EXT_API
BOOL OpenSTAVerify()
{
	return true;
}

extern "C" AFX_EXT_API
void GetTitle(CString& strTitle)
{
	strTitle = "Monitoring Tab";
}

extern "C" AFX_EXT_API
int GetType()
{
	return TEST_PLUGIN_TAB;
}

extern "C" AFX_EXT_API
void GetPackage(CString& strPackage)
{
	strPackage = "MONITORING TAB";
}

extern "C" AFX_EXT_API
void SetTestFile(CTestFile *pTestFile)
{
	m_pTestFile = pTestFile;
}

extern "C" AFX_EXT_API
void SetDocument(CDocument *pDoc)
{
	m_pDocument = pDoc;
}

extern "C" AFX_EXT_API
int GetNumTabs()
{
	return 1;
}

extern "C" AFX_EXT_API
CDialog* CreateTab(int nTab,CWnd *pParent,CString& strTabName)
{
	UINT uiIDD = 0;
	CDialog *pDialog = NULL;
	switch(nTab)
	{
	case 0:
		if(!m_pMonitoringDlg)
		{
			m_pMonitoringDlg = new CMonitoringDlg;
			if(m_pMonitoringBar)
				m_pMonitoringDlg->SetMonitoringBar(m_pMonitoringBar);
			m_pMonitoringDlg->SetTestFile(m_pTestFile);
		}
		pDialog = m_pMonitoringDlg;
		uiIDD = CMonitoringDlg::IDD;
		strTabName = "Monitoring";
	break;
	default:
		ASSERT(FALSE);
	break;
	}

	AfxSetResourceHandle(m_hInstance);
	if(!pDialog || !pDialog->Create(uiIDD,pParent))
	{
		AfxSetResourceHandle(AfxGetInstanceHandle());
		return NULL;
	}
	AfxSetResourceHandle(AfxGetInstanceHandle());

	return pDialog;
}

extern "C" AFX_EXT_API
void GetTabImages(CBitmap& bmpTabs)
{
	AfxSetResourceHandle(m_hInstance);
	bmpTabs.LoadBitmap(IDB_TAB_IMAGES);
	AfxSetResourceHandle(AfxGetInstanceHandle());
}

extern "C" AFX_EXT_API
void SetCorbaInfo(CCORBAInfo *pCorbaInfo)
{
	m_pCorbaInfo = pCorbaInfo;
}

extern "C" AFX_EXT_API
BOOL LoadToolbars(CFrameWnd* pWndFrame,CFrameWnd *pOwner,HMENU hMainMenu)
{
	if(!m_pMonitoringBar)
		m_pMonitoringBar = new CMonitoringBar;
	else
		m_pMonitoringBar->DestroyWindow();

	m_pMonitoringBar->SetOwner(pOwner);
	AfxSetResourceHandle(m_hInstance);
	if (!m_pMonitoringBar->Create(_T("Monitoring"), pWndFrame, 130))
	{
	    TRACE0("Failed to create Monitoring bar\n");
	    return false;
	}
	AfxSetResourceHandle(AfxGetInstanceHandle());
	m_pMonitoringBar->SetBarStyle(m_pMonitoringBar->GetBarStyle() |
	    CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_pMonitoringBar->EnableDocking(CBRS_ALIGN_ANY);

	if(m_pMonitoringDlg)
		m_pMonitoringDlg->SetMonitoringBar(m_pMonitoringBar);

	pWndFrame->DockControlBar(m_pMonitoringBar,AFX_IDW_DOCKBAR_RIGHT);
	pWndFrame->ShowControlBar(m_pMonitoringBar,false,false);
	
	m_pToolbar = new CToolBar;
	m_pToolbar->SetOwner(pOwner);

	AfxSetResourceHandle(m_hInstance);
	// Create toolbar on client's frame window
	if (!m_pToolbar->CreateEx(pWndFrame, TBSTYLE_FLAT,WS_CHILD| CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_pToolbar->LoadToolBar(IDR_MONITOR_TOOLBAR))
	{
		TRACE0("Failed to create toolbar\n");
		return FALSE;
	}

	AfxSetResourceHandle(AfxGetInstanceHandle());

	m_pToolbar->EnableDocking(CBRS_ALIGN_ANY);
	pWndFrame->DockControlBar(m_pToolbar,AFX_IDW_DOCKBAR_TOP);

	//This code will dock the new tool bar to the right of the plugins ones
	CDockBar *pDockBar = (CDockBar*)pWndFrame->GetControlBar(AFX_IDW_DOCKBAR_TOP);
	if(pDockBar)
	{
		for(int nBar = 0;nBar<pDockBar->m_arrBars.GetSize();nBar++)
		{
			CToolBar *pToolBar = (CToolBar*)pDockBar->m_arrBars[nBar];
			
			if(pToolBar == m_pToolbar)
				pDockBar->m_arrBars.RemoveAt(nBar--);
			else
			if(pToolBar)
			{
				while(pDockBar->m_arrBars[++nBar]);
				pDockBar->m_arrBars.InsertAt(nBar,m_pToolbar);
			}
		}
	}

	return true;
}	

extern "C" AFX_EXT_API
void SetPluginManager(CPluginManager* pPluginManager)
{
	m_pPluginManager = pPluginManager;
}

extern "C" AFX_EXT_API
void SetTestActive(BOOL bActive,const CString& strReg)
{
	if(m_pMonitoringDlg)
	{
		if(bActive)
			m_pMonitoringDlg->SetTestActive(strReg);
		else
			m_pMonitoringDlg->StopActiveTest();
	}
}

extern "C" AFX_EXT_API
void LoadMenus(HMENU hMenu)
{
	AfxSetResourceHandle(m_hInstance);

	CMenu *pMenu = CMenu::FromHandle(hMenu);
	
	UINT uiWindowPos = -1;
	UINT uiCount = pMenu->GetMenuItemCount();

	CString strItem;
	//Look for Window menu
	for(UINT uiItem=0;uiItem<uiCount;uiItem++)
	{
		pMenu->GetMenuString(uiItem,strItem,MF_BYPOSITION);
		if(strItem == "&Window")
		{
			uiWindowPos = uiItem;
			break;
		}
	}

	if(uiWindowPos == -1)
	{
		CMenu menu;
		menu.CreateMenu();
		menu.AppendMenu(MF_STRING,ID_CASCADE,"&Cascade");
		menu.AppendMenu(MF_STRING,ID_TILEHORIZONTALLY,"Tile &Horizontally");
		menu.AppendMenu(MF_STRING,ID_TILEVERTICALLY,"Tile &Vertically");
		menu.AppendMenu(MF_STRING,ID_CLOSEALL,"Close &All");

		pMenu->InsertMenu(uiCount-2,MF_BYPOSITION|MF_POPUP,(UINT)menu.Detach(),"&Window");
	}
	else
	{
		CMenu *pSubMenu = pMenu->GetSubMenu(uiWindowPos);
		
		if(pSubMenu->GetMenuState(ID_CASCADE,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_CASCADE,"&Cascade");
		if(pSubMenu->GetMenuState(ID_TILEHORIZONTALLY,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_TILEHORIZONTALLY,"Tile &Horizontally");
		if(pSubMenu->GetMenuState(ID_TILEVERTICALLY,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_TILEVERTICALLY,"Tile &Vertically");
		if(pSubMenu->GetMenuState(ID_CLOSEALL,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_CLOSEALL,"Close &All");
		pSubMenu->Detach();
	}
	
	pMenu->Detach();
	
	AfxSetResourceHandle(AfxGetInstanceHandle());
}