/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// I/O completion port implementation file.
//
// Return values: unless otherwise stated, a DWORD return value is a
// GetLastError() error code.
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Iocp.hpp"

#ifdef USE_PERF_DATA
#   include "PerfCounterKeys.hpp"
#endif

CIOCP::CIOCP(DWORD dwNumberOfConcurrentThreads, CTrace* pTrace)
    : m_dwNumberOfConcurrentThreads(dwNumberOfConcurrentThreads),
      m_hIOCP(NULL), m_dwPeakOutstandingIOs(0),
      m_pTrace(pTrace)
{
#ifdef USE_PERF_DATA
    *g_pPerformance->GetCounterValue(PRFCNT_OPENSTA_THREADPOOL_IOCPQUEUELENGTH) = 0;
#endif
}

CIOCP::~CIOCP()
{
    if(m_hIOCP)
        CloseHandle(m_hIOCP);
}

DWORD CIOCP::Init()
{
    DWORD dwRes = 0;
    m_hIOCP = CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL, 0, m_dwNumberOfConcurrentThreads);
    if(m_hIOCP == NULL)
        dwRes = GetLastError();

#ifdef USE_PERF_DATA
    m_pIOCPQueueLengthPerfCount = g_pPerformance->GetCounterValue(PRFCNT_OPENSTA_THREADPOOL_IOCPQUEUELENGTH);
#endif

    return dwRes;
}

DWORD CIOCP::PostCompletedIO(DWORD dwNumberOfBytes, IocpEventDispatcher *pEventDispatcher, OVERLAPPED *pOverlapped)
{
    DWORD dwRes = 0;
    IncrementOutstandingIoCount();
    if(!PostQueuedCompletionStatus(m_hIOCP, dwNumberOfBytes, (DWORD)pEventDispatcher, pOverlapped))
    {
        dwRes = GetLastError();
        DecrementOutstandingIoCount();
    }

    return dwRes;
}

DWORD CIOCP::GetNextCompletionStatus(DWORD *pdwBytesTransferred, IocpEventDispatcher*& pEventDispatcher, LPOVERLAPPED *ppOverlapped, DWORD dwMilliseconds)
{
    DWORD dwRes = 0;

    if(!GetQueuedCompletionStatus(m_hIOCP, pdwBytesTransferred, (PDWORD)(&pEventDispatcher), ppOverlapped, dwMilliseconds))
        dwRes = GetLastError();

    if(dwRes == 0 || *ppOverlapped != NULL)
        DecrementOutstandingIoCount();

    return dwRes;
}
