/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


%{
#undef alloca
#include "SclWriter.h"
#include <malloc.h>

char *ProcessHeaderStrings(char*,char*);
extern int Tof2Sclerror(const char* s);
extern int Tof2Scllex();

extern bool bDebugOn;

extern SCLWriter *pSclWriter;

%}

/*
 * Declare the type of values in the grammar
 */

%union 
{
	int integer_type;
	char* string_type;
}

/*
 * Token types: These are returned by the lexer
 */

%token INTEGER,STRING_WORD, CONNECT,SERVER,CONID, DISCONNECT, WAIT, 
		REQUEST,HEADER,SECTION_MARKER,BODY,URI, STRING_LINE,
		TDELAY,COMMENT,_TYPE,LOAD_RESPONSE_INFO,IDENTIFIER,
		QUOTED_STRING,DATA,INTO,VARDEF,NAME,OPTION,_SIZE,OPTIONS,
		STIMER,ETIMER,TIMDEF,_SYNCHRONIZE,SETVAR,VALUE,CERTIFICATE,
		BAUTHENTICATION,_BLOB,USERNAME,PASSWORD,DOMAIN,TPARGUMENT
%%

input: /*EMPTY SPACE*/ 
		| input keyword

		| input SERVER { if(bDebugOn) printf("Warning: Found \"server\" keyword in unknown block\n");}
		| input CONID { if(bDebugOn) printf("Warning: Found \"conid\" keyword in unknown block\n");}
		| input TDELAY { if(bDebugOn) printf("Warning: Found \"Wait\" keyword in unknown block\n");}
		| input HEADER { if(bDebugOn) printf("Warning: Found \"header\" keyword in unknown block\n");}
		| input SECTION_MARKER { if(bDebugOn) printf("Warning: Found \"~~~\" in unknown block\n");}
		| input URI { if(bDebugOn) printf("Warning: Found \"URI\" keyword in unknown block\n");}
		| input _TYPE { if(bDebugOn) printf("Warning: Found \"type\" keyword in unknown block\n");}
		| input INTEGER 
		| input STRING_LINE {free($<string_type>2);} 
		| input STRING_WORD {free($<string_type>2);}
		| input QUOTED_STRING {free($<string_type>2);}
		| input NAME { if(bDebugOn) printf("Warning: Found \"name\" keyword in unknown block\n");}
		| input _SIZE { if(bDebugOn) printf("Warning: Found \"size\" keyword in unknown block\n");}
		| input OPTION { if(bDebugOn) printf("Warning: Found \"option\" keyword in unknown block\n");}
		| input OPTIONS { if(bDebugOn) printf("Warning: Found \"options\" keyword in unknown block\n");}
		| input VALUE { if(bDebugOn) printf("Warning: Found \"value\" keyword in unknown block\n");}
		| input CERTIFICATE { if(bDebugOn) printf("Warning: Found \"certificate\" keyword in unknown block\n");}
		| input BAUTHENTICATION { if(bDebugOn) printf("Warning: Found \"BAutentication\" keyword in unknown block\n");}
		| input USERNAME { if(bDebugOn) printf("Warning: Found \"user\" keyword in unknown block\n");}
		| input PASSWORD { if(bDebugOn) printf("Warning: Found \"password\" keyword in unknown block\n");}
		| input DOMAIN { if(bDebugOn) printf("Warning: Found \"domain\" keyword in unknown block\n");}
		| input _BLOB { if(bDebugOn) printf("Warning: Found \"blob\" keyword in unknown block\n");}
		| input TPARGUMENT { if(bDebugOn) printf("Warning: Found \"tpargument\" keyword in unknown block\n");}

keyword: expConnect 
		| expDisconnect 
		| expWait 
		| expRequest
		| expComment
		| expLoadResponseInfo
		| expVardef
		| expSTimer
		| expETimer
		| expTimDef
		| expSynchronize
		| expSetVar
		| expBAuthentication

expConnect:	CONNECT expServer expConid
{
	char *szHostName = $<string_type>2;
	int nConId = $<integer_type>4;
  
	pSclWriter->WriteConnectString(szHostName,nConId);
	free(szHostName); 
};

expDisconnect: DISCONNECT expConid expOptions
{
	int nConId = $<integer_type>2;
	int nOptions = $<integer_type>3; 
	pSclWriter->WriteDisconnectString(nConId,nOptions);
}

expWait: WAIT expTdelay 
{
	int nWait = $<integer_type>2;
	pSclWriter->WriteWaitString(nWait);
}


expRequest:	REQUEST expConid expType expURI expHeaders expOptionalBody expOptionalCertificate expOptionalOption
{
	int nConId = $<integer_type>2;
	SCLWriter::RequestTypes nType = (SCLWriter::RequestTypes)$<integer_type>3;
	char *szHttpVersion = $<string_type>4;
	char *szHeaders = $<string_type>5;
	char *szBody = $<string_type>6;
	char *szCertificate = $<string_type>7;
	int nOption = $<integer_type>8;
	
	pSclWriter->WriteRequestString(nConId,nType,szHttpVersion,szHeaders,szBody,szCertificate,nOption);
	free(szHttpVersion); 
	free(szHeaders); 
	free(szBody);
	free(szCertificate);
}

expLoadResponseInfo: LOAD_RESPONSE_INFO expConid expData expIdentifier expInto
{
	int nConId = $<integer_type>2;
	int nData = $<integer_type>3;
	char* szIdentifier = $<string_type>4;
	char *szInto = $<string_type>5;
	
	pSclWriter->WriteLoadResponseInfoString(nConId,nData,szIdentifier,szInto);
	
	free(szIdentifier);
	free(szInto);
}

expLoadResponseInfo: LOAD_RESPONSE_INFO expConid expData expInto
{
	int nConId = $<integer_type>2;
	int nData = $<integer_type>3;
	char *szInto = $<string_type>4;
	
	pSclWriter->WriteLoadResponseInfoString(nConId,nData,NULL,szInto);
	
	free(szInto);
}

expVardef: VARDEF expName expOption expSize
{
	char *szName = $<string_type>2;
	int nOption = $<integer_type>3;
	int nSize = $<integer_type>4;
	
	pSclWriter->WriteVariableDef(szName,nOption,nSize);
	
	free(szName);
}

expSTimer: STIMER expName
{
	char *szName = $<string_type>2;
	pSclWriter->WriteTimer(szName,true);
	free(szName);
}

expETimer: ETIMER expName
{
	char *szName = $<string_type>2;
	pSclWriter->WriteTimer(szName,false);
	free(szName);
}

expTimDef: TIMDEF expName
{
	char *szName = $<string_type>2;
	pSclWriter->WriteTimerDef(szName);
	free(szName);
}

expSetVar: SETVAR expName expValue
{
	char *szName = $<string_type>2;
	char *szValue = $<string_type>3;
	pSclWriter->WriteSetVar(szName,szValue);
	free(szName);
	free(szValue);
}

expValue: VALUE STRING_LINE
{
	$<string_type>$ = $<string_type>2;
}

expStringToNewLine: STRING_LINE SECTION_MARKER
{
	$<string_type>$ = ProcessHeaderStrings($<string_type>1,NULL);
}
 
expStringToNewLine: STRING_LINE expStringToNewLine 
{
	$<string_type>$ = ProcessHeaderStrings($<string_type>1,$<string_type>2);
}

expURI:  URI QUOTED_STRING {$<string_type>$ = $<string_type>2;}

expHeaders: HEADER expStringToNewLine {$<string_type>$=$<string_type>2}

expIdentifier: IDENTIFIER QUOTED_STRING {$<string_type>$=$<string_type>2;}

expType: _TYPE INTEGER {$<integer_type>$=$<integer_type>2;}

expTdelay: TDELAY INTEGER {$<integer_type>$=$<integer_type>2;}

expServer: SERVER QUOTED_STRING { $<string_type>$=$<string_type>2; }

expConid: CONID INTEGER {$<integer_type>$=$<integer_type>2;}

expOptionalBody: /*OPTIONAL*/ { $<string_type>$ = NULL;}
		| BODY SECTION_MARKER QUOTED_STRING SECTION_MARKER { $<string_type>$ = $<string_type>3;}

expOptionalCertificate: /*OPTIONAL*/ { $<string_type>$ = NULL;}
		| CERTIFICATE QUOTED_STRING { $<string_type>$ = $<string_type>2;}

expData: DATA INTEGER {$<integer_type>$=$<integer_type>2;}

expInto: INTO STRING_WORD { $<string_type>$=$<string_type>2; }

expName: NAME STRING_LINE {$<string_type>$ = $<string_type>2;}

expOptionalOption: /*OPTIONAL*/ {$<integer_type>$=0;}
					| expOption

expOption: OPTION INTEGER {$<integer_type>$=$<integer_type>2;}

expOptions: OPTIONS INTEGER {$<integer_type>$=$<integer_type>2;}

expSize: _SIZE INTEGER {$<integer_type>$=$<integer_type>2;}

expComment: COMMENT expStringToNewLine
{
	char *szComment = $<string_type>2;
	pSclWriter->WriteCommentString(szComment);
	free(szComment);
}

expSynchronize: _SYNCHRONIZE
{
	pSclWriter->WriteSynchronize();
}

expTPArgument: TPARGUMENT INTEGER {$<integer_type>$=$<integer_type>2;}

expUserName: USERNAME STRING_LINE {$<string_type>$ = $<string_type>2;}

expPassword: PASSWORD STRING_LINE {$<string_type>$ = $<string_type>2;}

expOptionalDomain: /*EMPTY*/ {$<string_type>$ = NULL;}
		|DOMAIN STRING_LINE {$<string_type>$ = $<string_type>2;}

expBlob: _BLOB STRING_WORD {$<string_type>$ = $<string_type>2;}

expBAuthentication: BAUTHENTICATION expType expTPArgument expInto
{
	int nType = $<integer_type>2;
	int nTPArg = $<integer_type>3;
	char *szInto = $<string_type>4;
	pSclWriter->WriteBAuthentication(nType,nTPArg,NULL,NULL,NULL,NULL,szInto);
	free(szInto);
}

expBAuthentication: BAUTHENTICATION expType expTPArgument expUserName expPassword expOptionalDomain expInto
{
	int nType = $<integer_type>2;
	int nTPArg = $<integer_type>3;
	char *szUserName = $<string_type>4;
	char *szPassword = $<string_type>5;
	char *szDomain = $<string_type>6;
	char *szInto = $<string_type>7;
	pSclWriter->WriteBAuthentication(nType,nTPArg,szUserName,szPassword,szDomain,NULL,szInto);
	free(szUserName);
	free(szPassword);
	free(szDomain);
	free(szInto);
}

expBAuthentication: BAUTHENTICATION expType expTPArgument expBlob expInto
{
	int nType = $<integer_type>2;
	int nTPArg = $<integer_type>3;
	char *szBlob = $<string_type>4;
	char *szInto = $<string_type>5;
	pSclWriter->WriteBAuthentication(nType,nTPArg,NULL,NULL,NULL,szBlob,szInto);
	free(szBlob);
	free(szInto);
}

%%

/* programs */

/*
 * ???
 */
extern "C"
{
int
Tof2Sclwrap()
{
  return 1;
}
}

/*
 * Report an error situation discovered in a production
 *
 * This does not do anything since we report all error situations through
 * idl_global->err() operations
 */
int
Tof2Sclerror(const char *s)
{
	printf("Syntax error: %s\n",s);
	return 0;
}

char *ProcessHeaderStrings(char* psz1,char* psz2)
{
	int nLen1=0,nLen2=0;
	
	if(psz1!=NULL)
	{
		nLen1 = strlen(psz1);

/*		//strip trailing spaces from psz1
		while(nLen1>0)
		{
			if(psz1[nLen1-1]==' ')	psz1[--nLen1] = '\0';
			else break;
		}*/
	}

	if(psz2!=NULL)
	{
		nLen2 = strlen(psz2);

		//strip trailing spaces from psz2 
/*		while(nLen2>0)
		{
			if(psz2[nLen2-1]==' ')	psz2[--nLen2] = '\0';
			else break;
		}*/
	}
			
	char *pszNew = NULL;
	if(nLen2)
	{
		pszNew = (char*)malloc(nLen1+nLen2+2);
		strcpy(pszNew,psz1);
		strcat(pszNew,"\n");
		strcat(pszNew,psz2);

		free(psz1); psz1=NULL;
	}
	else
		pszNew = psz1;
	
	free(psz2); psz2=NULL;

	return pszNew;
}

