/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CSerialization.cpp: implementation of the CCSerialization class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CSerialization.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAX_DATA_SERIALIZE 10000
#define NO_OBJECT 0
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------
//---------             Structure                  -----------------//
//--------------------------------------------------------------------

CCPersistentObject::CCPersistentObject()
{
    Init() ;
}

//--------------------------------------------------------------------

CCPersistentObject::~CCPersistentObject()
{
    m_ArrayValue.RemoveAll() ;
}

//----------------------------------------------------------------------

void CCPersistentObject::Init()
{
    m_strLocation.Empty() ;
    m_nID = NO_OBJECT ;
    m_ArrayValue.RemoveAll() ;
    m_nArrayValueIndex = 0 ;
    m_pNext = NULL ;
}

//---------------------------------------------------------------------

BOOL CCPersistentObject::GetBool()
{
    BOOL bRet = TRUE ;
    //---
    bRet = (BOOL) atoi( m_ArrayValue[m_nArrayValueIndex] ) ;
    m_nArrayValueIndex ++ ;
    return bRet ;
}

//---------------------------------------------------------------------

void CCPersistentObject::SetBool( BOOL bSet )
{
    TCHAR pczBuffer[10] ;
    //--
    itoa( bSet , pczBuffer , 10 ) ;
    if( m_nArrayValueIndex >= m_ArrayValue.GetSize() )
        m_ArrayValue.Add( (CString) pczBuffer ) ;
    else
        m_ArrayValue[m_nArrayValueIndex]  = pczBuffer ;
    m_nArrayValueIndex ++ ;
}

//---------------------------------------------------------------------

CString CCPersistentObject::GetString()
{
    CString strValue;
    strValue = m_ArrayValue[m_nArrayValueIndex] ;
    m_nArrayValueIndex ++ ;

    return strValue;
}

//---------------------------------------------------------------------

void CCPersistentObject::SetString( CString strValue )
{
    if( m_nArrayValueIndex >= m_ArrayValue.GetSize() )
        m_ArrayValue.Add( strValue ) ;
    else
        m_ArrayValue[m_nArrayValueIndex]  = strValue ;
    m_nArrayValueIndex ++ ;
}

//---------------------------------------------------------------------

int CCPersistentObject::GetInt()
{
    int nRet ;
    //---
    nRet = atoi( m_ArrayValue[m_nArrayValueIndex] ) ;
    m_nArrayValueIndex ++ ;
    return nRet ;
}

//---------------------------------------------------------------------

void CCPersistentObject::SetInt( int nSet )
{
    TCHAR pczBuffer[10] ;
    //--
    itoa( nSet , pczBuffer , 10 ) ;
    if( m_nArrayValueIndex >= m_ArrayValue.GetSize() )
        m_ArrayValue.Add( (CString)pczBuffer ) ;
    else
        m_ArrayValue[m_nArrayValueIndex]  = pczBuffer ;
    m_nArrayValueIndex ++ ;
}

//---------------------------------------------------------------------

float CCPersistentObject::GetFloat()
{
    float fRet ;
    //---
    fRet = (float)atof( m_ArrayValue[m_nArrayValueIndex] ) ;
    m_nArrayValueIndex ++ ;
    return fRet ;
}

//---------------------------------------------------------------------

void CCPersistentObject::SetFloat( float fSet )
{
    CString strValue ;
    //--
    strValue.Format("%f", fSet ) ;
    if( m_nArrayValueIndex >= m_ArrayValue.GetSize() )
        m_ArrayValue.Add( strValue ) ;
    else
        m_ArrayValue[m_nArrayValueIndex]  = strValue ;
    m_nArrayValueIndex ++ ;
}

//---------------------------------------------------------------------

long CCPersistentObject::GetLong()
{
    long lRet ;
    //---
    lRet = atol( m_ArrayValue[m_nArrayValueIndex] ) ;
    m_nArrayValueIndex ++ ;
    return lRet ;
}

//---------------------------------------------------------------------

void CCPersistentObject::SetLong( long lSet )
{
    TCHAR pczBuffer[100] ;
    //--
    ltoa( lSet , pczBuffer , 10 ) ;
    if( m_nArrayValueIndex >= m_ArrayValue.GetSize() )
        m_ArrayValue.Add( (CString) pczBuffer ) ;
    else
        m_ArrayValue[m_nArrayValueIndex]  = pczBuffer ;
    m_nArrayValueIndex ++ ;
}

//---------------------------------------------------------------------

void CCPersistentObject::ResetEnum()
{
    m_nArrayValueIndex = 0 ;
}

//---------------------------------------------------------------------

BOOL CCPersistentObject::Read()
{
    TCHAR pczBuffer[10] ;
    CString strValues ;
    //---
    ASSERT( !m_strLocation.IsEmpty() ) ;
    ASSERT( m_nID != NO_OBJECT ) ;
    if( m_nID == NO_OBJECT )
        return FALSE;

    //---
    itoa( m_nID , pczBuffer , 10 ) ;
    strValues = GetExtendedProfileString( HKEY_CURRENT_USER,
                                          m_strLocation ,
                                          pczBuffer ,
                                          _T("") ) ;

    return FillValues( strValues ) ;
}

//---------------------------------------------------------------------

BOOL CCPersistentObject::FillValues( const CString& strValues )
{
    CString strTemp ;
    CString strRight = strValues ;
    int nSeparator ;
    //---
    if( strRight.IsEmpty() )
        return FALSE ;

    nSeparator = strRight.Find("|")  ;
    strTemp = strRight.Left( nSeparator ) ;
    strRight = strRight.Right( strRight.GetLength() - nSeparator - 1 ) ;

    while ( nSeparator != -1 )
    {
        m_ArrayValue.Add( strTemp ) ;
        nSeparator = strRight.Find("|")  ;
        strTemp = strRight.Left( nSeparator ) ;
        strRight = strRight.Right( strRight.GetLength() - nSeparator - 1 ) ;
    }

    return TRUE ;
}

//---------------------------------------------------------------------

void CCPersistentObject::SaveValues( CString &strStoreValues )
{
    int nCount = m_ArrayValue.GetSize() ;
    //---
    strStoreValues.Empty() ;

    for( int nIndex = 0 ; nIndex < nCount ; nIndex++ )
    {
        strStoreValues += m_ArrayValue[nIndex] ;
        strStoreValues += "|" ;
    }
}

//---------------------------------------------------------------------

void CCPersistentObject::Save()
{
    TCHAR pczBuffer[10] ;
    CString strStoreValues ;

    //---
    ASSERT( !m_strLocation.IsEmpty() ) ;
    ASSERT( m_nID != NO_OBJECT ) ;
    if( m_nID == NO_OBJECT )
        return ;

    //---
    SaveValues( strStoreValues ) ;
    itoa( m_nID , pczBuffer , 10 ) ;
    WriteExtendedProfileString(  HKEY_CURRENT_USER ,
                                 (LPCSTR)m_strLocation ,        // address of section name
                                 pczBuffer,        // address of key name
                                 (LPCSTR)strStoreValues ) ;// address of destination buffer
}

//--------------------------------------------------------------------
//---------             Serialization object       -----------------//
//--------------------------------------------------------------------


CCSerialization::CCSerialization()
        :m_pFirstElt( NULL )
{
}

//--------------------------------------------------------------------


CCSerialization::~CCSerialization()
{
    CCPersistentObject *elt1 = m_pFirstElt ;
    CCPersistentObject *elt2 ;
    while ( elt1 )
    {
        elt2 = elt1 ;
        elt1 = elt1->GetNext() ;
        delete elt2 ;
    }

}

//--------------------------------------------------------------------

void CCSerialization::SetLocation( LPCSTR pczLocation )
{
    DWORD dw;
    HKEY    hSoftKey;

    m_strLocation = pczLocation ;
    ASSERT( pczLocation != NULL && !m_strLocation.IsEmpty() ) ;


    ::RegCreateKeyEx(HKEY_CURRENT_USER, m_strLocation, 0, REG_NONE,
                     REG_OPTION_NON_VOLATILE, KEY_WRITE|KEY_READ, NULL,
                     &hSoftKey, &dw);
    //Close key
    ::RegCloseKey(HKEY_CURRENT_USER);

    //open registry key : if m_hKey == NULL ->key doesnot exist
    if(::RegOpenKey(  HKEY_CURRENT_USER,        // handle to open key
                      m_strLocation, // address of name of subkey to open
                      &m_hKey ) != ERROR_SUCCESS )   // address of handle to open key);
        MessageBox( NULL , _T("The RegOpenKey Failed !") , _T("Warning") , MB_OK ) ;
}

//--------------------------------------------------------------------

void CCSerialization::Read()
{
    int nIndex = 0 ;
    int nSuccess = ERROR_SUCCESS ;
    //---
    ASSERT( !m_strLocation.IsEmpty()  ) ;

    //read for each entry its value and stock it in the chained list
    while( m_hKey && nSuccess != ERROR_NO_MORE_ITEMS )
    {
        TCHAR pczBufferValueKey[MAX_DATA_SERIALIZE] ;
        DWORD dwBufferValueKeySize = MAX_DATA_SERIALIZE ;
        DWORD dwBufferValueSize = MAX_DATA_SERIALIZE ;
        DWORD dwType;

        //---
        nSuccess =   ::RegEnumValue(  m_hKey,              // handle to key to query
                                      nIndex,          // index of value to query
                                      pczBufferValueKey,     // address of buffer for value string
                                      &dwBufferValueKeySize ,  // address for size of value buffer
                                      NULL ,     // reserved
                                      &dwType,         // address of buffer for type code
                                      NULL,          // address of buffer for value data
                                      NULL ) ;        // address for size of data buffer);

        if ( nSuccess == ERROR_NO_MORE_ITEMS )
            continue;

        if( dwType == REG_SZ)
        {
            int nID = atoi( pczBufferValueKey ) ;
            if( nID != 0 )
            {
                if( m_pFirstElt == NULL )
                {
                    m_pFirstElt = new CCPersistentObject() ;
                    m_pFirstElt->SetLocation( m_strLocation ) ;
                    m_pFirstElt->SetID( nID ) ;
                    m_pFirstElt->Read();
                    m_pFirstElt->SetNext( NULL );
                }
                else
                {
                    CCPersistentObject *elt = new CCPersistentObject() ;
                    elt->SetLocation( m_strLocation ) ;
                    elt->SetID( nID ) ;
                    elt->Read();
                    elt->SetNext( m_pFirstElt );
                    m_pFirstElt = elt;
                }
            }
        }
        nIndex++;
    }
}

//--------------------------------------------------------------------
//save data
void CCSerialization::Save()
{
    ASSERT( !m_strLocation.IsEmpty() ) ;

    CCPersistentObject *elt = m_pFirstElt ;
    //----

    while( elt )
    {
        elt->Save() ;
        elt = elt->GetNext() ;
    }

}

//--------------------------------------------------------------------
//find an object according to ID
CCPersistentObject *CCSerialization::GetObject( int nID )
{
    CCPersistentObject *elt = m_pFirstElt ;
    //---
    while( elt )
    {
        if( elt->GetID() == nID )
            return elt ;
        else
            elt = elt->GetNext() ;
    }
    return NULL ;
}

//--------------------------------------------------------------------
//Add an object
CCPersistentObject *CCSerialization::AddObject( int nID )
{
    ASSERT( nID != NO_OBJECT ) ;

    CCPersistentObject *elt = new CCPersistentObject() ;

    elt->SetLocation( m_strLocation ) ;
    elt->SetID( nID ) ;
    elt->SetNext( NULL );
    if( m_pFirstElt == NULL  )
    {
        m_pFirstElt  = elt ;
    }
    else
    {
        elt->SetNext( m_pFirstElt ) ;
        m_pFirstElt = elt ;
    }

    return elt ;
}


//Extended Registry function

//---------------------------------------------------------------------

BOOL WriteExtendedProfileString(HKEY _hKey, LPCTSTR _lpSubKey,
                                LPCTSTR _szEntry, LPCTSTR _pszValue)
{
    LONG    lResult;
    DWORD dw;
    HKEY    hSoftKey;

    //create key or open it if already exist according to _lpSubKey
    ::RegCreateKeyEx(_hKey, _lpSubKey, 0, REG_NONE,
                     REG_OPTION_NON_VOLATILE, KEY_WRITE|KEY_READ, NULL,
                     &hSoftKey, &dw);
    //Close key
    ::RegCloseKey(_hKey);

    //Open key since it exist
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS)
    {
        //Set value _pszValue to _szEntry
        lResult = ::RegSetValueEx(hSoftKey, _szEntry, 0, REG_SZ, (BYTE * const)_pszValue, (lstrlen(_pszValue)+1)*sizeof(TCHAR)) ;
        //Close key
        ::RegCloseKey(hSoftKey);
    }

    return (lResult == ERROR_SUCCESS);
}

//---------------------------------------------------------------------

CString GetExtendedProfileString(HKEY _hKey, LPCTSTR _lpSubKey,
                                 LPCTSTR _lpszEntry , LPCTSTR _szDefault )
{
    LONG    lResult;
    DWORD   dwType;
    DWORD   dwCount;
    HKEY    hSoftKey;
    CString strValue ;
    //-----
    //open key
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS)
    {
        // test if the data exist
        lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_lpszEntry, NULL, &dwType,
                                  NULL, &dwCount);
        if (lResult == ERROR_SUCCESS)
        {
            ASSERT(dwType == REG_SZ);
            //get the data
            lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_lpszEntry, NULL, &dwType,
                                      (LPBYTE)strValue.GetBuffer(dwCount/sizeof(TCHAR)), &dwCount);
            strValue.ReleaseBuffer();
        }
        //close key
        RegCloseKey(hSoftKey);

        if (lResult == ERROR_SUCCESS)
        {
            ASSERT(dwType == REG_SZ);
            return strValue;
        }
    }

    return _szDefault ;
}

//---------------------------------------------------------------------

LPBYTE GetExtendedProfileBinary(HKEY _hKey, LPCTSTR _lpSubKey,
                                LPCTSTR _lpszEntry , LPBYTE _pDefault )
{
    LONG    lResult;
    DWORD   dwType;
    DWORD   dwCount;
    HKEY    hSoftKey;
    LPBYTE  pValue = NULL ;
    //-----
    //open key
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS)
    {
        // test if the data exist
        lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_lpszEntry, NULL, &dwType,
                                  NULL, &dwCount);
        if (lResult == ERROR_SUCCESS)
        {
            ASSERT(dwType == REG_BINARY);
            //get the data
            pValue = new BYTE[ dwCount] ;
            lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_lpszEntry, NULL, &dwType,
                                      (LPBYTE)pValue, &dwCount);
        }
        //close key
        RegCloseKey(hSoftKey);

        if (lResult == ERROR_SUCCESS)
        {
            ASSERT(dwType == REG_BINARY);
            return pValue ;
        }
    }

    return _pDefault ;
}

//---------------------------------------------------------------------

BOOL WriteExtendedProfileBinary(HKEY _hKey, LPCTSTR _lpSubKey,
                                LPCTSTR _szEntry, LPBYTE _pValue , int nSize )
{
    LONG    lResult;
    DWORD dw;
    HKEY    hSoftKey;

    //create key or open it if already exist according to _lpSubKey
    ::RegCreateKeyEx(_hKey, _lpSubKey, 0, REG_NONE,
                     REG_OPTION_NON_VOLATILE, KEY_WRITE|KEY_READ, NULL,
                     &hSoftKey, &dw);
    //Close key
    ::RegCloseKey(_hKey);

    //Open key since it exist
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS)
    {
        //Set value _pszValue to _szEntry
        lResult = ::RegSetValueEx(hSoftKey, _szEntry, 0, REG_BINARY, (BYTE * const)_pValue, sizeof(BYTE)*nSize ) ;
        //Close key
        ::RegCloseKey(hSoftKey);
    }

    return (lResult == ERROR_SUCCESS);
}

//---------------------------------------------------------------------

BOOL WriteExtendedProfileDword(HKEY _hKey, LPCTSTR _lpSubKey,
                               LPCTSTR _szEntry, DWORD dwValue )
{
    LONG    lResult;
    DWORD dw;
    HKEY    hSoftKey;

    //create key or open it if already exist according to _lpSubKey
    ::RegCreateKeyEx(_hKey, _lpSubKey, 0, REG_NONE,
                     REG_OPTION_NON_VOLATILE, KEY_WRITE|KEY_READ, NULL,
                     &hSoftKey, &dw);
    //Close key
    ::RegCloseKey(_hKey);

    //Open key since it exist
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS)
    {
        //Set value _pszValue to _szEntry
        //---
        lResult = ::RegSetValueEx(hSoftKey, _szEntry, 0, REG_DWORD, (BYTE * const)&dwValue, sizeof(DWORD) ) ;
        //Close key
        ::RegCloseKey(hSoftKey);
    }

    return (lResult == ERROR_SUCCESS);
}



//---------------------------------------------------------------------

DWORD GetExtendedProfileDword(HKEY _hKey, LPCTSTR _lpSubKey,
                              LPCTSTR _szEntry , DWORD dwDefault )
{
    LONG    lResult;
    DWORD   dwType;
    DWORD   dwCount;
    HKEY    hSoftKey;
    DWORD   dwValue ;
    //-----
    //open key
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS)
    {
        // test if the data exist
        lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_szEntry, NULL, &dwType,
                                  NULL, &dwCount);

        if (lResult == ERROR_SUCCESS)
        {
            ASSERT(dwType == REG_DWORD);
            lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_szEntry, NULL,
                                      &dwType ,(BYTE * const)&dwValue, &dwCount);

            if (lResult == ERROR_SUCCESS)
            {
                ASSERT(dwType == REG_DWORD);
                return dwValue ;
            }
        }
        //close key
        RegCloseKey(hSoftKey);
    }

    return dwDefault ;
}

