/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RecoverManager.cpp: implementation of the CRecoverManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "concertlib.h"
#include "RecoverManager.h"
#include "ConstantesLib.h"
#include "IE4Information.h"
#include "IE5Information.h"
#include "IE6Information.h"
#include "NS47Information.h"
#include "RecovererDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRecoverManager::CRecoverManager()
{
}

//-------------------------------------------------------------------


CRecoverManager::~CRecoverManager()
{
}

//-------------------------------------------------------------------


BOOL CRecoverManager::Show()
{
    BOOL                bDeleteOld ;
    BOOL                bNoRecoveryDone ;
    CString strBackupFileName ;
    //---
    bDeleteOld      = FALSE ;
    bNoRecoveryDone = TRUE ;

    strBackupFileName = GetBackupFileFullPath( STR_MODELLER_IE4 ) ;
    if( FileExist( strBackupFileName ) )
    {
        //---
        CRecovererDlg       RecoverDlg( NULL, STR_FULLNAME_BROWSER_IE4, &bDeleteOld ) ;

        if( RecoverDlg.DoModal() == IDOK )
        {
            CIE4Information     BrowserIE4(&RecoverDlg);
            Recover( &BrowserIE4, strBackupFileName, bDeleteOld ) ;
            bNoRecoveryDone = FALSE ;
        }
    }

    strBackupFileName = GetBackupFileFullPath( STR_MODELLER_IE5 ) ;
    if( FileExist( strBackupFileName ) )
    {
        //---
        CRecovererDlg       RecoverDlg( NULL, STR_FULLNAME_BROWSER_IE5, &bDeleteOld ) ;
        if( RecoverDlg.DoModal() == IDOK )
        {
            CIE5Information     BrowserIE5(&RecoverDlg) ;
            Recover( &BrowserIE5, strBackupFileName, bDeleteOld ) ;
            bNoRecoveryDone = FALSE ;
        }
    }

    strBackupFileName = GetBackupFileFullPath( STR_MODELLER_IE6 ) ;
    if( FileExist( strBackupFileName ) )
    {
        //---
        CRecovererDlg       RecoverDlg( NULL, STR_FULLNAME_BROWSER_IE6, &bDeleteOld ) ;
        if( RecoverDlg.DoModal() == IDOK )
        {
            CIE6Information     BrowserIE6(&RecoverDlg) ;
            Recover( &BrowserIE6, strBackupFileName, bDeleteOld ) ;
            bNoRecoveryDone = FALSE ;
        }
    }

    strBackupFileName = GetBackupFileFullPath( STR_MODELLER_NS47 ) ;
    if( FileExist( strBackupFileName ) )
    {
        //---
        CRecovererDlg       RecoverDlg( NULL, STR_FULLNAME_BROWSER_NS47, &bDeleteOld ) ;
        if( RecoverDlg.DoModal() == IDOK )
        {
            CNS47Information    BrowserNS47(&RecoverDlg) ;
            Recover( &BrowserNS47, strBackupFileName, bDeleteOld ) ;
            bNoRecoveryDone = FALSE ;
        }
    }

    return bNoRecoveryDone ;
}

//-------------------------------------------------------------------

CString CRecoverManager::GetBackupFileFullPath( CString _strFile )
{
    TCHAR   *pczTempDir[_MAX_PATH] ;
    CString strFileName ;
    CFile   File ;
    //----
    ::GetTempPath( _MAX_PATH - 1 , (char *)pczTempDir ) ;
    strFileName = (char*)pczTempDir ;
    strFileName += _strFile ;

    return strFileName ;
}

//-------------------------------------------------------------------

BOOL CRecoverManager::FileExist( CString _strFile )
{
    CFile   File ;
    //----
    if( !File.Open( _strFile, CFile::modeRead ) )
    {
        return FALSE ;
    }

    File.Close() ;

    return TRUE ;
}

//-------------------------------------------------------------------

void CRecoverManager::Recover( CBrowserInformation * _pBrowserInfo, CString _strBackupFile, BOOL _bDeleteOld )
{
    _pBrowserInfo->SetDefaultParametersInRegistry() ;
    if( _bDeleteOld )
    {
        DeleteFile( _strBackupFile ) ;
    }
}
