/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Tokenizer.cpp: implementation of the CTokenizer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Tokenizer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//---------------------------------------------------------------------------

CTokenizer::CTokenizer(CString _strValue, CString _strSeparator)
        :   m_strValue(_strValue),
        m_strSeparator(_strSeparator)
{
    Reset();
}
//---------------------------------------------------------------------------

CTokenizer::CTokenizer(CString _strValue, CString _strSeparatorBegin,
                       CString _strSeparatorEnd)
        :   m_strValue(_strValue),
        m_strSeparator(_strSeparatorBegin),
        m_strSeparatorEnd(_strSeparatorEnd)
{
    Reset();
}

//---------------------------------------------------------------------------

CTokenizer::CTokenizer(CString _strSeparator)
        :   m_strSeparator(_strSeparator)
{
    Clear();
}

//---------------------------------------------------------------------------

CTokenizer::~CTokenizer()
{}

//---------------------------------------------------------------------------


void CTokenizer::Clear()
{
    m_strValue.Empty();
    m_strTempValue.Empty();
}

//---------------------------------------------------------------------------

void CTokenizer::Add(CString _strToken)
{
    m_strValue += _strToken;
    m_strValue += m_strSeparator;
}

//---------------------------------------------------------------------------

void CTokenizer::Add(long _lToken)
{
    CString strTemp;

    strTemp.Format("%ld", _lToken);
    Add(strTemp);
}

//---------------------------------------------------------------------------

CString CTokenizer::GetResult()
{
    return m_strValue;
}

//---------------------------------------------------------------------------

void CTokenizer::Reset()
{
    m_strTempValue = m_strValue;
}

//---------------------------------------------------------------------------

long CTokenizer::GetLong()
{
    long    lResult;
    CString strLeft;

    strLeft = GetString();
    lResult = atol(strLeft);

    return lResult;
}

//---------------------------------------------------------------------------

CString CTokenizer::GetString()
{
    int     nLeft;
    CString strLeft;

    nLeft       = m_strTempValue.Find(m_strSeparator) ;
    strLeft     = m_strTempValue.Left( nLeft ) ;
    m_strTempValue  = m_strTempValue.Right( m_strTempValue.GetLength() - nLeft - m_strSeparator.GetLength() ) ;

    return strLeft;
}

//---------------------------------------------------------------------------

BOOL CTokenizer::Exist()
{
    return ( !m_strTempValue.IsEmpty() );
}


