/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// CORBAInfo.cpp: implementation of the CCORBAInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "testplugin.h"
#include "HTTPInfo.h"
#include <string>
#include <vector>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
UINT CHTTPInfo::GetUpdate(LPVOID pParam)
{
	try
	{
		CHTTPInfo* pInfo = (CHTTPInfo*)pParam;
		CORBAServer* pCorba = new CORBAServer ( 0, NULL );;

		CORBA::Object_var obj;
		obj=pCorba->FindObjectWithName(TestManager::CLASS_NAME, pInfo->m_strManagerReg); 
	    TestManager_var TM=TestManager::_narrow(obj.inout());

		if (!CORBA::is_nil(TM))
		{
			BOOL bComplete = false;
			while (!bComplete || pInfo->m_bActiveWindows > 0)
			{
				TaskIdList taskIDList;
				taskIDList.length(pInfo->m_taskIDList.size());
				for (int i =0; i<pInfo->m_taskIDList.size(); i++)
				{
					taskIDList[i] = pInfo->m_taskIDList[i];
				}

				TaskSnap2List snapList;
				TM->GetTaskSnapshot(taskIDList, snapList);
				pInfo->m_allVUSnaps = snapList;

				if (WaitForSingleObject(pInfo->m_hEndThreadEvent, pInfo->GetRefreshRate()) != WAIT_TIMEOUT)
				{
					bComplete = true;
				}

			}
		}

		delete pCorba;
	}
	catch(...)
	{
	}
	return 0;
}

CHTTPInfo::CHTTPInfo()
{
	m_strManagerReg = "";
	m_pUpdateThread = NULL;
	m_bActiveWindows = 0;
}

CHTTPInfo::~CHTTPInfo()
{
}


void CHTTPInfo::SetManagerReg(CString strReg)
{
	m_strManagerReg = strReg;
}

void CHTTPInfo::AddTaskId(TaskId taskID)
{
	m_taskIDList.push_back(taskID);
}

void CHTTPInfo::RemoveTaskId(TaskId taskID)
{
	for (int i =0; i<m_taskIDList.size(); i++)
	{
		if (m_taskIDList[i].lTaskGroupId == taskID.lTaskGroupId &&
			m_taskIDList[i].lTaskId == taskID.lTaskId)
		{
			m_taskIDList.erase(&m_taskIDList[i]);
			break;
		}
	}
}

void CHTTPInfo::StartCollector()
{
	m_bActiveWindows++;
	if (!m_pUpdateThread)
	{
		m_hEndThreadEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
		m_pUpdateThread = AfxBeginThread(&GetUpdate,this);
	}
}

void CHTTPInfo::StopCollector()
{
	m_bActiveWindows--;

	if(m_bActiveWindows == 0)
	{
		if(m_pUpdateThread)
		{
			SetEvent(m_hEndThreadEvent);
			m_pUpdateThread = NULL;

			TaskSnap2List snapList;
			m_allVUSnaps = snapList;
		}
	}
}

int CHTTPInfo::GetRefreshRate()
{
	int nRefreshSecs = 10;
	HKEY hKey;
	if(RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\OpenSTA\\TestPlugin\\Settings",0,KEY_READ,&hKey)!=ERROR_SUCCESS)
		return nRefreshSecs*1000;

	DWORD dwInterval;
	DWORD dwDataSize = sizeof(DWORD);
	if(RegQueryValueEx(hKey,"Task Monitoring Interval",NULL,NULL,(LPBYTE)&dwInterval,&dwDataSize)== ERROR_SUCCESS)
	{
		nRefreshSecs = dwInterval;
	}
	RegCloseKey(hKey);

	if (nRefreshSecs < 1)
		nRefreshSecs = 10;

	// return as millisecs
	return nRefreshSecs*1000;
}
