/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLDocInfo.cpp: implementation of the CSCLDocInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <CSerialization.h>
#include <Solutions.h>
#include <ConstantesLib.h>
#include "Constantes.h"
#include "SCLDocInfo.h"
#include "RegFunctions.h"

//---------------------------------------------------------------------------

CSCLDocInfo::CSCLDocInfo()
{
    m_lBrowserID = ID_BROWSER_NONE;
}

//---------------------------------------------------------------------------

CSCLDocInfo::~CSCLDocInfo()
{
}

//---------------------------------------------------------------------------


void CSCLDocInfo::SetProjectName(CString _strProjectName)
{
    m_strProjectName = _strProjectName;
}


//---------------------------------------------------------------------------

void CSCLDocInfo::SetFileInfo(CString _strSCLFileName, long _lBrowserID)
{
    m_strSCLFileName = _strSCLFileName;
    m_lBrowserID = _lBrowserID;
}

//---------------------------------------------------------------------------

void CSCLDocInfo::SetBrowserID(long _lBrowserID)
{
    m_lBrowserID = _lBrowserID;
}

//---------------------------------------------------------------------------

long   CSCLDocInfo::GetBrowserID()
{
    ASSERT( m_lBrowserID != ID_BROWSER_NONE );
    return m_lBrowserID;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetProjectName()
{
    ASSERT( !m_strProjectName.IsEmpty() );
    return m_strProjectName;
}

//---------------------------------------------------------------------------

BOOL CSCLDocInfo::IsProjectNameEmpty()
{
    return m_strProjectName.IsEmpty();
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetSCLName()
{
    ASSERT( !m_strSCLFileName.IsEmpty() );
    int nPos  = m_strSCLFileName.ReverseFind('.');

    if (  nPos < 0 )
    {
        ASSERT(FALSE);
        return _T("");
    }
    return m_strSCLFileName.Left(nPos) ;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetSCLFileName()
{
    ASSERT( !m_strSCLFileName.IsEmpty() );
    return m_strSCLFileName;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetTOFFileName()
{
    return GetSCLName()+ STR_TOF_EXTENSION;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetALLFileName()
{
    return GetSCLName()+ STR_ALL_EXTENSION;
}


//---------------------------------------------------------------------------

CString CSCLDocInfo::GetProjectPath()
{
    CString strProjectPath;

    strProjectPath = GetRepositoryPath();

    if( strcmp( strProjectPath.Right( 1 ) , "\\" ) != 0 )
        strProjectPath += "\\" ;

    GetShortPathName( (LPCSTR)strProjectPath,
                      strProjectPath.GetBuffer(_MAX_PATH),
                      _MAX_PATH);

    strProjectPath.ReleaseBuffer();

    return strProjectPath;

}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetTmpProjectPath()
{
    CString strTmpProjectPath;

    strTmpProjectPath = GetProjectPath() ;
    strTmpProjectPath+= STR_TMP ;
    strTmpProjectPath+= "\\" ;

    //create the directory if do not exists
    SECURITY_ATTRIBUTES SecurityAttributes ;

    SecurityAttributes.nLength = sizeof( SECURITY_ATTRIBUTES ) ;
    SecurityAttributes.lpSecurityDescriptor = NULL ;

    CreateDirectory(  strTmpProjectPath  , &SecurityAttributes );

    GetShortPathName( (LPCSTR)strTmpProjectPath,
                      strTmpProjectPath.GetBuffer(_MAX_PATH),
                      _MAX_PATH);

    strTmpProjectPath.ReleaseBuffer();

    return strTmpProjectPath;

}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetSCLPath()
{
    CString strSCLPath = GetWorkingFolder();

    GetShortPathName( (LPCSTR)strSCLPath,
                      strSCLPath.GetBuffer(_MAX_PATH),
                      _MAX_PATH);

    strSCLPath.ReleaseBuffer();

    return strSCLPath;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetTOFPath()
{
    return GetSCLPath();
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetALLPath()
{
    CString strAllPath = GetWorkingFolder();

    GetShortPathName( (LPCSTR)strAllPath,
                      strAllPath.GetBuffer(_MAX_PATH),
                      _MAX_PATH);

    strAllPath.ReleaseBuffer();

    return strAllPath;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetFVRPath()
{
    CString strFVRPath = GetWorkingFolder();

    GetShortPathName( (LPCSTR)strFVRPath,
                      strFVRPath.GetBuffer(_MAX_PATH),
                      _MAX_PATH);

    strFVRPath.ReleaseBuffer();

    return strFVRPath;
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetFullSCLFileName()
{
    return GetSCLPath() + GetSCLFileName();
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetFullTOFFileName()
{
    return GetTOFPath() + GetTOFFileName();
}

//---------------------------------------------------------------------------

CString CSCLDocInfo::GetFullALLFileName()
{
    return GetALLPath() + GetALLFileName();
}

