/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SelectBrowserDlg.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "SelectBrowserDlg.h"
#include <ConstantesLib.h>
#include "htmlhelp.h"
#include "Help\SelectBrowserDlg.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSelectBrowserDlg dialog


CSelectBrowserDlg::CSelectBrowserDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CSelectBrowserDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CSelectBrowserDlg)
	m_strPrefsPath = _T("");
	//}}AFX_DATA_INIT
    m_nBrowserID =0;
}


void CSelectBrowserDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CSelectBrowserDlg)
	DDX_Control(pDX, IDC_PREFS_LOCATION, m_ctlPrefsLocation);
	DDX_Control(pDX, IDC_BROWSEPREFS, m_ctlBrowsePrefs);
	DDX_Control(pDX, IDC_PREFSPATH_FRAME, m_ctlPrefsFrame);
    DDX_Control(pDX, IDC_BROWSERS, m_ctlBrowsers);
	DDX_Text(pDX, IDC_PREFS_LOCATION, m_strPrefsPath);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CSelectBrowserDlg, CDialog)
//{{AFX_MSG_MAP(CSelectBrowserDlg)
ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_BROWSEPREFS, OnBrowseprefs)
	ON_CBN_SELCHANGE(IDC_BROWSERS, OnSelchangeBrowsers)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSelectBrowserDlg message handlers

BOOL CSelectBrowserDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    m_ctlBrowsers.AddString("Internet Explorer 4");
    m_ctlBrowsers.AddString("Internet Explorer 5");
    m_ctlBrowsers.AddString("Internet Explorer 6");
    m_ctlBrowsers.AddString("Netscape");

    int nSel = 0;
    switch(m_nBrowserID)
    {
    case ID_BROWSER_IE4:
        nSel = 0;
        break;
    case ID_BROWSER_IE5:
        nSel = 1;
        break;
    case ID_BROWSER_IE6:
        nSel = 2;
        break;
    case ID_BROWSER_NS47:
        nSel = 3;
        break;
    }
    m_ctlBrowsers.SetCurSel(nSel);
    
	m_strPrefsPath = GetExtendedProfileString( HKEY_CURRENT_USER , STR_MODELLER_KEY,
                          STR_MDL_NETSCAPE_ENTRY , _T("") ) ;

	OnSelchangeBrowsers();

	UpdateData(FALSE);
	return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CSelectBrowserDlg::OnOK()
{
	UpdateData();

    int nSel =m_ctlBrowsers.GetCurSel();
    switch(nSel)
    {
    case 0:
        m_nBrowserID = ID_BROWSER_IE4;
        break;
    case 1:
        m_nBrowserID = ID_BROWSER_IE5;
        break;
    case 2:
        m_nBrowserID = ID_BROWSER_IE6;
        break;
    case 3:
        m_nBrowserID = ID_BROWSER_NS47;
        break;
    }

	if(m_nBrowserID == ID_BROWSER_NS47)
	{
		if(m_strPrefsPath.IsEmpty())
		{
			AfxMessageBox("Please enter the path to your Netscape preferences file");
			return;
		}
		
		CFileStatus fs;
		if(!CFile::GetStatus(m_strPrefsPath,fs))
		{
			AfxMessageBox("The Netscape preferences file specified does not exist.");
			return;
		}
	}

	WriteExtendedProfileString( HKEY_CURRENT_USER, STR_MODELLER_KEY,
                        STR_MDL_NETSCAPE_ENTRY, m_strPrefsPath);

    CDialog::OnOK();
}

BOOL CSelectBrowserDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CSelectBrowserDlg::OnBrowseprefs() 
{
    DWORD dwFlags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST|OFN_HIDEREADONLY;

    CFileDialog FileDlg(TRUE,"js","prefs.js",dwFlags,NULL,this);
    if (FileDlg.DoModal() == IDOK)
    {
        m_strPrefsPath = FileDlg.GetPathName();
		UpdateData(FALSE);
    }	
}

void CSelectBrowserDlg::OnSelchangeBrowsers() 
{
	int nCurSel = m_ctlBrowsers.GetCurSel();
	BOOL bEnablePrefs = (nCurSel == ID_BROWSER_NS47);
	
	m_ctlPrefsFrame.EnableWindow(bEnablePrefs);
	m_ctlPrefsLocation.EnableWindow(bEnablePrefs);
	m_ctlBrowsePrefs.EnableWindow(bEnablePrefs);
}
