/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarDBReqPPage.cpp : implementation file
//

#include "stdafx.h"
#include "VarDBReqPPage.h"
#include "ConstantesLib.h"
#include "resource.h"
#include "htmlhelp.h"
#include "Help\VarDBReqPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarDBReqPPage, CPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarDBReqPPage, CPropertyPage)
//{{AFX_MSG_MAP(CVarDBReqPPage)
ON_BN_CLICKED(ID_TABLE_RADIO, OnTableRadio)
ON_BN_CLICKED(ID_REQUEST_RADIO, OnRequestRadio)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CVarDBReqPPage::CVarDBReqPPage(CVarDBParam* _pVarDBParam)
        :   CPropertyPage(CVarDBReqPPage::IDD)
{
    m_pVarDBParam = _pVarDBParam ;
}

//---------------------------------------------------------------------------

CVarDBReqPPage::~CVarDBReqPPage()
{}

//---------------------------------------------------------------------------


BOOL CVarDBReqPPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    Values2Ctrl() ;

    return TRUE ;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarDBReqPPage::Ctrl2Values()
{
    CButton * pBut ;
    pBut = static_cast<CButton*>(GetDlgItem(ID_REQUEST_RADIO)) ;

    m_pVarDBParam->m_lReqType =
        pBut->GetCheck() ? VAR_DBSOURCE_REQUEST : VAR_DBSOURCE_TABLE ;

    GetDlgItemText(ID_REQ_EDIT, m_pVarDBParam->m_strReqSource) ;
}

//---------------------------------------------------------------------------

void CVarDBReqPPage::Values2Ctrl()
{
    CButton * pBut ;
    CWnd    * pEdit ;
    bool    bQueryAsSource ;

    bQueryAsSource = m_pVarDBParam->m_lReqType == VAR_DBSOURCE_REQUEST ;

    pBut = static_cast<CButton*>(GetDlgItem(ID_TABLE_RADIO)) ;
    pBut->SetCheck(!bQueryAsSource) ;

    pBut = static_cast<CButton*>(GetDlgItem(ID_REQUEST_RADIO)) ;
    pBut->SetCheck(bQueryAsSource) ;

    pEdit = static_cast<CWnd*>(GetDlgItem(ID_REQ_EDIT)) ;
    pEdit->EnableWindow(bQueryAsSource) ;
    pEdit->SetWindowText( m_pVarDBParam->m_strReqSource ) ;

}

//---------------------------------------------------------------------------

void CVarDBReqPPage::OnTableRadio()
{
    CWnd    * pEdit ;

    pEdit = static_cast<CWnd*>(GetDlgItem(ID_REQ_EDIT)) ;
    pEdit->EnableWindow(FALSE) ;
}

//---------------------------------------------------------------------------

void CVarDBReqPPage::OnRequestRadio()
{
    CWnd    * pEdit ;

    pEdit = static_cast<CWnd*>(GetDlgItem(ID_REQ_EDIT)) ;
    pEdit->EnableWindow(TRUE) ;
}

//---------------------------------------------------------------------------

LRESULT CVarDBReqPPage::OnWizardBack()
{
    return IDD_VAR_DB_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarDBReqPPage::OnWizardNext()
{
    Ctrl2Values() ;

    switch( m_pVarDBParam->m_lReqType )
    {
    case VAR_DBSOURCE_TABLE :
        return IDD_VAR_DBCOLSEL_PPAGE ;
        break ;

    case VAR_DBSOURCE_REQUEST :
        if(IsRequestValid())
        {
            return IDD_VAR_DBROWSEL_PPAGE ;
        }
        else
            return -1 ;
    }

    return CPropertyPage::OnWizardNext() ;
}

//---------------------------------------------------------------------------

BOOL CVarDBReqPPage::IsRequestValid()
{
    CString     strReqSource ;
    CString     strValue ;
    CDBVariant  varValue ;
    CRecordset  rs(&(m_pVarDBParam->m_VarDBSession.m_Database)) ;

    try
    {
        GetDlgItemText(ID_REQ_EDIT, strReqSource) ;

        if(strReqSource.IsEmpty())
            return FALSE ;

        if(!rs.Open( CRecordset::forwardOnly, strReqSource ))
            return FALSE ;

        rs.Close() ;
    }

    catch(CDBException * pDBEx)
    {
        pDBEx->BuildErrorString(&(m_pVarDBParam->m_VarDBSession.m_Database), rs.m_hstmt);
        pDBEx->ReportError() ;
        pDBEx->Delete() ;
        return FALSE ;
    }

    return TRUE ;
}

//---------------------------------------------------------------------------

HBRUSH CVarDBReqPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

BOOL CVarDBReqPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp+="::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
