/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExcelExport.cpp: implementation of the CExcelExport class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ExcelExport.h"
#include "ExcelEnums.h"
#include "XChartCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CExcelExport::CExcelExport():
        covOptional((long)DISP_E_PARAMNOTFOUND, VT_ERROR)
{
    m_pGraph = NULL;
}

CExcelExport::~CExcelExport()
{}


BOOL CExcelExport::SetGraph(CXChartCtrl *pGraph)
{
    if(pGraph == NULL)
        return false;

    m_pGraph = pGraph;

    try
    {
        if(!InitialiseExcel())
            return false;

        if(!SetupSheetData())
        {
            m_app.Quit();
            return false;
        }
    }
    catch(COleException *e)
    {
        CString str;
        str.Format("COleException. SCODE: %08lx.",(long)e->m_sc);
        AfxMessageBox(str,MB_OK);
        return false;
    }
    catch(COleDispatchException *e)
    {
        CString str;
        str.Format("COleDispatchException. SCODE: %08lx, Description: \"%s\".",
                   (long)e->m_wCode,
                   (LPSTR)e->m_strDescription.GetBuffer(1024));
        AfxMessageBox(str, MB_OK);
        return false;
    }
    catch(...)
    {
        AfxMessageBox("General Exception caught.",MB_OK);
        return false;
    }

    return true;
}

BOOL CExcelExport::InitialiseExcel()
{
    // Start Excel and get the Application object.
    if(!m_app.CreateDispatch("Excel.Application"))
    {
        AfxMessageBox("Couldn't start Excel and get an application 0bject");
        return false;
    }

    m_app.SetUserControl(TRUE);
    m_app.SetVisible(TRUE);

    LPDISPATCH lpDisp;

    // Get Workbooks collection.
    lpDisp = m_app.GetWorkbooks();  // Get an IDispatch pointer.
    Workbooks books(lpDisp);

    // Open a new workbook and attach that IDispatch pointer to the
    // Workbook object.
    lpDisp = books.Add( covOptional );
    _Workbook book(lpDisp);

    // Get sheets.
    lpDisp = book.GetSheets();

    Worksheets sheets(lpDisp);
    lpDisp = sheets.GetItem( COleVariant((short)(1)) );

    // Attach the lpDisp pointer to a Worksheet object.
    m_sheet.AttachDispatch(lpDisp,TRUE);

    return true;
}

BOOL CExcelExport::SetupSheetData()
{
    m_pGraph->EditCopy();

    m_sheet.Paste(covOptional,covOptional);
    m_sheet.PasteSpecial(COleVariant("Bitmap"),covOptional,covOptional,covOptional,covOptional,covOptional);

    return TRUE;
}

BOOL CExcelExport::IsExcelRegistered()
{
    CString strSubKey(_T("Software\\Classes\\Excel.Application"));

    HKEY hKey;
    LONG lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,strSubKey,0,KEY_EXECUTE,&hKey);
    if(lResult == ERROR_SUCCESS)
    {
        RegCloseKey(hKey);
        return true;
    }

    return false;
}

