/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// GraphTypes.cpp: implementation of the CTimerGraph class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GraphTypes.h"
#include "XChartCtrl.h"
#include "ExcelExport.h"
#include "FilterDlg.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


extern HINSTANCE m_hInstance;
//////////////////////////////////////////////////////////////////////
// CGraphTypeBase Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGraphTypeBase::CGraphTypeBase(CXChartCtrl *pChart)
{
	m_bHasFilter = false;
    m_nNextGroup = 1;
	m_pChart = pChart;
}

CGraphTypeBase::~CGraphTypeBase()
{}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Initialize the display component

void CGraphTypeBase::SetGraphType(int nType)
{
    for(int nGroup=1;nGroup<=m_nNextGroup;nGroup++)
        m_pChart->SetSeriesStyle(nGroup,nType);
}

int CGraphTypeBase::GetGraphType()
{
    return m_pChart->GetSeriesStyle(1);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Add a legend to the graph
void CGraphTypeBase::DisplayLegend()
{
    //  pLegend->GetStyle()->SetColor(RGB(192,192,192));
}

BOOL CGraphTypeBase::RemoveTestRun(int nDataGroup)
{
    CString strTestRun;
	strTestRun.Format("%d",nDataGroup);

	int nNumGroups=m_nNextGroup;
    for(int nGroup=1;nGroup<=nNumGroups;nGroup++)
	{
		CString str = m_pChart->GetSeriesAnnotation(nGroup);
		if(str == strTestRun)
			m_nNextGroup--;
	}

	m_pChart->RemoveSeries(strTestRun);

    return true;
}

BOOL CGraphTypeBase::ExportToExcel()
{
    CExcelExport ExcelExport;
    return ExcelExport.SetGraph(m_pChart);
}

	void CGraphTypeBase::GetMenuItems(MenuItemArray& arMenuItems)
{
    arMenuItems.RemoveAll();
}

void CGraphTypeBase::OnCommand(UINT uiCommandID,CWnd *pParent)
{}


/*void CGraphTypeBase::AddDataFromGraph(CXChartCtrl *pChart)
{
	VARIANT &var = pChart->m_ctlChart.GetData();
	m_pChart->SetData(var);

	COleSafeArray arData(var);
    long lNumGroups;
    arData.GetUBound(2,&lNumGroups);

    int nFirstGroup = m_nNextGroup;
    m_nNextGroup += lNumGroups;

    for(int nSeries=1;nSeries<=lNumGroups+1;nSeries++)
    {
        int nStyle = pChart->m_ctlChart.GetSeriesStyle(nSeries);
        m_pChart->SetSeriesStyle(nSeries+nFirstGroup-1,nStyle);
		
		CString strText = pChart->m_ctlChart.GetLegendText(nSeries);
		m_pChart->SetLegendText(nSeries+nFirstGroup-1,strText);

        CString str = pChart->m_ctlChart.GetSeriesAnnotation(nSeries);
        m_pChart->SetSeriesAnnotation(nSeries+nFirstGroup-1,str);

        BOOL b = pChart->m_ctlChart.GetSeriesOnSecondaryAxis(nSeries);
       m_pChart->SetSeriesOnSecondaryAxis(nSeries+nFirstGroup-1,b);
    }

}
*/

//////////////////////////////////////////////////////////////////////
// CSNMPGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNMPGraph::CSNMPGraph(CXChartCtrl *pChart):
        CGraphTypeBase(pChart)
{
	m_bHasFilter=true;
    m_nNextGroup=1;

    // Set Axis/Graph styles for an area chart
    pChart->SetXAxisLabel("Elapsed Time (s)");
    pChart->SetYAxisLabel("Query Value");
}

CSNMPGraph::~CSNMPGraph()
{
}


BOOL CSNMPGraph::AddTestRun(SNMPHeaderList* pHeaderList,DWORD dwTestStartTime)
{
	SNMPHeader *pHeader =pHeaderList->GetHead();

	POSITION pos = pHeaderList->GetHeadPosition();
	while(pos)
	{
		SNMPHeader *pHeader =pHeaderList->GetNext(pos);

		if(!pHeader->bInFilter)
			continue;
		
		SNMPStatList& stats = pHeader->listStats;
		if(!stats.GetCount())
			return false;

		DWORD dwTime=0;
		DWORD dwFirstTime = dwTestStartTime;
		DWORD dwLastTime = stats.GetTail().dwStartTime;
		DWORD dwNextTime = dwFirstTime;
		
		DWORD dwDuration = dwLastTime-dwFirstTime+1;
		DWORD dwNumElements[]= {dwDuration , 2};
    
		VARIANT *pVar;
		COleSafeArray saData;
		saData.Create(VT_VARIANT,2,dwNumElements);
		saData.AccessData((void**)&pVar);

		POSITION pos = stats.GetHeadPosition();
		if(!pos)
		{
			pVar[0].vt = VT_BSTR;
			pVar[0].bstrVal = CString().AllocSysString();
			pVar[1].vt = VT_R8;
			pVar[1].bstrVal = 0;
		}

		while(pos)
		{
			SNMPStat& stat = stats.GetNext(pos);

			for(;dwNextTime<stat.dwStartTime;dwNextTime++)
			{
				dwTime = dwNextTime-dwFirstTime;
				VariantInit(&pVar[dwTime+dwDuration]);

				pVar[dwTime].vt = VT_BSTR;
				pVar[dwTime].bstrVal = GetElapsedTimeString(dwTime,dwDuration).AllocSysString();
			}

			dwTime = dwNextTime-dwFirstTime;
			pVar[dwTime+dwDuration].vt = VT_R8;
			pVar[dwTime+dwDuration].dblVal = stat.dValue;

			pVar[dwTime].vt = VT_BSTR;
			pVar[dwTime].bstrVal = GetElapsedTimeString(dwTime,dwDuration).AllocSysString();
			dwNextTime++;
		}

		saData.UnaccessData();

		m_pChart->SetData(saData);
		m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);
		
		CString strText;
		strText.Format("%s (Task Group %d)",pHeader->strQuery,pHeader->dwIP);
		m_pChart->SetLegendText(m_nNextGroup,strText);

		CString strTestRun;
		strTestRun.Format("%d",m_nNextGroup);
		m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
		m_nNextGroup++;
	}	
	
	if(m_nNextGroup==1)
	{
		COleSafeArray saData;
		DWORD dwNumElements[]= {1 , 2};
		saData.Create(VT_VARIANT,2,dwNumElements);

		VARIANT *pVar;
		saData.AccessData((void**)&pVar);
		pVar[1].vt = VT_R8;
		pVar[1].dblVal = 0;
		pVar[0].vt = VT_BSTR;
		pVar[0].bstrVal = CString().AllocSysString();
		saData.UnaccessData();

		m_pChart->SetData(saData);
		m_pChart->SetSeriesAnnotation(m_nNextGroup,"1");
		m_pChart->ShowLegend(FALSE);
		m_nNextGroup++;
	}	
	else
		m_pChart->ShowLegend(TRUE);

	m_pListHeaders = pHeaderList;
	m_dwTestStartTime = dwTestStartTime;

    return true;
}

void CSNMPGraph::GetMenuItems(MenuItemArray& arMenuItems)
{
    arMenuItems.RemoveAll();

    MenuItems MI;

    MI.strItem = "Filter";
    MI.uiCommandID = enMenuItemFilter;
    arMenuItems.Add(MI);
}

void CSNMPGraph::OnCommand(UINT uiCommandID,CWnd *pParent)
{
    switch(uiCommandID)
    {
    case enMenuItemFilter:
	{
		ShowFilter();
	}
	break;
	}
}

void CSNMPGraph::ShowFilter()
{
	CFilterDlg dlg(m_pListHeaders);

	AfxSetResourceHandle(m_hInstance);
	int nResult = dlg.DoModal();
	AfxSetResourceHandle(AfxGetInstanceHandle());
	
	if(nResult ==IDOK)
	{
		//reload data
		while(m_nNextGroup>1)
			RemoveTestRun(m_nNextGroup-1);
			
		AddTestRun(m_pListHeaders,m_dwTestStartTime);
    }
}

CString CGraphTypeBase::GetElapsedTimeString(DWORD dwElapsedTime,DWORD dwTotalDuration)
{
	CString str;
	if(dwTotalDuration < 120)	//Less than 2 mins so show in secs
	{
		str.Format("%d", dwElapsedTime);
	}
	else 
	if(dwTotalDuration < 7200)	//Less than 2 hours so show in mins
	{
		str.Format("%d:%02d",dwElapsedTime/60,dwElapsedTime%60);
	}
	else
	{
		int nHours = dwElapsedTime/3600;
		int nMins =	(dwElapsedTime%3600) / 60;
		int nSecs = dwElapsedTime % 60;
		str.Format("%d:%02d:%02d",nHours,nMins,nSecs);		//show in hours
	}
	return str;			
}