/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// SNMPGraphs.cpp: implementation of the CSNMPGraphs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNMPGraphs.h"
#include "GraphTypes.h"
#include "GraphWnd.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNMPGraphs::CSNMPGraphs():
	m_strCustomSNMP("Custom SNMP")
{
}

CSNMPGraphs::~CSNMPGraphs()
{
	POSITION pos = m_listSNMPData.GetHeadPosition();
	while(pos)
		delete m_listSNMPData.GetNext(pos);
	m_listSNMPData.RemoveAll();
}

void CSNMPGraphs::LoadRun(const CString& strRunFolder)
{
	CString strRunDir = strRunFolder;
	strRunDir.TrimRight("\\");
	if(strRunDir == m_strLoadedRun)
		return;

	m_strLoadedRun = strRunDir;
	LoadStats();

	int nSlashPos = m_strLoadedRun.ReverseFind('\\');
	CString strRunName(m_strLoadedRun.Mid(nSlashPos+1));
	
	int nDay,nMonth,nYear,nHour,nMinute,nSecond;
	char *pszToken = strtok(strRunName.GetBuffer(0),"-");
	if(pszToken)
		nDay = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMonth = atoi(pszToken);

	if(pszToken = strtok(NULL," "))
		nYear = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nHour = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMinute = atoi(pszToken);

	if(pszToken = strtok(NULL,"."))
		nSecond = atoi(pszToken);

	CTime T(nYear,nMonth,nDay,nHour,nMinute,nSecond);
	m_dwTestStartTime = T.GetTime();
}

void CSNMPGraphs::GetAvailableGraphs(GraphInfoArray& arGraphs)
{
	arGraphs.RemoveAll();

	GraphInfo GI;
	GI.bFolder = false;
	if(m_listSNMPData.GetCount())
	{
		GI.strName = m_strCustomSNMP;
		arGraphs.Add(GI);
	}
}

CWnd* CSNMPGraphs::DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_CHILD|WS_VISIBLE|WS_BORDER,rcBounds,pParent,0);
	
	CSNMPGraph *pGraph = NULL;
	if(strName == m_strCustomSNMP)
		pGraph = new CSNMPGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;

	if(!pGraph || !pGraph->AddTestRun(&m_listSNMPData,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();
		
	return pGraphWnd;
}

void CSNMPGraphs::LoadStats()
{
	POSITION pos = m_listSNMPData.GetHeadPosition();
	while(pos)
		delete m_listSNMPData.GetNext(pos);
	m_listSNMPData.RemoveAll();

	CString strHeaderPath(m_strLoadedRun + "\\SNMPHeader.txt");
	CString strStatsPath(m_strLoadedRun + "\\SNMPStat.txt");
    
    try
    {
        CStdioFile SNMPFile(strHeaderPath,CFile::modeRead);

		CString strHeader;
		while(SNMPFile.ReadString(strHeader))
		{
			SNMPHeader *pHeader = new SNMPHeader;
			char *pszToken = strtok(strHeader.GetBuffer(0),",");
			if(!pszToken)
				continue;
			pHeader->dwIP=atol(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			pHeader->nIndex = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			pHeader->strQuery = pszToken;

			m_listSNMPData.AddTail(pHeader);
		}

		SNMPFile.Close();

        SNMPFile.Open(strStatsPath,CFile::modeRead);

		CString strStat;
		while(SNMPFile.ReadString(strHeader))
		{
			SNMPStat stat;
			char *pszToken = strtok(strHeader.GetBuffer(0),",");
			if(!pszToken)
				continue;
			stat.dwIP = atol(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.nIndex = atoi(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwStartTime = atoi(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwEndTime = atoi(pszToken); 

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dValue = atoi(pszToken);

			//Find header for this stat
			POSITION pos = m_listSNMPData.GetHeadPosition();
			while(pos)
			{
				SNMPHeader *pHeader = m_listSNMPData.GetNext(pos);
				if((pHeader->dwIP == stat.dwIP) && (pHeader->nIndex == stat.nIndex))
				{
					pHeader->listStats.AddTail(stat);
					pos = NULL;
				}
			}
		}
	}
	catch(CFileException *e)
	{
		e->Delete();
	}
	catch(...)
	{
	}
}

