/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include <afxdllx.h>
#include "PluginTypes.h"
#include "resource.h"       // main symbols
#include "SNMPGraphs.h"
#include "GraphWnd.h"
#include "GraphTypes.h"
#include "OstaDefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static AFX_EXTENSION_MODULE CSNMPResultsDLL = { NULL, NULL };

CSNMPGraphs g_SNMPGraphs;
HINSTANCE m_hInstance = NULL;
extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	m_hInstance = hInstance;
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CSNMPResultsDLL.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CSNMPResultsDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.

    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CSNMPResultsDLL.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CSNMPResultsDLL);

    }

    return 1;   // ok
}

extern "C" AFX_EXT_API
BOOL OpenSTAVerify()
{
	return true;
}

extern "C" AFX_EXT_API
void GetTitle(CString& strTitle)
{
	strTitle = "SNMP Results Plugin";
}

extern "C" AFX_EXT_API
int GetType()
{
	return TEST_RESULTS_PLUGIN;
}

extern "C" AFX_EXT_API
void GetPackage(CString& strPackage)
{
	strPackage = "SNMP";
}

extern "C" AFX_EXT_API
BOOL IsSupportedExtension(CString strExt)
{
    return false;
}

extern "C" AFX_EXT_API
void LoadRun(const CString& strRunFolder)
{
	g_SNMPGraphs.LoadRun(strRunFolder);	
}

extern "C" AFX_EXT_API
void GetAvailableGraphs(GraphInfoArray& arGraphs)
{
	g_SNMPGraphs.GetAvailableGraphs(arGraphs);
}

extern "C" AFX_EXT_API
CWnd* DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds)
{
	return g_SNMPGraphs.DisplayGraph(strName,pParent,rcBounds);
}

extern "C" AFX_EXT_API
BOOL GetDefaultOperationName(CWnd *pWnd,CString& strName)
{
	strName = "Filter";
	if(pWnd->IsKindOf(RUNTIME_CLASS(CGraphWnd)))
	{
		CGraphWnd *pGraphWnd = (CGraphWnd *)pWnd;
		if(pGraphWnd->m_pGraph)
			return pGraphWnd->m_pGraph->DoesGraphHaveFilter();
	}

	return false;
}

extern "C" AFX_EXT_API
void StartDefaultOperation(CWnd *pWnd)
{
	if(pWnd->IsKindOf(RUNTIME_CLASS(CGraphWnd)))
	{
		CGraphWnd *pGraphWnd = (CGraphWnd *)pWnd;
		if(pGraphWnd->m_pGraph)
			pGraphWnd->m_pGraph->ShowFilter();
	}
}

extern "C" AFX_EXT_API
void PerformMenuCommand(const CString& strRun,const CString& strGraph,const CString& strCommand)
{
	OutputDebugString("SNMP PMC " + strGraph + "," + strCommand + "\n");
}